function [v,errmsg]=excelclippaste
% Pastes a numeric matrix from MS Excel to Matlab.
%
% [v,errmsg]=excelclippaste
%
% The matrix must be numeric and dense (no holes).
%
% errmsg is empty upon return if the paste was
% successful.
%
% excelclippaste only works on the Windows platform
%
% See also: excelclipcopy, clipcopy, clippaste

% Lars Gregersen
% Copyright, 1999
%
% 1999.11.03, version 1.0

v = [];
errmsg =[];

if ~clipistext
  errmsg = 'The clipboard does not contain text';
  return
end

% find number of cols
s = clippaste;
newlinepos = find(s==13);

if ~any(newlinepos)
  errmsg = 'Clipboard has wrong format';
  warning(['excelclippaste: ' errmsg])
  return
end

ncols = length( find(s(1:newlinepos(1)-1)==9) )+1;

% replace commas with decimal points
fc = find(s==',');
if any(fc)
  s(fc) = '.';
end

% create pattern
pattern = '%f';
for i=1:ncols-1
  pattern = [pattern 9 '%f'];
end
pattern = [pattern 13 10];

% convert to numerics
[v,count,errmsg,nextindex] = sscanf(s, pattern);
if any(errmsg)
  warning(['excelclippaste(sscanf): ' errmsg])
  v = [];
  return
end

if floor(prod(size(v))/ncols)~=prod(size(v))/ncols
  errmsg = 'The number of elements does not match the template'
  warning(['excelclippaste: ' errmsg])
  return
end

v = reshape(v, ncols, prod(size(v))/ncols)';
