function [out1, out2, out3, out4]=process(action, in1,in2,in3);
% process 	- data analysis program
% UNITRET MODULE
%--------------------------------------------------------------------------------
% Written by A.Beylin and I.Kagan						
% (C) 1996-2002 
% *******************************						 
% * Last modification: 31.07.02 *						 
% *******************************
%
% Requires unispectrum.m, unistem.m and zoom1.m (Unitret) subroutines
%--------------------------------------------------------------------------------

global TRET_DAT NAME figcounter v FIGCOLOR SF RF FBH PFR;
% SF - spatial frequencies for tuning curve
% RF - response levels for tuning curve

% FBH - First Border Handler: When in "Index Mode on" (index of INC/DEC overlapping)
% used to decide where to prescribe the border value - to x1 or x2

% Set figure background color
if strcmp(v(1),'5'),
        FIGCOLOR='w';
else
        FIGCOLOR='w';
end

DefValues = get(0,'UserData');

if nargin < 2,   % processing will be occured with the current screen
        % data
        Spk = unitret('get_markeddata');
        
        np=size(Spk,1);
        
        Spk0 = [];
        Spk1 = [];
        
        for i=1:np,
                if Spk(i,5) == 0,
                        Spk0 = [Spk0;Spk(i,:)];
                else
                        Spk1 = [Spk1;Spk(i,:)];
                end;
        end;
        for i=0:1,
                s(i+1,:) = ['CurSpk = []; CurSpk = Spk', num2str(i),';'];
        end; 
end;


%----------------------------------------------------------------

if strcmp(action,'average'),
        global AV_1 AV_2
	if nargin==4, % concatenated
		notconc = 0;
        	tit = str2mat('Aver. resp','Conc. resp');
	else
		notconc = 1;
	        tit = str2mat('Aver. ''even'' resp','Aver. ''odd'' resp');  
	end
        for i=1:2,
                s(i,:) = ['CurSpk = []; CurSpk =in',num2str(i), ';'];
        end;
        
        if DefValues(123) == 1,     % Histogram in #spikes
                Ylabel = '#spikes';
        elseif  DefValues(123) == 2 % Histogram in spikes/sec
                Ylabel = 'spikes/sec';
                % Calculation of spike rate is as in Abeles 1982:
                % dt - bin, n - count in bin, N - #segm.,
                % rate = n/(N*dt)*1000
	end

	
	
        if DefValues(35) | ~notconc,			% Separate Drawing
                
                for m=1:2,	% MODALITY COUNTER
                        eval(s(m,:));
                        if (sum(size(CurSpk))>2), % in1 or in2 == 0
                                [vect,histog]=process('average_spk_calculation',CurSpk);

				if DefValues(182), % auto RF mapping
					% Calculate significance for each bin
					baseline = DefValues(54)/1000*DefValues(20);
					for k=1:size(histog,2),
						[H(k),sign(k)]=ttest(histog(:,k),baseline,DefValues(183),1);
					end
				end
				
                                [rate,ifr] = process('mean_rate_calculation',CurSpk);
                                if notconc
                                	velocity = process('average_vel_calculation',CurSpk); 
				else
					velocity = 0;
                                end
                                xmin = 0;
                                xmax = max(CurSpk(:,4) - CurSpk(:,3));
                                axpos = [.075+0.5*(~(m-1)) .12 .4 .1];
                                set(axes,'Position',axpos,'DrawMode','fast','TickDir','out','XLimMode',...
                                        'Manual','XLim',[xmin xmax],'UserData',velocity);
                                h=get(gca,'title'); 
                                
                                
                                if length(vect)>0
                                        % bins=(xmin:DefValues(20):xmax);
                                        bins=(xmin+DefValues(20)/2:DefValues(20):xmax-DefValues(20)/2);
                                        [N,SpikeHist]=hist(vect,bins);
                                        if DefValues(123)==2,
                                                BinNormVector = process('bin_norm_calculation',CurSpk,bins); 
                                                %N = N./BinNormVector*1000/xmax;
                                                N = N./(BinNormVector*DefValues(20)/1000);
						if DefValues(182),
							histog = histog./(DefValues(20)/1000); % (repmat(BinNormVector,size(histog,1),1)
						end
                                        end 
                                        if DefValues(125), % Normalize PSTH by pre-trigger FR
                                                Ylabel = 'Norm. Rate';
                                                if DefValues(126), 
                                                        N = N/DefValues(126);	
                                                else
                                                        PreTrigFR = mean(N(find(SpikeHist<DefValues(124))));
                                                        if PreTrigFR>0 
                                                                N = N/PreTrigFR;
                                                        end
                                                end
                                        end
                                        if DefValues(86)==1,
						if DefValues(196), % Smooth PSTH
							N = unitret('smooth',N,DefValues(20),5);
							set(line,'Xdata',SpikeHist,'Ydata',N);
						else
							[xx,yy] = bar(SpikeHist,N,1);
							set(line,'Xdata',xx,'YData',yy,'Color','b');
						end

						if DefValues(182) % auto AR mapping
							hold on;
							signind = find(H==1);
							nosignind = find(H==0);
							% plot(SpikeHist(signind),N(signind)+10,'r*');
							if DefValues(184)==2
								er = std(histog,2);
							elseif DefValues(184)==3
								er = std(histog,2)./sqrt(size(histog,1));
							end
							if DefValues(184)==1
								plot(SpikeHist(signind),N(signind)+10,'r*');
							else
								for i=1:length(H)
									if find(i==signind)
										line([SpikeHist(i) SpikeHist(i)],[N(i) N(i)+er(i)],'Color','r');
									else
										line([SpikeHist(i) SpikeHist(i)],[N(i) N(i)+er(i)],'Color','k');
									end
								end
							end

							% Estimate AR width as distance between first and last significant bin
							ResponseTime = (max(signind) - min(signind)+1)*DefValues(20);
							ARwidth = ResponseTime*velocity/1000;
							
							text(0.1,-0.5,['AR ',int2str(ARwidth),' minarc'],'Units','normalized');
						end

					elseif DefValues(86)==2	% mean of smoothed spikes
						spike_train = process('make_train',CurSpk);
						nrows = size(spike_train,1);
						for nrow = 1:nrows,
							smoothed_spike_train(nrow,:)=unitret('smooth',spike_train(nrow,:),1,3);
						end
						BinNormVector = process('bin_norm_calculation',CurSpk,[0.5:1:length(smoothed_spike_train)-0.5]);
						BinNormVector = BinNormVector/nrows;
						sd = std(smoothed_spike_train,1);
						if length(sd)==1,
							sd = 0;
						else
							sd = sd./(BinNormVector*1/1000);
						end
						se = sd/sqrt(size(smoothed_spike_train,1)); % SEM
						timeaxis = [1:length(smoothed_spike_train)];
						smoothed_spike_train = mean(smoothed_spike_train,1);

						smoothed_spike_train = smoothed_spike_train ./(BinNormVector*1/1000);
		
						if sum(sd)>0,
							switch DefValues(184)

								case 1
								case 2
									patch([timeaxis fliplr(timeaxis)],[smoothed_spike_train+sd fliplr(smoothed_spike_train-sd)],'r','EdgeColor','none','FaceColor',[0.8 0.85 0.8])
								case 3
									patch([timeaxis fliplr(timeaxis)],[smoothed_spike_train+se fliplr(smoothed_spike_train-se)],'r','EdgeColor','none','FaceColor',[0.8 0.85 0.8])
							end
						end
						line(timeaxis, smoothed_spike_train, 'Color','k','LineWidth',2);
							
					elseif DefValues(86)==3 % IFR
						IFR = mean(ifr,1);
						IFR = unitret('smooth',IFR);
						xaxis = [xmin:DefValues(172):xmax];
						sd = std(ifr,1);
						if length(sd)==1,
							sd = 0;
						end
						se = sd/sqrt(size(ifr,1)); % SEM
						%line(xaxis, IFR+sd,'Color',[0 0 0],'LineWidth',0.5);
						%line(xaxis, IFR-sd,'Color',[0 0 0],'LineWidth',0.5);
						switch DefValues(184)
							
							case 1
							case 2
								patch([xaxis fliplr(xaxis)],[IFR+sd fliplr(IFR-sd)],'r','EdgeColor','none','FaceColor',[0.8 0.85 0.8])
							case 3
								patch([xaxis fliplr(xaxis)],[IFR+se fliplr(IFR-se)],'r','EdgeColor','none','FaceColor',[0.8 0.85 0.8])
						end
												
						line(xaxis, IFR, 'Color',[ 0 0.2510 0.5020],'LineWidth',2);
					end
					
					meanFR = mean(N); % mean of binned histogram
                                        com = ['AV_',num2str(m), '= [SpikeHist; N];'];
                                        eval(com) ;               
                                        
                                        
                                        
                                        %set(line,'Xdata',xx,'YData',yy/(size(CurSpk,1)*DefValues(20)*0.001),'Color','b');
                                        
                                        ylim=get(gca,'YLim');
                                        xlim=get(gca,'XLim');
                                        
                                        if DefValues(124), % PSTH Trigger 
                                                set(line,'Xdata',[DefValues(124) DefValues(124)],'YData',ylim,'Color','r');	
                                        end
                                        
                                        
                                        
                                        
                                        
                                        ylabel(Ylabel);
                                        
                                        if DefValues(41),          			%if autoscale   
                                                drawYlims = [0,ylim(2)];
                                        else,
                                                drawYlims = [0,DefValues(42)];		%manual scale
                                        end;
                                        
                                        if DefValues(43),          			%if autoscale   
                                                drawXlims = [0,xlim(2)];
                                        else,
                                                drawXlims = [DefValues(44),DefValues(45)];	%manual scale
                                        end;
                                        
                                        set(gca,'XLim',drawXlims,'YLim',drawYlims);
                                        
                                end  
                               
                                if isempty(h)
                                        h = text('horiz','center');
                                        set(gca,'title',h);
                                end;
                                str=sprintf('%s, %5.1f spks, %5.2f s/s, %5.2f s/s',deblank(tit(m,:)), length(vect)/size(CurSpk,1),rate,meanFR);
                                R(m) = length(vect)/size(CurSpk,1);
                                
                                set(h,'string',str, 'color',[0.5020 0 0.2510],'FontSize',9,'FontWeight','demi');
                                
                                
                                AccDrawMenu= get(TRET_DAT(7),'UserData');
                                AccAverageDraw = get(AccDrawMenu(2),'Checked');
                                if strcmp(AccAverageDraw,'on'),    % accumulation flag turned 'on'
                                        unitret('redraw_accumulation',sort(vect));
				end;


				if DefValues(190), % TI
				
					if DefValues(191) <= 0,
						base = DefValues(54);
					else
						base = mean(N(find( SpikeHist<=DefValues(191) )));
					end
					peaks = fliplr(sort( N(find(SpikeHist>DefValues(191) & SpikeHist<=DefValues(192))) ));
					peak = mean(peaks(1:3));
					tail = mean(N(find( SpikeHist>DefValues(192) & SpikeHist<=DefValues(193) )));
					% Calculate transiency index
					TI = (peak - tail)/abs((peak-base + tail-base) + eps);
					text(0.2,-0.5,['TI ',num2str(TI,2)],'Units','normalized');
				end
                                
                        end;

	
                        
                end;  
                
                % Direction Index
                % DI is defined as DI = 1 - Response in the nonpreferred dir./Response in the pref. dir.
                % Casanova et al. (1992) 
                if DefValues(85),
                        if (size(in1,2)>1) & (size(in2,2)>1)  
                                DI = 1 - min(R)/max(R);
				if R(1)>R(2)
					str=' even>odd';
				else
					str=' odd>even';
				end
                                text(1.0,-0.5,['DI ',num2str(DI,2),str],'Units','normalized');
                        end
                end
                
                if DefValues(155),		% Peak Firing Rate
                        meanPFR = mean(PFR);
                        histPFR = max(N);
                        xlabel(['mean PFR ',num2str(meanPFR),'  hist PFR ',num2str(histPFR)]);
                        unitret('grand_handle','mean	hist',[meanPFR histPFR]);
                end
                
        else	%%%%%%%%%%%%%%%%%%%%%%%%%%% draw them together
                
                Spk=[];
                
                for m=1:2,	% MODALITY COUNTER           
                        eval(s(m,:));
                        if  max(size(CurSpk)) ~= 1,		%nonzero CurSpk
                                Spk=[Spk;CurSpk];
                        end;  
                end;
                CurSpk=Spk;
                
                vect=process('average_spk_calculation',CurSpk);
                [rate,ifr] = process('mean_rate_calculation',CurSpk);
                
		if notconc
                	velocity = process('average_vel_calculation',CurSpk); 
		else
			velocity = 0;
		end
                
                xmin = 0;
                xmax = max(CurSpk(:,4) - CurSpk(:,3));
                axpos = [.3 .12 .4 .1];
                set(axes,'Position',axpos,'DrawMode','fast','TickDir','out','XLimMode',...
                        'Manual','XLim',[xmin xmax],'UserData',velocity);
                h=get(gca,'title'); 
                
                
                
                if length(vect)>0
                        %bins=(xmin:DefValues(20):xmax);
                        bins=(xmin+DefValues(20)/2:DefValues(20):xmax-DefValues(20)/2);
                        [N,SpikeHist]=hist(vect,bins);
                        
                        if DefValues(123)==2,
                                BinNormVector = process('bin_norm_calculation',CurSpk,bins); 
                                %N = N./BinNormVector*1000/xmax;
                                N = N./(BinNormVector*DefValues(20)/1000);
                        end 
                        if DefValues(125), % Normalize PSTH by pre-trigger FR
                                if DefValues(126), 
                                        Ylabel = 'Norm. Rate';
                                        N = N/DefValues(126);
                                else 
                                        PreTrigFR =mean(N(find(SpikeHist<=DefValues(124))));
                                        if PreTrigFR>0 
                                                N = N/PreTrigFR;
                                        end
                                end
			end

			if DefValues(86)==1,
				if DefValues(196), % Smooth PSTH
					N = unitret('smooth',N,DefValues(20),5);
					set(line,'Xdata',SpikeHist,'Ydata',N);
				else
					[xx,yy] = bar(SpikeHist,N,1);
					set(line,'Xdata',xx,'YData',yy,'Color','b');
				end

				if DefValues(182) % auto AR mapping
					hold on;
					signind = find(H==1);
					nosignind = find(H==0);
					% plot(SpikeHist(signind),N(signind)+10,'r*');
					if DefValues(184)==2
						er = std(histog,2);
					elseif DefValues(184)==3
						er = std(histog,2)./sqrt(size(histog,1));
					end
					if DefValues(184)==1
						plot(SpikeHist(signind),N(signind)+10,'r*');
					else
						for i=1:length(H)
							if find(i==signind)
								line([SpikeHist(i) SpikeHist(i)],[N(i) N(i)+er(i)],'Color','r');
							else
								line([SpikeHist(i) SpikeHist(i)],[N(i) N(i)+er(i)],'Color','k');
							end
						end
					end

					% Estimate AR width as distance between first and last significant bin
					ResponseTime = (max(signind) - min(signind)+1)*DefValues(20);
					ARwidth = ResponseTime*velocity/1000;

					text(0.1,-0.5,['AR ',int2str(ARwidth),' minarc'],'Units','normalized');
				end

			elseif DefValues(86)==2	% mean of smoothed spikes
				spike_train = process('make_train',CurSpk);
				nrows = size(spike_train,1);
				for nrow = 1:nrows,
					smoothed_spike_train(nrow,:)=unitret('smooth',spike_train(nrow,:),1,3);
				end
				BinNormVector = process('bin_norm_calculation',CurSpk,[0.5:1:length(smoothed_spike_train)-0.5]);
				BinNormVector = BinNormVector/nrows;
				sd = std(smoothed_spike_train,1);
				if length(sd)==1,
					sd = 0;
				else
					sd = sd./(BinNormVector*1/1000);
				end
				se = sd/sqrt(size(smoothed_spike_train,1)); % SEM
				timeaxis = [1:length(smoothed_spike_train)];
				smoothed_spike_train = mean(smoothed_spike_train,1);

				smoothed_spike_train = smoothed_spike_train ./(BinNormVector*1/1000);

				if sum(sd)>0,
					switch DefValues(184)

						case 1
						case 2
							patch([timeaxis fliplr(timeaxis)],[smoothed_spike_train+sd fliplr(smoothed_spike_train-sd)],'r','EdgeColor','none','FaceColor',[0.8 0.85 0.8])
						case 3
							patch([timeaxis fliplr(timeaxis)],[smoothed_spike_train+se fliplr(smoothed_spike_train-se)],'r','EdgeColor','none','FaceColor',[0.8 0.85 0.8])
					end
				end
				line(timeaxis, smoothed_spike_train, 'Color','k','LineWidth',2);

			elseif DefValues(86)==3 % IFR
				IFR = mean(ifr,1);
				IFR = unitret('smooth',IFR);
				xaxis = [xmin:DefValues(172):xmax];
				sd = std(ifr,1);
				if length(sd)==1,
					sd = 0;
				end
				se = sd/sqrt(size(ifr,1)); % SEM
				%line(xaxis, IFR+sd,'Color',[0 0 0],'LineWidth',0.5);
				%line(xaxis, IFR-sd,'Color',[0 0 0],'LineWidth',0.5);
				switch DefValues(184)

					case 1
					case 2
						patch([xaxis fliplr(xaxis)],[IFR+sd fliplr(IFR-sd)],'r','EdgeColor','none','FaceColor',[0.8 0.85 0.8])
					case 3
						patch([xaxis fliplr(xaxis)],[IFR+se fliplr(IFR-se)],'r','EdgeColor','none','FaceColor',[0.8 0.85 0.8])
				end

				line(xaxis, IFR, 'Color',[ 0 0.2510 0.5020],'LineWidth',2);
			end
			
                        meanFR = mean(N); % mean of binned histogram
                        ylim=get(gca,'YLim');
                        xlim=get(gca,'XLim');
                        
                        
                        
                        if DefValues(124), % PSTH Trigger 
                                set(line,'Xdata',[DefValues(124) DefValues(124)],'YData',ylim,'Color','r');	
                        end
                        
                        ylabel(Ylabel);  
                        if DefValues(41),          			%if autoscale   
                                drawYlims = [0,ylim(2)];
                        else,
                                drawYlims = [0,DefValues(42)];		%manual scale
                        end;
                        
                        if DefValues(43),          			%if autoscale   
                                drawXlims = [0,xlim(2)];
                        else,
                                drawXlims = [DefValues(44),DefValues(45)];	%manual scale
                        end;
                        
                        set(gca,'XLim',drawXlims,'YLim',drawYlims);
                        
                        if isempty(h)
                                h = text('horiz','center');
                                set(gca,'title',h);
                        end;
                        str=sprintf('%s, %5.1f spks, %5.2f s/s, %5.2f s/s','Both', length(vect)/size(CurSpk,1),rate, meanFR);
                        set(h,'string',str, 'color',[0.5020 0 0.2510],'FontSize',9,'FontWeight','demi');

			if DefValues(190), % TI
				
				if DefValues(191) <= 0,
					base = DefValues(54);
				else
					base = mean(N(find( SpikeHist<=DefValues(191) )));
				end
	                        peaks = fliplr(sort( N(find(SpikeHist>DefValues(191) & SpikeHist<=DefValues(192))) ));
	                        peak = mean(peaks(1:3));
	                        tail = mean(N(find( SpikeHist>DefValues(192) & SpikeHist<=DefValues(193) )));
	                        % Calculate transiency index
	                        TI = (peak - tail)/abs((peak-base + tail-base) + eps);
				text(0.2,-0.5,['TI ',num2str(TI,2)],'Units','normalized');
			end
                        
                        
                end  
                
                
                AccDrawMenu= get(TRET_DAT(7),'UserData');
                AccAverageDraw = get(AccDrawMenu(2),'Checked');
                if strcmp(AccAverageDraw,'on'),    % accumulation flag turned 'on'
                        unitret('redraw_accumulation',sort(vect));
                end;
                
	end
	DefValues(173)= velocity;
	set(0,'Userdata',DefValues);
        %----------------------------------------------------------------
        
elseif(strcmp(action,'average_norm')),
        % for batch spike files presentation
        
        global AV_1 AV_2 NORMIR SACCADES_STRUCTURE
        tit = str2mat('Aver. ''even'' resp','Aver. ''odd'' resp'); 
        for i=1:2,
                s(i,:) = ['CurSpk = []; CurSpk =in',num2str(i), ';'];
        end;
        
        if DefValues(123) == 1,     % Histogram in #spikes
                Ylabel = '#spikes';
        elseif  DefValues(123) == 2 % Histogram in spikes/sec
                Ylabel = 'spikes/sec';
                % Calculation of spike rate is as in Abeles 1982:
                % dt - bin, n - count in bin, N - #segm.,
                % rate = n/(N*dt)*1000
        end
        
        
        if DefValues(35),			%Separate Drawing
                
                for m=1:2,	% MODALITY COUNTER
                        eval(s(m,:));
                        if (sum(size(CurSpk))>2),			% in1 or in2 == 0
                                NSegm = size(CurSpk,1);
                                process('average_rate_per_segment',CurSpk);
                                vect=process('average_spk_calculation',CurSpk);
                                [rate,ifr] = process('mean_rate_calculation',CurSpk);
                                
                                velocity = process('average_vel_calculation',CurSpk); 
                                
                                xmin = 0;
                                xmax = max(CurSpk(:,4) - CurSpk(:,3));
                                % axpos = [.075+0.5*(~(m-1)) .12 .4 .1]; set(axes,'Position',axpos)
                                % axes = gca;
                                set(gca,'DrawMode','fast','TickDir','out','XLimMode',...
                                        'Manual','XLim',[xmin xmax],'UserData',velocity);
                                h=get(gca,'title'); 
                                
                                
                                if length(vect)>0
                                        bins=(xmin+DefValues(20)/2:DefValues(20):xmax-DefValues(20)/2);
                                        [N,SpikeHist]=hist(vect,bins);
                                        
                                        if DefValues(123)==2,
                                                BinNormVector = process('bin_norm_calculation',CurSpk,bins); 
                                                %N = N./BinNormVector*1000/xmax;
                                                N = N./(BinNormVector*DefValues(20)/1000);
                                        end 
                                        if DefValues(125), % Normalize PSTH by pre-trigger FR
                                                Ylabel = 'Norm. Rate';
                                                if DefValues(126), 
                                                        N = N/DefValues(126);	
                                                else
                                                        PreTrigFR = mean(N(find(SpikeHist<DefValues(124))));
                                                        if PreTrigFR>0 
                                                                N = N/PreTrigFR;
                                                        end
                                                end
                                        end
                                        
                                        [xx,yy] = bar(SpikeHist,N,1);
                                        
                                        com = ['AV_',num2str(m), '= [SpikeHist; N];'];               
                                        eval(com) ;               
                                        set(line,'Xdata',xx,'YData',yy,'Color','b');
                                        
                                        
                                        %set(line,'Xdata',xx,'YData',yy/(size(CurSpk,1)*DefValues(20)*0.001),'Color','b');
                                        
                                        ylim=get(gca,'YLim');
                                        xlim=get(gca,'XLim');
                                        
                                        if DefValues(124), % PSTH Trigger 
                                                set(line,'Xdata',[DefValues(124) DefValues(124)],'YData',ylim,'Color','k');	
                                        end
                                        
                                        
                                        
                                        if DefValues(41),          			%if autoscale   
                                                drawYlims = [0,ylim(2)];
                                        else,
                                                drawYlims = [0,DefValues(42)];		%manual scale
                                        end;
                                        
                                        if DefValues(43),          			%if autoscale   
                                                drawXlims = [0,xlim(2)];
                                        else,
                                                drawXlims = [DefValues(44),DefValues(45)];	%manual scale
                                        end;
                                        
                                        set(gca,'XLim',drawXlims,'YLim',drawYlims);
                                        
                                end  
                                
                                
                                if isempty(h)
                                        h = text('horiz','center');
                                        set(gca,'title',h);
                                end;
                                str=sprintf('%s, %5.1f spks, %5.2f s/s, %5.2 s/s ',deblank(tit(m,:)), length(vect)/size(CurSpk,1),rate,meanFR);
                                R(m) = length(vect)/size(CurSpk,1);
                                
                                set(h,'string',str, 'color',[0.5020 0 0.2510],'FontSize',9,'FontWeight','demi');
                                
                                
                                % AccDrawMenu= get(TRET_DAT(7),'UserData');
                                % AccAverageDraw = get(AccDrawMenu(2),'Checked');
                                % if strcmp(AccAverageDraw,'on'),    % accumulation flag turned 'on'
                                %    unitret('redraw_accumulation',sort(vect));
                                % end;
                                
                        end;
                        
                end;  
                
                % Direction Index
                % DI is defined as DI = 1 - Response in the nonpreferred dir./Response in the pref. dir.
                % Casanova et al. (1992) 
                if DefValues(85),
                        if (size(in1,2)>1) & (size(in2,2)>1)  
                                DI = 1 - min(R)/max(R);
				if R(1)>R(2)
					str=' even>odd';
				else
					str=' odd>even';
				end
                                text(1.05,-0.5,['DI ',num2str(DI,2),str],'Units','normalized');
                        end
                end  
                
        else	%%%%%%%%%%%%%%%%%%%%%%%%%%% draw them together
                
                Spk=[];
                
                for m=1:2,	% MODALITY COUNTER           
                        eval(s(m,:));
                        if  max(size(CurSpk)) ~= 1,		%nonzero CurSpk
                                Spk=[Spk;CurSpk];
                        end;  
                end;
                CurSpk=Spk;
                NSegm = size(CurSpk,1);
                
                process('average_rate_per_segment',CurSpk);
                vect=process('average_spk_calculation',CurSpk);
                [rate,ifr] = process('mean_rate_calculation',CurSpk);
                
                velocity = process('average_vel_calculation',CurSpk); 
                
                xmin = 0;
                xmax = max(CurSpk(:,4) - CurSpk(:,3));
                
                %  axpos = [.3 .12 .4 .1];
                %  axes = gca; set(axes,'Position',axpos)
                set(gca,'DrawMode','fast','TickDir','out','XLimMode',...
                        'Manual','XLim',[xmin xmax],'UserData',velocity);
                h=get(gca,'title'); 
                
                
                
                if length(vect)>0
                        %%% bins=(xmin:DefValues(20):xmax);
                        bins=(xmin+DefValues(20)/2:DefValues(20):xmax-DefValues(20)/2);
                        [N,SpikeHist]=hist(vect,bins);
                        
                        if DefValues(123)==2,
                                BinNormVector = process('bin_norm_calculation',CurSpk,bins); 
                                %N = N./BinNormVector*1000/xmax;
                                N = N./(BinNormVector*DefValues(20)/1000);
                        end 
                        if DefValues(125), % Normalize PSTH by pre-trigger FR
                                if DefValues(126), 
                                        Ylabel = 'Norm. Rate';
                                        N = N/DefValues(126);
                                else 
                                        PreTrigFR = mean(N(find(SpikeHist<=DefValues(124))));
                                        if PreTrigFR>0 
                                                N = N/PreTrigFR;
                                        end
                                end
                        end
                        
                        [xx,yy] = bar(SpikeHist,N,1);
                        %             com = ['AV_',num2str(0), '= [SpikeHist; N];'];               
                        %             eval(com) ;               
                        %             set(line,'Xdata',xx,'YData',yy/size(CurSpk,1),'Color','b');
                        %             set(line,'Xdata',xx,'YData',yy/(size(CurSpk,1)*DefValues(20)*0.001),'Color','b');
                        set(line,'Xdata',xx,'YData',yy,'Color','b');
                        
                        ylim=get(gca,'YLim');
                        xlim=get(gca,'XLim');
                        
                        
                        
                        if DefValues(124), % PSTH Trigger 
                                set(line,'Xdata',[DefValues(124) DefValues(124)],'YData',ylim,'Color','k');	
                        end
                        
                        
                        if DefValues(41),          			%if autoscale   
                                drawYlims = [0,ylim(2)];
                        else,
                                drawYlims = [0,DefValues(42)];		%manual scale
                        end;
                        
                        if DefValues(43),          			%if autoscale   
                                drawXlims = [0,xlim(2)];
                        else,
                                drawXlims = [DefValues(44),DefValues(45)];	%manual scale
                        end;
                        
                        set(gca,'XLim',drawXlims,'YLim',drawYlims);
                        
                        if isempty(h)
                                h = text('horiz','center');
                                set(gca,'title',h);
                        end;
                        str=sprintf('%s, %5.1f spks, %5.2f spk/s','On and Off', length(vect)/size(CurSpk,1),rate);
                        set(h,'string',str, 'color',[0.5020 0 0.2510],'FontSize',9,'FontWeight','demi');
                        
                end  
                
                
                % AccDrawMenu= get(TRET_DAT(7),'UserData');
                % AccAverageDraw = get(AccDrawMenu(2),'Checked');
                % if strcmp(AccAverageDraw,'on'),    % accumulation flag turned 'on'
                %      unitret('redraw_accumulation',sort(vect));
                % end;
                
                
        end % of average_norm
        
        if isempty(N),
                bins=(xmin+DefValues(20)/2:DefValues(20):xmax-DefValues(20)/2);
                N = zeros(1,length(bins));
                SpikeHist = bins;
        end
        
        % Fix first and last bins
        % N(1) = 2*N(1);
        % N(length(N)) = 2*N(length(N));
        
        % Zero first and last bins
        % N(1) = 0;
        % N(length(N)) = 0;
        
        if DefValues(136) & 0, % Classification Coefficients
                flashes = 1; saccades = ~flashes;
                
                if saccades, % Classification of saccades-triggered activity (see sacind.m)
                        % std values for sort_s-2b+97_new_ind2.lst
                        % stdv = [0	1	1.2	0	0	2.6	0.5	0	14	4	2	2	0.7	0	0	12	4.27	21	2	1.4	2	23	8	1.71	0.6	0	21	0	0	11	0.6	2.6	0	0	0	0	0	21	0	1	0	0	1.5	28	0	38	0	1	8	11	13.76	0.5	0	3	3	15	0	0	1.66	0	0];
                        % std values for sort_s-2std+97_new_ind2.lst
                        % stdv  =  [0	1	2.6	1.2	0	0	0.5	0.7	23	0	14	2	2	4.27	4	0	0	21	2	12	8	1.71	1.4	2	21	0	21	0	0	0	11	0.6	0.6	2.6	0	0	0	0	0	0	38	0	1	0	28	0	8	1	1.5	11	13.76	0	0.5	3	3	15	0	0	1.66	0	0];
                        % std values for static u with flashes: 
                        % stdv =  [1	2.6	0	0	15	0	14	2	4.27	12	8	1.71	1.39	21	0	0	0	0.63	2.6	0	0	38	28	0	1	1.5	11	13.76	3	0	0];
                        
                        PeakWin = 250;
                        PeakWin = 120;
                        
                        base = mean(N(find(SpikeHist <= DefValues(124))));
                        % base = stdv(in3(2));
                        peaks = fliplr(sort( N(find(SpikeHist>DefValues(124)+40 & SpikeHist<=DefValues(124)+PeakWin)) ));
                        %peaks =  N(find(SpikeHist>DefValues(124)+40 & SpikeHist<=DefValues(124)+PeakWin));
                        %peakloc = find(peaks==max(peaks));
                        %peak = mean(peaks(1:3));
                        peak = mean(peaks);
                        tail = mean(N(find(SpikeHist > (DefValues(124)+PeakWin) ))); 
                        [ind1,ind2] = sacind(base, peak, tail);
                        
                        % text(10,100,[num2str(ind1),' ',num2str(ind2)]);
                        sac_struct.numb = in3(2);
                        sac_struct.name = NAME;
                        sac_struct.base = base;
                        sac_struct.peak = peak;
                        sac_struct.tail = tail;
                        sac_struct.ind1 = ind1;
                        sac_struct.ind2 = ind2;
                        sac_struct.nsegms = NSegm;
                        if isempty(SACCADES_STRUCTURE),
                                SACCADES_STRUCTURE = sac_struct;
                        else
                                SACCADES_STRUCTURE = [SACCADES_STRUCTURE sac_struct];
                        end
                        ind2_string = [sprintf('\n'),num2str(ind2,3),' std/base ',num2str(base)];
                        
                elseif flashes, % Flash response index
                        % std values for all_flash_for_stat_sort-2b.lst
                        % stdv = [1	0	0	2.6	0	14	2	12	0	21	1.39	15	8	0	0.63	0	0	0	2.6	0	0	1.5	28	38	0	1	11	0	3	0	0];
                        % std values for all_flash_for_stat_sort-2std.lst
                        % stdv = [1	2.6	0	0	15	0	14	2	4.27	12	8	1.71	1.39	21	0	0	0	0.63	2.6	0	0	20	28	0	1	1.5	11	13.76	3	0	0];
			% std for 1f cells
			% stdv = [0	0	18	14	0	0	0	28	0];
			% std for s cells
			stdv = [0	13	0	0	0	1.4	0	0];
			% std for d cells
			stdv = [0	1	0	3	0	1	21	5	0	8.5	0.5	0	0	0	0	0	28	1	0	3	0	0	0	0.5	0	0.01	32	27	0	0	0.5	7	2.5	1	0	21	2	3	12	11	23	0	0	0	0	2.6	0	10	0	6	17	17	14	32	8	0.6	1	8	5	0	0	1.5	7	21	38	18	0.5	0	0	0	0	2.6	4	1	0	0	5	1	1	2	2	2	12];

                        PeakWin = 60; % ms
                        base = stdv(in3(2));
                        % base = mean(N(find( SpikeHist<=40 )));
                        peaks = fliplr(sort( N(find(SpikeHist>0 & SpikeHist<=100)) ));
                        peak = mean(peaks(1:3));
                        % peak = mean(N(find( SpikeHist>50 & SpikeHist<=100 )));
                        tail = mean(N(find( SpikeHist>100 & SpikeHist<=150 ))); 
                        % Calculate transiency index
                        ind2 = (peak - tail) / (peak + tail -2*base);
                        ind1 = 0;
                        % text(10,100,[num2str(ind1),' ',num2str(ind2)]);
                        sac_struct.numb = in3(2);
                        sac_struct.name = NAME;
                        sac_struct.base = base;
                        sac_struct.peak = peak;
                        sac_struct.tail = tail;
                        sac_struct.ind1 = ind1;
                        sac_struct.ind2 = ind2;
                        sac_struct.nsegms = NSegm;
                        if isempty(SACCADES_STRUCTURE),
                                SACCADES_STRUCTURE = sac_struct;
                        else
                                SACCADES_STRUCTURE = [SACCADES_STRUCTURE sac_struct];
                        end
                        ind2_string = [sprintf('\n'),num2str(ind2,3),' std ',num2str(base)];
                end % if which classification scheme to apply
                
        else % No classification scheme
                ind2_string = '';
        end % of if Classification Coefficients
        
        if in3(1),    
                NORMIR = [NORMIR; N];
        end
        title([num2str(in3(2)),': ',NAME,' ',num2str(NSegm),' segm. ',ind2_string],'FontSize',7)  
        
        
        %----------------------------------------------------------------------------------------------
elseif(strcmp(action,'bin_norm_calculation'))
        % Returns vector for normalization of PSTH time bins according to the segment duration:
        % Example: if one segment is 100 ms, second 200 ms, bin 50 ms, then BinNormVector will be
        % [2 2 1 1]
        
        CurSpk = in1;             
        bins = in2;
        
        for k=1:length(bins), 
                BinNormVector(k) = length(find(CurSpk(:,4) - CurSpk(:,3)>=bins(k))); 
        end
        out1 = BinNormVector;
        %----------------------------------------------------------------------------------------------
elseif(strcmp(action,'average_rate_per_segment'))
        
        Spk = in1;
        TrNum = size(Spk,1);
        Trig = DefValues(124);
        t_preTrigFR = [];
        t_postTrigFR = [];
        t_FR = [];	        
        for(i=1:TrNum)
                SpikesNum = Spk(i,2);
                StartSpikeData = Spk(i,3);
                EndSpikeData = Spk(i,4);
                mode = Spk(i,5);
                SpikesData = Spk(i,6:SpikesNum+5);
                if Trig,
                        % preTrigFR = 1000*length( find(SpikesData < (StartSpikeData+Trig)) )/Trig;
                        % postTrigFR = 1000*length( find(SpikesData > (StartSpikeData+Trig)) )/(EndSpikeData-StartSpikeData-Trig);
                        preTrigFR = 1000*length( find(SpikesData > (StartSpikeData+250) & SpikesData < (StartSpikeData+300)) )/50;
                        postTrigFR = 1000*length( find(SpikesData > (StartSpikeData+300) & SpikesData < (StartSpikeData+500)) )/200;
                        
                        t_preTrigFR = [t_preTrigFR preTrigFR];
                        t_postTrigFR = [t_postTrigFR postTrigFR];
                else
                        % FR = 1000*SpikesNum /(EndSpikeData-StartSpikeData);
                        FR = 1000*length( find(SpikesData < (StartSpikeData+200)) )/200;
                        t_FR = [t_FR FR];
                        
                end  
                
        end;
        if Trig
                eval(['save ',NAME(1:length(NAME)-3),'st2 t_preTrigFR t_postTrigFR']);
        else
                eval(['save ',NAME(1:length(NAME)-3),'st1 t_FR']);
        end
        
        
        
        %----------------------------------------------------------------------------------------------
elseif(strcmp(action,'average_spk_calculation'))
	% return vector of SpikeArrTimes for all segments, "head to tail"
        CurSpk = in1;
        TrNum = size(CurSpk,1);
        vect = [];

	xmin = 0;
        xmax = max(CurSpk(:,4) - CurSpk(:,3));
	bins=(xmin+DefValues(20)/2:DefValues(20):xmax-DefValues(20)/2);
		
        for(i=1:TrNum)
                num = CurSpk(i,2);
                offset = CurSpk(i,3);
	
		histog(i,:) = hist(CurSpk(i,6:num+5)-offset,bins);
                vect = [vect CurSpk(i,6:num+5)-offset];
        end;
        
        out1 = vect;
	out2 = histog;
        %----------------------------------------------------------------------------------------------
elseif(strcmp(action,'mean_rate_calculation'))
        CurSpk = in1;
        TrNum = size(CurSpk,1);
	IFR =[];
	istep = DefValues(172);			% Interpolation step
        for(i=1:TrNum)
                num = CurSpk(i,2);
                offset = CurSpk(i,6);
                SpikeArrTime=CurSpk(i,6:num+5)-offset;		  		  
                len = (CurSpk(i,4)-CurSpk(i,3)); 
                if DefValues(36),
                        if num > 0,					% calc. of firing rate for stimuli-driven resp
                                mrate(i) = 1000*(num-1)/SpikeArrTime(num);	% as (Number of Spikes -1) / [(time of the last Spike) - (time of the first Spike)];
                        else
                                mrate(i)=0;
                        end;  						
                else
                        LowLim = 0;					% calc. of firing rate for spontaneous resp
                        UpLim = 0.75*CurSpk(i,4);		  		% (Number of spikes occured in 75% interval of fixation)/                                           
                        [ind] = ...					% (duration of 75% fix. interval);			
                                find(SpikeArrTime >=LowLim & SpikeArrTime <= UpLim)  ;
                        mrate(i) = 1000*length(ind)/UpLim;
		end;
		if DefValues(86)==3,	% IFR
			SpikeArrTime = SpikeArrTime + offset - CurSpk(i,3);
			ifr = [0 1./(diff([SpikeArrTime])/1000 )]; % in spk/seconds
			if (len - SpikeArrTime(num))>istep,
				% ifri = interp1([0 SpikeArrTime len],[0 ifr 0],[0:istep:len]);
				ifri = interp1([0 SpikeArrTime SpikeArrTime(num)+istep len],[0 ifr 0 0],[0:istep:len]);
				% ifri = interp1([SpikeArrTime len],[ifr 0],[0:istep:len]);
			elseif (num>0 & SpikeArrTime(1)>0)
				ifri = interp1([0 SpikeArrTime],[0 ifr],[0:istep:len]);
			else
				ifri = interp1([SpikeArrTime],[ifr],[0:istep:len]);
			end
			IFR = [IFR; ifri];
		end
        end;
	
        rate = mean(mrate);
        out1 = rate;
	out2 = IFR;
        
        %----------------------------------------------------------------------------------------------
elseif(strcmp(action,'average_vel_calculation'))
        CurSpk = in1;
        TrNum = size(CurSpk,1);
        for(i=1:TrNum)
                vel(i) = process('get_inform',CurSpk(i,:),'velocity');
	end;
        velocity=mean(vel);
        out1 = velocity;
        %----------------------------------------------------------
        
elseif(strcmp(action,'shift_maximum'))
        process('remember_origin',1);
        NewSpikeData = [];
        for m=1:2,
                eval(s(m,:));
                if (~isempty(CurSpk)) 			% if selected only 'on' or 'off' group
                        if (sum(size(CurSpk))>2),			% if only one element, derivative not wxists
                                vect=process('average_spk_calculation',CurSpk);
                                vect = sort(vect);
                                ncol = length(vect);
                                if(ncol>1),
                                        j=2;
                                        i=1;
                                        while j<=ncol
                                                if vect(i)==vect(j),
                                                        vect(j)=vect(i)+.1;
                                                else,
                                                        i=j;
                                                        j=j+1;
                                                end;
                                        end;
                                        [tmp,n]=min(diff(vect));
                                        AvMaxTime=vect(n);
                                else
                                        AvMaxTime=vect(1);
                                end;
                                % here vect is an averaged spikearrivetime of CurSpk-non-uniform,
                                % AM signal
                                
                                %---------- Processing of Spikes ---------------
                                nrow = size(CurSpk,1);
                                
                                for i=1:nrow,
                                        vect = [];
                                        
                                        % PFM --> non-monotonic AM
                                        if(CurSpk(i,2) >1)
                                                vect = diff(CurSpk(i,6:(CurSpk(i,2)+5)));
                                                [tmp,n]=min(vect);
                                                SpkMaxTime = CurSpk(i,6+n)-CurSpk(i,3);
                                        else
                                                SpkMaxTime = CurSpk(i,6)-CurSpk(i,3);
                                        end;
                                        
                                        % Shifting 
                                        shift=AvMaxTime-SpkMaxTime;
                                        SpikesNum = CurSpk(i,2);
                                        CurSpk(i,6:(SpikesNum+5)) = CurSpk(i,6:(SpikesNum+5)) + shift;
                                end;
                        end;
                        if isempty(NewSpikeData);
                                NewSpikeData = CurSpk;
                        else
                                NewSpikeData = [NewSpikeData;CurSpk];
                        end;
                end;
                unitret('set_markeddata',NewSpikeData);
                set(TRET_DAT(6),'Enable','on');
                
                unitret('redraw_spk',1);
        end;
        %----------------------------------------------------------------------------------------------
elseif(strcmp(action,'shift_mean'))
        
        process('remember_origin',1);
        NewSpikeData = [];
        
        if DefValues(35),
                for m=1:2,
                        eval(s(m,:));
                        if (sum(size(CurSpk))>2),			% in1 or in2 == 0
                                vect=process('average_spk_calculation',CurSpk);
                                AvMeanTime = sum(vect)/length(vect);
                                
                                
                                
                                %---------- Processing of Spikes ---------------
                                nrow = size(CurSpk,1);
                                
                                for i=1:nrow,
                                        SpikesNum = CurSpk(i,2);
                                        vect = [];
                                        
                                        vect = CurSpk(i,6:SpikesNum+5);
                                        ApMeanTime = (sum(vect)/SpikesNum)-CurSpk(i,3);
                                        shift = AvMeanTime-ApMeanTime;
                                        CurSpk(i,6:(SpikesNum+5)) = CurSpk(i,6:(SpikesNum+5)) + shift;
                                end;
                        end;
                        if isempty(NewSpikeData);
                                NewSpikeData = CurSpk;
                        else
                                NewSpikeData = [NewSpikeData;CurSpk];
                        end;
                end;
        else
                
                Spk=[];
                for m=1:2,	% MODALITY COUNTER           
                        eval(s(m,:));
                        Spk=[Spk;CurSpk];
                end;
                CurSpk=Spk;
                vect=process('average_spk_calculation',CurSpk);
                AvMeanTime = sum(vect)/length(vect);
                
                
                
                %---------- Processing of Spikes ---------------
                nrow = size(CurSpk,1);
                
                for i=1:nrow,
                        SpikesNum = CurSpk(i,2);
                        vect = [];
                        
                        vect = CurSpk(i,6:SpikesNum+5);
                        ApMeanTime = (sum(vect)/SpikesNum)-CurSpk(i,3);
                        shift = AvMeanTime-ApMeanTime;
                        CurSpk(i,6:(SpikesNum+5)) = CurSpk(i,6:(SpikesNum+5)) + shift;
                end;
                NewSpikeData=CurSpk;
                
        end;
        unitret('set_markeddata',NewSpikeData);
        set(TRET_DAT(6),'Enable','on');
        
        unitret('redraw_spk',1);
        
        %----------------------------------------------------------------------------------------------
        
elseif(strcmp(action,'align_first'))
        process('remember_origin',1);
        NewSpikeData = [];
        
        if DefValues(35),
                for m=1:2,
                        eval(s(m,:));
                        if (sum(size(CurSpk))>2),			% in1 or in2 == 0
                                FirstSpikes = CurSpk(:,6);
                                FirstDiffs   = FirstSpikes - CurSpk(:,3);
                                MinFirstDiff = min(FirstDiffs);
                                FirstFirstSpike = FirstSpikes(find(FirstDiffs == MinFirstDiff)); 	
                                
                                
                                %---------- Processing of Spikes ---------------
                                nrow = size(CurSpk,1);
                                
                                for i=1:nrow,
                                        SpikesNum = CurSpk(i,2);
                                        CurSpk(i,6:(SpikesNum+5)) = CurSpk(i,6:(SpikesNum+5)) - (FirstDiffs(i) - MinFirstDiff);
                                end;
                        end;
                        if isempty(NewSpikeData);
                                NewSpikeData = CurSpk;
                        else
                                NewSpikeData = [NewSpikeData;CurSpk];
                        end;
                end;
        else
                
                Spk=[];
                for m=1:2,	% MODALITY COUNTER           
                        eval(s(m,:));
                        Spk=[Spk;CurSpk];
                end;
                CurSpk=Spk;
                FirstSpikes = CurSpk(:,6);
                FirstDiffs   = FirstSpikes - CurSpk(:,3);
                MinFirstDiff = min(FirstDiffs);
                FirstFirstSpike = FirstSpikes(find(FirstDiffs == MinFirstDiff)); 	
                
                
                %---------- Processing of Spikes ---------------
                nrow = size(CurSpk,1);
                
                for i=1:nrow,
                        SpikesNum = CurSpk(i,2);
                        CurSpk(i,6:(SpikesNum+5)) = CurSpk(i,6:(SpikesNum+5)) - (FirstDiffs(i) - MinFirstDiff);
                end;
                
                NewSpikeData = CurSpk;
        end;
        unitret('set_markeddata',NewSpikeData);
        set(TRET_DAT(6),'Enable','on');
        
        unitret('redraw_spk',1);
        unitret('text2fig',['First Spike: ',num2str(MinFirstDiff),' ms'])
        %----------------------------------------------------------------------------------------------
        
elseif(strcmp(action,'restore'))
        GenuineSpikesData = get(TRET_DAT(6),'UserData');
        unitret('set_markeddata',GenuineSpikesData);
        set(TRET_DAT(6),'Enable','off');
        unitret('redraw_spk',1);
        %----------------------------------------------------------------------------------------------
        
elseif(strcmp(action,'remember_origin'))
        % here we check if this processing was first. If so, we remember the origin spike data 
        % in the userdata of 'restore' menu userdata buffer
        
        GenuineSpikedata=get(TRET_DAT(6),'UserData');
        if isempty(GenuineSpikedata),    % first processing
                GenuineSpikedata = unitret('get_markeddata');
                set(TRET_DAT(6),'UserData',GenuineSpikedata,'enable','on');
        end;
        
        %----------------------------------------------------------------------------------------------
        
elseif(strcmp(action,'accumulation'))
        co=get(gcf,'CurrentMenu');
        
        checked=get(co,'Checked');
        if (strcmp(checked,'off'))
                set(co,'Checked','on');
        else
                set(co,'Checked','off');
        end;
        unitret('redraw_spk',1);
        
        %----------------------------------------------
elseif(strcmp(action,'accumulation_pressed'))
        accum_line=in1;
        curax = get(accum_line,'Parent');
        set(TRET_DAT(2),'CurrentAxes',curax);
        ud=get(accum_line,'UserData');
        
        PressPointNumber=ud(1); 
        if PressPointNumber < 2,		% allow only 2 points for the linear regression (0,1)
                PresPnt=get(curax,'CurrentPoint');
                x=PresPnt(1,1); 
                y=PresPnt(1,2); 
                ylimit=get(curax,'YLim');         
                
                temp_line=line([x, x], ylimit,'linestyle',':','erasemode','xor');
                
                strverline = sprintf(' %17.13f',temp_line);
                straccumline = sprintf(' %17.13f',accum_line);
                
                set(temp_line,'ButtonDownFcn',['process(''accumulation_down'',',strverline,',',...
                                straccumline,');']);    
                PressPointNumber = PressPointNumber+1;
                
                if PressPointNumber == 1, % First border (for Index)
                        FBH=temp_line;
                end
                
                % as UserData of accumulation curve we transfer number of pressed points (max 2) and pointers to
                % vertical vernier lines. So, the maximal size of accumulation curve userdata is 3 elements
                
                ud(1) = PressPointNumber;
                ud(PressPointNumber+1) = temp_line;
                set(accum_line,'UserData', ud);
                
        end;
        
        
        %%%%%%% $$$ %%%%%%% for Index (Igor)
        if 0 % Index Mode on
                ud=get(0,'Userdata');
                ax=gca;
                pt=get(ax,'CurrentPoint');
                x=pt(1,1);
                if DefValues(68) % Inc
                        ind=64;
                else
                        ind=66;
                end
                if FBH==temp_line % first border
                        ud(ind)=x;
                else
                        ud(ind+1)=x;
                end    
                set(0,'Userdata',ud);      
        end  
        %%%%%%% $$$ %%%%%%%
        
        if PressPointNumber==2,
                process('accumulation_regression',accum_line,temp_line); 
        end;  
        if PressPointNumber==1,
                process('distance_calculation',accum_line); %%% ???
        end;  
        
        %---------------------------------------------------------------------
        
elseif strcmp(action,'accumulation_down'),
        strverline=sprintf(' %17.13f',in1);
        straccumline=sprintf(' %17.13f',in2);
        
        set(gcf,'WindowButtonMotionFcn',['process(''accumulation_move'',',strverline,');']);
        set(gcf,'WindowButtonUpFcn', ['process(''accumulation_up'',',strverline,',',...
                        straccumline,');']); 
        
        %---------------------------------------------------------------------         
elseif strcmp(action,'accumulation_move')
        ax=get(in1,'Parent');
        pt=get(ax,'CurrentPoint');
        x=pt(1,1);
        y=pt(1,2);
        set(in1,'Xdata',[x x]);
        
        
        %---------------------------------------------------------------------         
elseif strcmp(action,'accumulation_up')
        verline = in1;
        accumline=in2;
        
        
        ud= get(accumline,'UserData');
        PressPointNumber = ud(1);
        set(gcf,'WindowButtonMotionFcn','');
        set(gcf,'WindowButtonUpFcn', '');       
        if PressPointNumber==2,
                process('accumulation_regression',accumline,verline); 
        end;
        
        if PressPointNumber==1,
                process('distance_calculation',accumline,verline); 
        end;
        
        
        %---------------------------------------------------------------------         
elseif strcmp(action,'accumulation_regression')
        
        accumline=in1;
        verline  =in2;
        ax=get(accumline,'Parent');
        xl=get(ax,'XLim');
        yl=get(ax,'YLim');
        SpkInf = get(ax,'UserData');
        if length(SpkInf)==1,		% we work with averaged data
                velocity=SpkInf;
        else
                velocity = process('get_inform',SpkInf,'velocity');
        end;
        
        set(TRET_DAT(2),'CurrentAxes',ax);
        
        udl=get(accumline,'UserData');
        %udl(1) = # of vernier lines (=2);
        %udl(2:ud(1)+1), = pointer to these lines;
        %udl(ud(1)+2:2*ud(1)+1) = pointers on the linear regression lines
	%udl(2*ud(1)+2:2*ud(1)+3)= pointers to intersections
	%udl(2*ud(1)+4:2*ud(1)+5)= pointers to 0.95 lines
	%udl(2*ud(1)+6)= pointers to the text - distance

	
	%udl(2*ud(1)+2:3*ud(1)+1= pointers on the text - distance
	
        
        for i=1:udl(1)
                xj(i,:) = get(udl(1+i),'Xdata');
        end;
        
        %vernier line are vertical, so we can erase redundant xdata
        xj(:,2:udl(1)) = [];
        
        % put vernier coordinates into TRET_DAT(9) userdata array - intervals of
        % quasy-stationarity
        qs = get(TRET_DAT(9),'UserData');
        if(isempty(qs)),
                set(TRET_DAT(9),'UserData',[xj(1) xj(2)]);
        else
                set(TRET_DAT(9),'UserData',[qs;xj(1) xj(2)]);
        end;
        set(TRET_DAT(9),'Enable','on');
        
        xdata=get(accumline,'XData'); 
        ydata=get(accumline,'YData'); 
        xj(udl(1)+1) = min(xdata);
        xj(udl(1)+2) = max(xdata);
        xj = sort(xj);
        
        for i=1:(udl(1)+1),
                if i==1,
                        index = find((xdata >= xj(i)) & (xdata <= xj(i+1)));
                else
                        index = find((xdata > xj(i)) & (xdata <= xj(i+1)));
                end;
                
                X = xdata(index); 
                Y = ydata(index);
                
                [a(i), b(i)] = process('regression_calculation',X,Y);

                yreg = a(i)+b(i)*X; 
                
                lineidx = udl(1)+1+i;
                if (length(udl) < lineidx)
                        
                        l=line(X,yreg,'Color',[0 1 0],'Linewidth',1.5,'erasemode','xor');
                        udl(lineidx) = l;
                else
                        set(udl(lineidx),'Xdata',X, 'Ydata',yreg);
                end;  
        end;
        
        % coordinates of line crossing points
        for i=1:udl(1),
                xcross(i) = (a(i+1) - a(i)) / (b(i) - b(i+1));
		ycross(i) = b(i)*xcross(i) + a(i);
		if isnan(xcross(i)),
			if i==1
				xcross(i)= xdata(1);
				ycross(i)= 0; %ydata(1);
			else
				xcross(i)= xdata(length(xdata));
				ycross(i)= ydata(length(ydata));
			end
		end
	end
	
	if length(udl)<(2*udl(1)+3)
		udl(2*udl(1)+2)=line([xcross(1) xcross(1)],[ycross(1) yl(2)/2],'Color',[0 0.5020 0.7529],'erasemode','xor');
		udl(2*udl(1)+3)=line([xcross(2) xcross(2)],[ycross(2) yl(2)/2],'Color',[0 0.5020 0.7529],'erasemode','xor');
	else
		set(udl(2*udl(1)+2),'Xdata',[xcross(1) xcross(1)],'Ydata',[ycross(1) yl(2)/2]);
		set(udl(2*udl(1)+3),'Xdata',[xcross(2) xcross(2)],'Ydata',[ycross(2) yl(2)/2]);
	end
	

	% AR borders estimation based on 95% of spikes in regression line
	
	xdatai	 = xdata(1):1:xdata(length(xdata));
	[xdatau,iu,ia]= unique(xdata);
	ydatau = ydata(iu);
	ydatai	 = interp1(xdatau,ydatau,xdatai);
	indcrossl = find(abs(xdatai-xcross(1)) == min(abs(xdatai-xcross(1))));
	indcrossr = find(abs(xdatai-xcross(2)) == min(abs(xdatai-xcross(2))));
	
	% y100	 = ycross(2)-ycross(1); BUG!!! (affected 109 simple/complex cells...)
	% ycenteri = ycross(1)+y100/2;

	y100 = ydatai(indcrossr) - ydatai(indcrossl);
	ycenteri = ydatai(indcrossl)+y100/2;
	
	indcenter = find(abs(ydatai-ycenteri) == min(abs(ydatai-ycenteri))); 
	xcenter = xdatai(indcenter);
	yli = ycenteri-0.475*y100;
	yri = ycenteri+0.475*y100;
	indl = find(abs(ydatai-yli) == min(abs(ydatai-yli)));
	indr = find(abs(ydatai-yri) == min(abs(ydatai-yri)));

	if length(udl)<(2*udl(1)+5)
		udl(2*udl(1)+4)=line([xdatai(indl) xdatai(indl)],[ydatai(indl) yl(2)/2],'Color', [1.0000    0.5020 0],'erasemode','xor');
		udl(2*udl(1)+5)=line([xdatai(indr) xdatai(indr)],[ydatai(indr) yl(2)/2],'Color', [1.0000    0.5020 0],'erasemode','xor');	
	else
		set(udl(2*udl(1)+4),'Xdata',[xdatai(indl) xdatai(indl)],'Ydata',[ydatai(indl) yl(2)/2]);
		set(udl(2*udl(1)+5),'Xdata',[xdatai(indr) xdatai(indr)],'Ydata',[ydatai(indr) yl(2)/2]);
	end
	
	% CALCULATE DISTANCE BETWEEN LINES, TAKING INTO ACCOUNT REGRESSION LINE
	str1=[];
	if DefValues(69) % IndexMode on
                ud=get(0,'Userdata');
                ax=get(in1,'Parent');
		pt=get(ax,'CurrentPoint');
		x=pt(1,1);
		
                if DefValues(68) % Inc
                        ind=64;
                else
                        ind=66;
		end
		
                if FBH==verline		% first border
			if 0
				ud(ind)=x;
				xleft = x; xright= ud(ind+1);
			else
				ud(ind)=xdatai(indl);
				xleft = xdatai(indl);
				xright= xdatai(indr);
			end
			
			
		else
			if 0
				ud(ind+1)=x;
				xleft = ud(ind); xright = x;
			else
				ud(ind)=xdatai(indl);
				ud(ind+1)=xdatai(indr);
				xleft = xdatai(indl);
				xright= xdatai(indr);
			end
			
                end    
                set(0,'Userdata',ud);
		str1 = ['  95%: ' num2str((abs(xright - xleft))*velocity/1000,2)];      
        end  
        
               
        %%%%%%% $$$ %%%%%%%   
        % 95% interval between intersection of the lines:
        
        for i=1:udl(1) - 1,
                
		% NOT USED
                if DefValues(35), 
                        reldist=(xj(3)-xj(2))*0.95*velocity*.001;
                        
                else
                        
                        distance = abs(abs(xcross(i+1)) - abs(xcross(i)));
                        reldist=distance*0.95*velocity*.001;
		end;
		% end of NOT USED

		for i=1:udl(1)
			xt(i,:) = get(udl(1+i),'Xdata');
		end;
		absdist=abs((xt(2,1)-xt(1,1)))*velocity*.001;
                
                str = ['abs dis = ', num2str(absdist,2),str1]; 
                textidx=2*udl(1)+6;
                if (length(udl) < textidx)
                        t=text(i*.01, .9, str, 'Units','normalized','erasemode','background');
                        set(t,'FontSize',7,'color',[0.5020 0 0.2510],'FontWeight','normal');
                        udl(textidx) = t;
                else
                        set(udl(textidx),'String',str);
                end;  
        end;  
        set(accumline,'UserData',udl);                         
        set(ax,'XLimMode','Manual','XLim',xl,'YLimMode','Manual','YLim',yl); 
        
        drawnow discard;
	
        %--------------------------------------------------
elseif(strcmp(action, 'get_inform')),
        CurSpk = in1;
        
        TrialNum =CurSpk(1); 
        SpikesNum = CurSpk(2);
        StartSpikeData = CurSpk(3);
        EndSpikeData = CurSpk(4);
        Modality = CurSpk(5);
        SpikeArrTime = CurSpk(6:SpikesNum+5);
        
        EyeMotionLength=CurSpk(SpikesNum+6);
        EyeTimeInterval= CurSpk((SpikesNum+7):(SpikesNum+EyeMotionLength+6));
        HorEyeInterval = CurSpk((SpikesNum+EyeMotionLength+7):(SpikesNum+2*EyeMotionLength+6));
        VerEyeInterval = CurSpk((SpikesNum+2*EyeMotionLength+7):(SpikesNum+3*EyeMotionLength+6));
        Stimvelocity = CurSpk(SpikesNum+3*EyeMotionLength+7);
        Tilt = CurSpk(SpikesNum+3*EyeMotionLength+8);
        
        if (strcmp(in2,'velocity'))
                out1 = Stimvelocity;
        elseif(strcmp(in2,'eyemovement'))
                out1 = [EyeTimeInterval;HorEyeInterval;VerEyeInterval];
        elseif(strcmp(in2,'header'))
                out1 = CurSpk(1:5);
        elseif (strcmp(in2,'tilt'))
                out1 = Tilt;
        end;
        
        %----------------------------------------------
elseif(strcmp(action,'regression_calculation'));
        X = in1; 
        Y = in2; 
        
        
        
        n = length(X);
        SX = sum(X);
        SX2 = sum(X.^2);
        SY = sum(Y);
        SXY = sum(X.*Y);
        a = (SY*SX2 - SX*SXY)/(n*SX2 - SX^2);  
        b = (n*SXY - SX*SY)/(n*SX2 - SX^2);  
        
        
        out1 = a;
        out2 = b;
        
        %-------------------------------------------
elseif(strcmp(action, 'distance_calculation')),
        
        Scatter=get(TRET_DAT(8),'UserData');
        
        accumline=in1;
        ax=get(accumline,'Parent');
        
        SpkInf = get(ax,'UserData');		%Spikes information
        if length(SpkInf)~=1,		% we will not work with averaged data
                
                set(TRET_DAT(2),'CurrentAxes',ax);
                
                ud=get(accumline,'UserData');
                %ud(1) = # of vernier lines (=2);
                %ud(2:ud(1)+1), = pointer to these lines;
                %ud(ud(1)+2:2*ud(1)+1) = pointers on the linear regression lines
                %ud(2*ud(1)+2:3*ud(1)+1= pointers on the text - distance
                
                xvernier = get(ud(2),'Xdata');
                shiftx = get(gca,'XLim');
                
                
                
                velocity = process('get_inform',SpkInf,'velocity');
                Tilt = process('get_inform',SpkInf,'tilt');
                EyeMovementData=process('get_inform',SpkInf,'eyemovement');
                header = process('get_inform',SpkInf,'header');
                
                EyeTimeInterval=EyeMovementData(1,:);
                HorEyeInterval=EyeMovementData(2,:);
                VerEyeInterval=EyeMovementData(3,:);
                
                % for scatter diagram me must transform eye coordinates to the orthogonal basis
                
                EM = [HorEyeInterval;VerEyeInterval];
                alfa = pi*Tilt/180;
                Trans = [cos(alfa) sin(alfa); -sin(alfa) cos(alfa)];
                EMNew = Trans * EM;
                HorEyeInterval = EMNew(1,:);
                VerEyeInterval = EMNew(2,:);
                
                
                index = find((EyeTimeInterval <=xvernier(1)) & (EyeTimeInterval >= xvernier(1) - 100) ) ;
                
                Xmean = mean(HorEyeInterval(index));
                Ymean = mean(VerEyeInterval(index));
                
                tnum = header(1);
                modality=header(5);
                
                
                tmp = [tnum, modality, velocity, xvernier(1), shiftx, Xmean, Ymean];
                
                
                % check if we already chose this segment.
                sscat=size(Scatter,1);
                
                Correction = 0;
                for i=1:sscat,
                        if (~sum(Scatter(i,1:3) -tmp(1:3))) & (tmp(5:6) == Scatter(i,5:6)),
                                % tnum, modality and velocity are the same, distance between xlim and Scatter(4) < stim. length
                                Scatter(i,:) = tmp;
                                Correction = 1;
                                break;
                        end;
                end;
                if (~Correction)
                        Scatter = [Scatter;tmp];
                end;
                
                set(TRET_DAT(8),'Enable','on','UserData',Scatter);
        end;
        
        %-------------------------------------------
elseif(strcmp(action,'redraw_sct'))
        
        Scatter = in1;
        n = size(Scatter,1);
        index0 = 0;
        index1 = 0;
        S0 = [];
        S1 = [];
        
        for i = 1:n,
                if(Scatter(i,2)==0)
                        index0 = index0+1;
                        S0(index0,:) = Scatter(i,:);
                else
                        index1 = index1+1;
                        S1(index1,:) = Scatter(i,:);
                end;
        end;
        
        lb = ['distance, min'];
        xlb = ['X eye movement, min'];
        ylb = ['Y eye movement, min'];
        
        
        % format scatterd data:
        %  tmp = [tnum, modality, velocity, marker_time, start_stim_time, end_stim_time, Xmean, Ymean];
        
        ax = axes('Position',[0 0 1 1],'Units','normal','xcolor','w','ycolor','w','zcolor','w');
        fname = get(gcf,'Name');
        text(.5,.98,fname,'Units','norm','Color','r','HorizontalAlignment','Center',...
                'FontSize',12,'Color',[0 0 1],'FontName','Times');
        
        
        if(~isempty(S0)),     
                dist0 = S0(:,3).*(S0(:,4) - S0(:,5)).*.001;
                
                [aonx,bonx] = process('regression_calculation',dist0,S0(:,7));
                reg0x = process('regression_line',[aonx,bonx], dist0);
                coeff = process('regression_test',[aonx,bonx],dist0,S0(:,7));
                t =   coeff(1);
                Rr =  coeff(2);
                Rxy = coeff(3);
                n =   coeff(4);
                Tponx =  coeff(5);
                
                strx = ['n = ',num2str(n), ', t = ',num2str(t), ', Rr = ',num2str(Rr), ', Rxy = ',num2str(Rxy)];
                
                
                [aony,bony] = process('regression_calculation',dist0,S0(:,8));
                reg0y = process('regression_line',[aony,bony], dist0);
                coeff = process('regression_test',[aony,bony],dist0,S0(:,8));
                t =   coeff(1);
                Rr =  coeff(2);
                Rxy = coeff(3);
                n =   coeff(4);
                Tpony =  coeff(5);
                
                stry = ['n = ',num2str(n), ', t = ',num2str(t), ', Rr = ',num2str(Rr), ', Rxy = ',num2str(Rxy)];
                
                set(gcf,'CurrentAxes', ax);
                text(.7,.95,'''Off'' Segments','Units','norm','Color','r','HorizontalAlignment','Center',...
                        'FontSize',9,'Color',[ 1 0 .5],'FontName','Times');
                
                
                axes('Position',[.6 .6 .3 .3],'Units','normal','xcolor','k','ycolor','k');
                plot(dist0,S0(:,7),'or',dist0, reg0x,'b');
                xlabel(lb);
                ylabel(xlb);
                title (strx);
                axis('square');
                text(.6, .9, ['Paired t-test = ',num2str(Tponx)],'Units','Normalized','FontSize',8,'FontWeight','norm');
                text(.6, .85, ['Intersept = ',num2str(aonx)],'Units','Normalized','FontSize',8,'FontWeight','norm');
                text(.6, .8, ['Slope = ',num2str(bonx)],'Units','Normalized','FontSize',8,'FontWeight','norm');
                
                %     subplot(223);
                axes('Position',[.6 .1 .3 .3],'Units','normal','xcolor','k','ycolor','k');
                plot(dist0,S0(:,8),'or',dist0, reg0y,'b');
                xlabel(lb);
                ylabel(ylb);
                title(stry);
                axis('square');
                text(.6, .9, ['Paired t-test = ',num2str(Tpony)],'Units','Normalized','FontSize',8,'FontWeight','norm');
                text(.6, .85, ['Intersept = ',num2str(aony)],'Units','Normalized','FontSize',8,'FontWeight','norm');
                text(.6, .8, ['Slope = ',num2str(bony)],'Units','Normalized','FontSize',8,'FontWeight','norm');
                
                
        end; 
        
        if(~isempty(S1)),
                dist1 = S1(:,3).*(S1(:,4) - S1(:,5)).*.001;;
                
                [aoffx,boffx] = process('regression_calculation',dist1,S1(:,7));
                reg1x = process('regression_line',[aoffx,boffx], dist1);
                coeff = process('regression_test',[aoffx,boffx],dist1,S1(:,7));
                t =   coeff(1);
                Rr =  coeff(2);
                Rxy = coeff(3);
                n =   coeff(4);
                Tpoffx =  coeff(5);
                
                
                
                strx = ['n = ',num2str(n), ', t = ',num2str(t), ', Rr = ',num2str(Rr), ', Rxy = ',num2str(Rxy)];
                
                [aoffy,boffy] = process('regression_calculation',dist1,S1(:,8));
                reg1y = process('regression_line',[aoffy,boffy], dist1);
                coeff = process('regression_test',[aoffy,boffy],dist1,S1(:,8));
                t =   coeff(1);
                Rr =  coeff(2);
                Rxy = coeff(3);
                n =   coeff(4);
                
                Tpoffy =  coeff(5);
                
                
                stry = ['n = ',num2str(n), ', t = ',num2str(t), ', Rr = ',num2str(Rr), ', Rxy = ',num2str(Rxy)];
                
                
                %     subplot(222);
                
                set(gcf,'CurrentAxes',ax);
                text(.2,.95,'''On'' Segments','Units','norm','Color','r','HorizontalAlignment','Center',...
                        'FontSize',9,'Color',[ 1 0 .5],'FontName','Times');
                
                axes('Position',[.1 .6 .3 .3],'Units','normal','xcolor','k','ycolor','k');
                
                plot(dist1,S1(:,7),'or',dist1, reg1x,'b');
                xlabel(lb);
                ylabel(xlb);
                title (strx);
                axis('square');
                text(.6, .9, ['Paired t-test = ',num2str(Tpoffx)],'Units','Normalized','FontSize',8,'FontWeight','norm');
                text(.6, .85, ['Intersept = ',num2str(aoffx)],'Units','Normalized','FontSize',8,'FontWeight','norm');
                text(.6, .8, ['Slope = ',num2str(boffx)],'Units','Normalized','FontSize',8,'FontWeight','norm');
                
                
                %     subplot(224);
                
                axes('Position',[.1 .1 .3 .3],'Units','normal','xcolor','k','ycolor','k');
                
                plot(dist1,S1(:,8),'or',dist1, reg1y,'b');
                xlabel(lb);
                ylabel(ylb);
                title(stry);
                axis('square');
                text(.6, .9, ['Paired t-test = ',num2str(Tpoffy)],'Units','Normalized','FontSize',8,'FontWeight','norm');
                text(.6, .85, ['Intersept = ',num2str(aoffy)],'Units','Normalized','FontSize',8,'FontWeight','norm');
                text(.6, .8, ['Slope = ',num2str(boffy)],'Units','Normalized','FontSize',8,'FontWeight','norm');
                
        end;   
        set(gcf,'Pointer','arrow');
        
        
        %-------------------------------------------
elseif(strcmp(action,'regression_line'))
        
        a = in1(1);
        b = in1(2);
        x = in2;
        out1 = a+b*x;
        %-------------------------------------------
elseif(strcmp(action,'regression_test'))
        a = in1(1);
        b = in1(2);
        x = in2;
        y = in3;   
        n = length(x);
        %means
        mx = mean(x);
        my = mean(y);
        
        %standard deviation of the data
        sx=std(x);
        sy=std(y);
        %standard error of the estimate    
        syx = sqrt((n-1)*(sy^2 - b^2 * sx^2)/(n-2));
        %standard error of the slope
        sb  = syx/(sqrt(n-1)*sx);
        %t-statistic
        t  = b/sb;
        %r - correlation
        Rr = sqrt(1-((n-2)*syx^2)/((n-1)*sy^2));
        Rxy = sum((x-mx).*(y-my))/sqrt(sum((x-mx).^2)*sum((y-my).^2));
        % paired t-test
        d=x-y;
        nom = sqrt(n-1)*sum(d)/n;
        denom = sqrt((sum(d.^2) - ((sum(d))^2)/n)/n);
        Tp = nom/denom;
        
        
        % out    
        out1 = [t Rr Rxy n Tp];
        %-------------------------------------------
        
        %-------------------------------------------
elseif(strcmp(action,'spectra'))
        
        % OPEN NEW GRAPH FIGURE
        figcounter = figcounter + 1; 
        
	if in1==15
		eyedata = unitret('display_concatenated',1);
	end
	
        % CurFig=figure(figcounter);
        CurFig = figure('Units','Normal','Position',[.01+figcounter/100 .01+figcounter/100 .60 .70],'Userdata',TRET_DAT(2),'visible','off');
        set(gcf,'WindowButtonDownFcn','ud = get(0,''Userdata'');ud(72) = gcf;set(0,''Userdata'',ud);');
        % grey color: [0.7843 0.7255 0.7765]
        
        %set(CurFig,'WindowButtonDownFcn','set(get(gcf,''Userdata''),''Userdata'',gcf);');
        set(CurFig ,'Backingstore','off','Name',['Spectrum: ',NAME],'Color',FIGCOLOR,...
                'Resize','On', 'Menubar','none','Number','off','Visible','off');
        set(gcf,'Pointer','Watch');
        PrntHndl=uimenu(CurFig,'Label','&Print Graph');
        uimenu(PrntHndl,'Label','P&rint Setup...','Separator','on','Callback','print -dsetup;');     
        uimenu(PrntHndl,'Label','&Print','Callback','print;');
        uimenu(PrntHndl,'Label','&Save in HGL format As...','Separator','on','Callback','process(''save_hgl'');');      
        uimenu(PrntHndl,'Label','&Save in BMP format As...','Separator','on','Callback','process(''save_bmp'');'); 
        uimenu(PrntHndl,'Label','&Copy to Clipboard as bitmap','Separator','on','Callback','print -dbitmap'); 
        uimenu(PrntHndl,'Label','&Copy to Clipboard as metafile','Separator','on','Callback','print -dmeta'); 
        
        %cellname1=get(jpsth_UI(5),'string');
        %cellname2=get(jpsth_UI(6),'string');
        %uicontrol(gcf,'style','text','String',[cellname1(8:length(cellname1)),cellname2(8:length(cellname2))],'Backgroundcolor','w','ForegroundColor',[.2 .6 .5],'FontSize',8,'Position',[0.001 0.94 0.15 .065]);
        
        set(0,'Currentfigure',CurFig);
        
        if in1==3 | in1==4 | in1==5 | in1==6 | in1==7 | in1==8 | in1==9 | in1==10 | in1==15
                
                set(gca,'drawmode','fast','Position',[.25 .1 .70 .70],'XColor','k','YColor','k');
        else  
                set(gca,'drawmode','fast','Position',[.05 .1 .70 .95],'XColor','k','YColor','k');
                
        end   
        
        
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	if nargin==3			% in2 exists
		spikes_inform = in2;
	else
		spikes_inform = unitret('get_markeddata');
	end
        
        
        
        
        if in1==1,		% high resolution, standard fft procedure
                Spk = process('make_train',spikes_inform);
                Spectrum = process('spec_calc_high',Spk);
                out1 = process('spec_draw_high',Spectrum);
                figname='High Resolution Spectrum: ';
                
        elseif in1 ==2,		% low resolution, boxcar window - RAW
                Spk = process('make_train',spikes_inform);
                set(gcf,'inverthardcopy','off');
                out1 = process('spec_calc_low',Spk,' ');
                figname='Low Resolution Spectrum: ';
                
        elseif in1 ==3,		% average
                set(gcf,'inverthardcopy','off');
                out1 = process('spec_calc_aver',0);
                figname='PSTH Spectrum: ';
                
        elseif in1 ==4,		% average - AUTOCORRELATION
                set(gcf,'inverthardcopy','off');
                out1 = process('spec_calc_aver',1);
                figname='Spectrum of autocorrelation: ';
                
        elseif in1 ==5		% concatenated
                Spk = process('make_train',spikes_inform);
                set(gcf,'inverthardcopy','off');
                ConcSpk=[];
                SpkLen=size(Spk,2);
                for i=1:size(Spk,1)
                        % OffSet=(i-1)*SpkLen+1;
                        ConcSpk=[ConcSpk Spk(i,1:spikes_inform(i,4)-spikes_inform(i,3))];
                end
                out1 = process('spec_calc_low',ConcSpk,'concatenated');
                figname='Spectrum on Concatenated Raw: ';
                
        elseif in1 == 6             % Harmonic Analysis on PSTH
                set(gcf,'inverthardcopy','off');
                out1 = process('harmonic_anal',0); 
                figname='Harmonic Analysis on PSTH: ';
                
        elseif in1 == 7             % Harmonic Analysis on Raw (Concatenated)
                Spk = process('make_train',spikes_inform);
                set(gcf,'inverthardcopy','off');
                ConcSpk=[];
                SpkLen=size(Spk,2);
                for i=1:size(Spk,1)
                        % OffSet=(i-1)*SpkLen+1;
                        ConcSpk=[ConcSpk Spk(i,1:spikes_inform(i,4)-spikes_inform(i,3))];
                end
                out1 = process('harmonic_anal',1,ConcSpk);
                figname='Harmonic Analysis on Concatenated Raw: ';
                
        elseif in1 == 8             % Harmonic Analysis on Averaged
                set(gcf,'inverthardcopy','off');
                out1 = process('harmonic_anal',2); 
                figname='Harmonic Analysis on Averaged: ';
                
        elseif in1 == 11            % Harmonic Analysis on Raw (Concatenated): Hochstein
                Spk = process('make_train',spikes_inform);
                set(gcf,'inverthardcopy','off');
                ConcSpk=[];
                SpkLen=size(Spk,2);
                for i=1:size(Spk,1)
                        % OffSet=(i-1)*SpkLen+1;
                        ConcSpk=[ConcSpk Spk(i,1:spikes_inform(i,4)-spikes_inform(i,3))];
                end
                out1 = process('harmonic_anal',3,ConcSpk); 
                figname='Harmonic Analysis - Hochstein: ';
                
        elseif in1 == 12            % Harmonic Analysis per each segment (and then average)
                Spk = process('make_train',spikes_inform);
                set(gcf,'inverthardcopy','off');
                out1 = process('harmonic_anal',4,Spk); 
                figname='Harmonic Analysis per each segment: ';
                
        elseif in1 == 13            % Harmonic Analysis per each segment (no averaging)
                Spk = process('make_train',spikes_inform);
                set(gcf,'inverthardcopy','off');
                out1 = process('harmonic_anal',5,Spk); 
                figname='Harmonic Analysis per each segment: ';

	elseif in1 == 14	    % Full Harmonic
		delete(CurFig);
		out1 = process('spectra',7);
		out2 = process('spectra',12);  
		data = [out1.rm out2.rm out1.dc out2.phase out1.max_harmonic out1.max_harmonic_amp];
		unitret('grand_handle','full harmonic',data);         

	elseif in1 == 15	    % Eye spectra
		out1 = process('eye_spectra',eyedata);       
        end 
        
        %set(gca,'Xcolor','k','Ycolor','k');  
        %LineHndl=get(gca,'Children');
        %set(LineHndl,'Color','k');
        
        set(CurFig,'Pointer','Arrow','Visible','on','Name',[figname NAME]);
        set(TRET_DAT(2),'Pointer','Arrow');

	
	% out1 comes from harmonic_anal
        
        %-------------------------------------------
elseif(strcmp(action,'correlogram'))
        % subroutine works only for 2 spont. segments
        
        spikes_inform = get(gcf,'UserData');
        % they should be the same, but anyway...
        
        Trials = size(spikes_inform,1);
        if Trials ~= 2,
                warndlg('sorry, can not process more then 2 trials!');
                return;
        else,
                
                SpkStart = min(spikes_inform(:,3));
                SpkEnd = max(spikes_inform(:,4));
                A = spikes_inform(1,6:spikes_inform(1,2)+5);
                B = spikes_inform(2,6:spikes_inform(2,2)+5);
                w = 120;
                dt = 2;
                xc = xcor(A/1000, B/1000, SpkStart/1000, SpkEnd/1000, dt/1000, w/1000);
                a = xc(:,1); b = xc(:,2); 
                figure;
                set(gcf,'WindowButtonDownFcn','ud = get(0,''Userdata'');ud(72) = gcf;set(0,''Userdata'',ud);');
                stairs(a*1000,b);
                ylabel('spikes/sec','color','w');
                xlabel('time, msec');
                
                % baseline
                
                
                for j=1:2,
                        if j==1,SpkInt = diff(A);
                        elseif j==2, SpkInt  = diff(B); 
                        end;
                        
                        IntNum = length(SpkInt);
                        SpkCpy = SpkInt;
                        ShufSpk = zeros(size(SpkInt));
                        
                        for i=1:IntNum,
                                index = IntNum-i;
                                k=fix(rand(1,1) * index) +1;
                                ShufSpk(i) = SpkCpy(k);
                                SpkCpy(k) = [];
                        end;
                        for k=1:length(ShufSpk),
                                EventTime = sum(ShufSpk(1:k));
                                if j==1, ShufA(k) = EventTime;
                                elseif j==2, ShufB(k) = EventTime;
                                end;
                        end; 
                end;
                shufxc = xcor(ShufA/1000, ShufB/1000, SpkStart/1000, SpkEnd/1000, dt/1000, w/1000);
                
                b = shufxc(:,2);
                
                x=mean(b);lowp = 0;upp=0;
                for m=0:100*x,
                        pmx = (exp(-x)*x^m)/process('factorial',m);    
                        lowp = lowp+pmx;
                        if lowp >=0.005,
                                break;
                        end;
                end;
                lowlim = m;
                
                upp = 0;
                for m=0:100*x,
                        pmx = (exp(-x)*x^m)/process('factorial',m);    
                        upp = upp+pmx;
                        if upp >=0.995,
                                break;
                        end;
                end;
                uplim = m;
                lowlim = ones(size(a))*lowlim;
                uplim = ones(size(a))*uplim;
                
                
                hold on;
                plot(1000*a, lowlim,'r--');
                plot(1000*a, uplim, 'r--');
                set(gca,'XLim',[-w,w]);
                
                hold off
                
                
                
                
        end;
        
        %--------------------------------------------------------------------------------
elseif strcmp(action,'make_train'),
        
        %  Fsamp = 1000;    
        % Actually, we work with F sampling = 10 kHz, that gives 0.1 msec accuracy level.
        % However, this accuracy is redundant, 1 msec is enough, so we can use rounding operation,
        % then F sampling = 1kHz .
        
        
        Trials = size(in1,1);
        
        SpkStarts = (in1(:,3));
        SpkEnds =  (in1(:,4));
        veclength = max(SpkEnds-SpkStarts);     %record time for the trials could be different!
        
        Spk = zeros(Trials, veclength); 
        
        for i = 1:Trials,
                idx = in1(i,6:in1(i,2)+5);
                Spk(i,ceil(idx-in1(i,3)+eps)) = ones(1,in1(i,2));
                %Spk(i,ceil(idx-in1(i,3)+eps)) = ones(1,in1(i,2));
        end;
        
        
        out1 = Spk;
        %---------------------------------------------------------------------------------
        
elseif(strcmp(action,'spec_calc_high'))
        
        [Trials,lgn] = size(in1);
        N = 2^(nextpow2(lgn));
        Spectrum = zeros(Trials, N);
        for i =1:Trials,
                in1(i,:) =(in1(i,:) - mean(in1(i,:)))/(N/2);
                Spectrum(i,:) = fft(in1(i,:),N);
                
        end;
        out1 = Spectrum;     
        %---------------------------------------------------------------------------------
        
elseif(strcmp(action,'spec_calc_low'))
        WC = get(gcf, 'Color');
        
        [Trials,m] = size(in1);
        % N = 2^(nextpow2(m));
        Fsample  = 1000;
        Fdisplay = DefValues(53);
        nfft     = DefValues(59);
        noverlap = DefValues(60);
        window   = boxcar(DefValues(61));
        p        = DefValues(62);
        
        TF = DefValues(75); 
        
        if DefValues(50) % with DC
                dflag='none';
        else
                dflag='mean';
        end
        
        
        % FreqAxis = (0:nfft/2-1)*Fsample/nfft;
        
        spikes_inform = unitret('get_markeddata');
        
        for i =1:Trials,
                if Trials > 1,
                        
                        SpkStarts = spikes_inform(i,3);
                        SpkEnds =  spikes_inform(i,4);
                        
                        cur_train = in1(i,1:SpkEnds-SpkStarts);
                else % Concatenated or one segment
                        cur_train = in1;
                end
                
                if DefValues(59) == 0,
                        nfft = length(cur_train);
                end
                
                if DefValues(61) == 0,
                        window   = boxcar(length(cur_train));
                end
                
                [P,FreqAxis] = unispectrum(cur_train, nfft,noverlap,window,Fsample,p,dflag);
                P = 1000*P;
                r = find(FreqAxis <= Fdisplay);
                subplot(Trials,1,i);
                c = [P(r,1)+P(r,2) P(r,1)-P(r,2)];
                
                
                plot(FreqAxis(r),P(r,1),'k'), hold on
                plot(FreqAxis(r),c(:,1),'r:',FreqAxis(r),c(:,2),'r:'), hold on
                
                
                F1ind = find(abs(FreqAxis-TF) == min(abs(FreqAxis-TF))); 
                AC1 = P(F1ind,1);
                
                
                
                xlabel(['Frequency, Hz    Resolution: ',num2str(FreqAxis(2)-FreqAxis(1)),' Hz. AC1/DC = ', num2str( AC1^.5/( P(1,1)^.5/2 ) , 2)],'Color','b');
                ylabel(['Power (spikes/s)^2/Hz    (#spikes: ',num2str(sum(in1(i,:)>0)),')'],'Color','b')
        end;
        
        
        TF = DefValues(75); 
        F1ind = find(abs(FreqAxis-TF) == min(abs(FreqAxis-TF))); 
        AC1 = P(F1ind,1);
        
        if ( Trials == 1 & strcmp(dflag,'none') ),
                MaxPower=max(P(2:max(r),1));
                uicontrol(gcf,'Style','text','Position',[.2 .95 .3 .05],...
                        'Units','normalized','BackgroundColor',WC,...
                        'ForegroundColor',[0 0.410 0.6510],'String',['AC1/DC: ',num2str( AC1^.5/( P(1,1)^.5/2 ) )],'HorizontalAlignment','center');
        else
                uicontrol(gcf,'Style','text','Position',[.2 .95 .3 .05],...
                        'Units','normalized','BackgroundColor',WC,...
                        'ForegroundColor',[0 0.410 0.6510],'String',['Detrended'],'HorizontalAlignment','center');
        end	
        
        %---------------------------------------------------------------------------------
        
elseif(strcmp(action,'spec_calc_aver'))
        
        global AV_1 AV_2
        
        for i = 1:2,      
                com = ['data = AV_', num2str(i),';'];
                eval(com);
                [m,n] = size(data);
                if n > 0,
                        SpecLgn = fix(n/2);
                        
                        Fsample = 1000/(data(1,2) - data(1,1));
                        Fdisplay = DefValues(53);
                        FreqAxis = (0:SpecLgn-1)*Fsample/n;
                        
                        if in1==0,			% spectrum of histogram
                                if DefValues(50) % with DC
                                        P = ( abs(fft(data(2,:))) ).^2/length(data(2,:));
                                        MaxPower=max(P(2:SpecLgn));
                                        uicontrol(gcf,'Style','text','Position',[.03 .005 .3 .05],...
                                                'Units','normalized','BackgroundColor','w',...
                                                'ForegroundColor',[0 0.410 0.6510],'String',['Ratio Max/DC: ',num2str( MaxPower^.5/( P(1,1)^.5/2 ) )],'HorizontalAlignment','center');
                                else
                                        P = ( abs(fft(data(2,:)-mean(data(2,:)))) ).^2/length(data(2,:)); % without DC
                                        uicontrol(gcf,'Style','text','Position',[.03 .005 .3 .05],...
                                                'Units','normalized','BackgroundColor','w',...
                                                'ForegroundColor',[0 0.410 0.6510],'String',['Without DC'],'HorizontalAlignment','center');
                                        
                                        %% P = abs( fft(data(2,:))-fft(mean(data(2,:))) ); % without DC (the same result)
                                end
                                
                                subind = 210+i;
                                subplot(subind);
                                plot(FreqAxis, P(1:SpecLgn));
                                xlabel('Frequency, Hz','Color','k'); 
                                set(gca,'XLim',[0 Fdisplay]);
                                
                                
                                
                                
                        elseif in1==1, 			% spectrum of autocorrelation
                                
                                s = xcorr(data(2,:),'unbiased');
                                S = s(n:2*n-1);
                                P =  abs(fft(S));
                                MinMaxInd = process('minmax',S);
                                FirstMin = S(MinMaxInd(1));
                                FirstMax = S(MinMaxInd(2));
                                
                                B = sqrt(FirstMax-FirstMin);
                                A = mean(data(2,:));
                                
                                subind = 220+i;
                                subplot(subind);
                                TimeAx = (0:n-1)./Fsample;
                                stairs(TimeAx, S);
                                hold on; plot(TimeAx(MinMaxInd(1)),FirstMin,'go',...
                                        TimeAx(MinMaxInd(2)),FirstMax,'ro'); 
                                str = sprintf('A=%4.2f, B= %4.2f, Cell Index = %4.2f',A,B,B/A);
                                title(str,'color','w','FontSize',8);
                                xlabel('Time, sec','Color','b');
                                hold off;
                                subind = 222+i;
                                subplot(subind);
                                plot(FreqAxis, P(1:SpecLgn));                  
                                xlabel('Frequency, Hz','Color','b'); 
                                set(gca,'XLim',[0 Fdisplay]);
                                
                        end;
                        
                end;
        end;
        
        
        
        %-----------------------------------------------------
elseif(strcmp(action,'spec_draw_high'))
        
        Fsample = 1000;
        Fdisplay = 100;
        
        %figure;set(gcf,'inverthardcopy','off');
        set(gcf,'WindowButtonDownFcn','ud = get(0,''Userdata'');ud(72) = gcf;set(0,''Userdata'',ud);');
        
        RealSpectrum = abs(in1);
        
        [n,m]=size(RealSpectrum);
        
        FreqAxis = (0:m/2-1)*Fsample/m;
        r = find(FreqAxis <= Fdisplay);
        
        %SubInit = [num2str(n),'1','0'];
        %SubInit = str2num(SubInit);
        for i = 1:n,
                subplot(n,1,i);
                plot(FreqAxis(r),RealSpectrum(i,r));
                xlabel('Frequency, Hz','color','w');
        end;
        
        
        
        %-------------------------------------------
elseif(strcmp(action, 'minmax')),
        S = in1;
        FirstMin = 99999;
        FirstMax = -99999;
        Vicinity = 3;
        i = 1;
        % First Minimum
        
        while (1),
                
                if S(i) < FirstMin,
                        FirstMin = S(i);
                        MinIndex = i;
                        i = i+1;
                else, 		% oops, it might be local minima only. Try to look in  vicinity
                        if FirstMin < mean(S(i:i+Vicinity)) ,		% it is global minimum
                                break;
                        else
                                i = i+Vicinity;		%jump over the gap
                        end;
                end;
        end;
        
        % First Maximum after the first minimum
        
        while (1),
                
                if S(i) > FirstMax,
                        FirstMax = S(i);
                        MaxIndex = i;
                        i = i+1;
                else, 		%oops, it might be local minima only. Try to look in  vicinity
                        if FirstMax > mean(S(i:i+Vicinity)) ,		% it is global minimum
                                break;
                        else
                                i = i+Vicinity;		%jump over the gap
                        end;
                end;
        end;
        out1 = [MinIndex MaxIndex];
        
        
        %-------------------------------------------
elseif(strcmp(action, 'factorial')),
        if in1 == 0,
                out1 = 1;
        else
                out1 = 1;
                for i = 1:in1,
                        out1 = out1*i;
                end;
        end;  
        
        %-------------------------------------------
elseif(strcmp(action,'ori_tuning_old'))
        % subroutine for drawing orientation tuning curve
        
        spikes_inform = get(gcf,'UserData');
        
        Segments = size(spikes_inform,1);
        for i = 1:Segments,
                SpkInf = spikes_inform(i,:);
                header = process('get_inform',SpkInf,'header');
                Tilt(i) = process('get_inform',SpkInf,'tilt');
                SpkNum(i) = header(2);
                Modality(i) = header(5);    
	end;

        
        % save tuning curve
        dot=(in1 - '.');
        b=find(dot==0);
        b = max(b);
        
        extent = 'tun';		%tuning curve
        
        if(~isempty(b))
                TunName= [in1(1:b),extent];
        else
                TunName = [in1,'.',extent];;
        end;
        [fid,message] = fopen(TunName,'wt');
        fprintf(fid,'%s\n', in1);
        fprintf(fid,'%s\t%s\t%s\n', 'Mod','Tilt','Spikes');
        for i = 1:Segments,
                fprintf(fid,'%d\t%d\t%d\n', Modality(i),Tilt(i),SpkNum(i));
        end;
        fclose(fid);
        
        
        figure;
        set(gcf,'WindowButtonDownFcn','ud = get(0,''Userdata'');ud(72) = gcf;set(0,''Userdata'',ud);');
        set(gcf,'name',['Tuning curve: ',TunName],'inverthardcopy','off');
				       
        for j = 0:2,
                if j == 2,
                        RawTunCurve = [Tilt' SpkNum'];
                else, 
                        CurMod = find(Modality==j);
                        RawTunCurve = [Tilt(CurMod)' SpkNum(CurMod)'];
                end;
                
                if (size(RawTunCurve,1)),
                        [tmp, ind] = sort(RawTunCurve(:,1));
                        RawTunCurve = [RawTunCurve(ind,1) RawTunCurve(ind,2)];
                        SegNum = length(ind);
                        
                        
                        
                        index = 1;
                        Num = [];
                        
                        
                        
                        if SegNum >1,
                                while(index <= size(RawTunCurve,1))
                                        mstep=1;
                                        
                                        if (index < size(RawTunCurve,1)),
                                                if   RawTunCurve(index,1) == RawTunCurve(index+1,1),
                                                        
                                                        AverResp = RawTunCurve(index,2);
                                                        while   (RawTunCurve(index,1) == RawTunCurve(index+mstep,1)),
                                                                AverResp =  AverResp + RawTunCurve(index+mstep,2);
                                                                mstep = mstep+1;
                                                                if (index+mstep > size(RawTunCurve,1));
                                                                        break;
                                                                end;   
                                                        end;
                                                        AverResp = AverResp/mstep;
                                                        
                                                        
                                                        RawTunCurve(index,:) =  [RawTunCurve(index,1), AverResp]; 
                                                        
                                                        
                                                        RawTunCurve(index+1:index+mstep-1,:) = [];
                                                        
                                                end;					%endif
                                                
                                        end;
                                        Num(index)= mstep;
                                        index = index+1;         
                                        %        num_ind = num_ind+1;
                                        
                                end;					%endwhile  
                        end;		%endif
                        
                        
                        
                        tmp = process('tuning_width',RawTunCurve);
                        xx1 = tmp(1);
                        xx2 = tmp(2);
                        YBound = tmp(3);
                        
                        subplot(311+j);     
                        plot(RawTunCurve(:,1), RawTunCurve(:,2),RawTunCurve(:,1), RawTunCurve(:,2),'og',...
                                [xx1 xx2],[YBound YBound],'g--');
                        grid;
                        
                        if ~isempty(Num),
                                for k = 1:length(RawTunCurve(:,1)),
                                        text(RawTunCurve(k,1),1.05*RawTunCurve(k,2),num2str(Num(k)),'units','data','color','w','fontsize',10,...
                                                'FontWeight','norm');
                                end;         
                        end;
                        
                        nonOptVsOpt=min(RawTunCurve(:,2)/max(RawTunCurve(:,2)));
                        width=xx2-xx1;
                        
                        if (j==0),
                                title(['''On'' resp., Width, deg =', num2str(width),' nonOpt/Opt= ',num2str(nonOptVsOpt)],'color','k')
                        elseif (j==1),
                                title(['''Off'' resp., Width, deg =', num2str(width),' nonOpt/Opt= ',num2str(nonOptVsOpt) ],'color','k');
                        elseif (j==2),
                                title(['''On & Off'' resp., Width, deg =', num2str(width),' nonOpt/Opt= ',num2str(nonOptVsOpt) ],'color','k');
                        end; 
                        xlabel('Degrees','color','k');
                        ylabel('Spikes','color','k');
                end;			%endif  
	end
       %-------------------------------------------
       %
elseif(strcmp(action,'ori_tuning'))
        % new subroutine for drawing orientation tuning curve


	dot=(in1 - '.');
        b=find(dot==0);
        b = max(b);
        
        extent = 'ori';		%tuning curve
        
        if(~isempty(b))
                TunName= [in1(1:b),extent];
        else
                TunName = [in1,'.',extent];;
	end;
	
        spikes_inform = get(gcf,'UserData');
	
        nsegm = size(spikes_inform,1);
        for i = 1:nsegm,
                SpkInf = spikes_inform(i,:);
                header = process('get_inform',SpkInf,'header');
                Tilt(i) = process('get_inform',SpkInf,'tilt');
                SpkNum(i) = header(2);
		StartTime(i) = header(3);
		EndTime(i) = header(4);
                Modality(i) = header(5);    
	end;

	uTilt = unique(Tilt);
	[theta] = sort(uTilt);

	for i=1:length(theta),
		thisTiltInds = find(Tilt==theta(i));
		r(i) = sum(SpkNum(thisTiltInds))/sum(EndTime(thisTiltInds)-StartTime(thisTiltInds))*1000; % spikes/s
	end

	maxind = find(r==max(r));
	rightshift = 90-theta(maxind);

	if abs(rightshift)<DefValues(186), % peak shift threshold
		rightshift = 0;
	end
		
	theta = theta+rightshift;
	negind = find(theta<0);
	overind = find(theta>180);
	
	theta(negind)=theta(negind)+ 180;
	theta(overind)=theta(overind)- 180;

	% re-check uniqueness
	[theta,uniqind] = unique(theta);
	r = r(uniqind);

	% re-sort theta
	[theta,sortind] = sort(theta);
	r = r(sortind);
	

	% Interpolate signal 
	thetai = [min(theta):max(theta)];
	thetax = theta;			% for interpolation - add "ends"
	ry	= r;
	
	if min(theta)>0
		if min(theta)>1
			thetai = [0:(min(theta)-1) thetai];
			thetax = [0 min(theta)-1 thetax];
			ry = [0 0 ry];
		else
			thetai = [0:(min(theta)-1) thetai];
			thetax = [0 thetax];
			ry = [0 ry];
		end
	end
	if max(theta)<180
		if max(theta)<179
			thetai = [thetai (max(thetai)+1):180];
			thetax = [thetax max(theta)+1 180];
			ry = [ry 0 0];
		else
			thetai = [thetai (max(thetai)+1):180];
			thetax = [thetax 180];
			ry = [ry 0];
		end
	end

	ri = interp1(thetax,ry,thetai);
	
	% Filter interpolated curve
	halfwidth = DefValues(185);
	rs = conv(hanning(halfwidth*4),ri);
	rs = rs(halfwidth*2:end-halfwidth*2);
	rs = rs/max(rs)*max(r);
	
	% Circular variance (Ringach et al. 2002)

	% resample interpolated curve in 15 deg intervals
	thetai_rs = 0:15:165;
	ri_rs = rs(1:15:166);
		
	j=sqrt(-1);
	R = sum(ri_rs.*exp(j*2*(thetai_rs/180*pi)))/sum(ri_rs);
	CV = 1 - abs(R);

	% half-bandwidth @ 1/sqrt(2) height (Ringach et al. 2002)
	% rs = unitret('smooth',ri);

	peak_rs = max(rs);
	peak_r = max(r);
	
	peak_thetai_ind = find(rs==peak_rs);
	peak_theta_ind = find(r==peak_r);
	
	peak_thetai = thetai(peak_thetai_ind);
	peak_theta = theta(peak_theta_ind);

	leftband_thetai = thetai(find( abs(rs(1:peak_thetai_ind)-1/sqrt(2)*peak_rs) ==...
				 min(abs(rs(1:peak_thetai_ind)-1/sqrt(2)*peak_rs) )));
	rightband_thetai = thetai(peak_thetai_ind + find( abs(rs(peak_thetai_ind+1:end)-1/sqrt(2)*peak_rs) ==...
				min(abs(rs(peak_thetai_ind+1:end)-1/sqrt(2)*peak_rs) )));
	
	if isempty(leftband_thetai) | isempty(rightband_thetai),
		bw = 180;
	else
		bw = 0.5*(rightband_thetai - leftband_thetai);
	end
	
        hFig = figure('Units','Normalized','Position',[0.2 0.2 0.7 0.5],'WindowButtonDownFcn','ud = get(0,''Userdata'');ud(72) = gcf;set(0,''Userdata'',ud);',...
		      'name',['Orientation tuning: ',TunName],'inverthardcopy','off');
	subplot(1,2,1)
	hOriPlot = plot(theta,r,'LineStyle','-','Color','k','Marker','o','MarkerFaceColor','k','Tag','OriPlot');
	hold on
	hSmoothedOriPlot = plot(thetai,rs,'LineStyle','-','Color', [0.8 0 0.2],'Marker','none','MarkerFaceColor','k','Tag','SmoothedOriPlot');
	line([leftband_thetai leftband_thetai],[0 1/sqrt(2)*peak_rs],'Color','b','LineStyle',':');
	line([rightband_thetai rightband_thetai],[0 1/sqrt(2)*peak_rs],'Color','b','LineStyle',':');
	hold on
	hResampledOriPlot = plot(thetai_rs,ri_rs,'LineStyle','-','Color', [0.5 0 0.5],'Marker','s','Tag','ResampledOriPlot');
		
	Xlim = get(gca,'Xlim');
	Ylim = get(gca,'Ylim');
	set(gca,'Xlim',[0 180],'Ylim',[0 Ylim(2)]);

	if abs(rightshift)
		xlabel(['Degrees (shifted by ' int2str(rightshift) ' deg)'],'color','k');
	else
		xlabel('Degrees','color','k');
	end

	ylabel('Spikes/s','color','k');
	
	pref = peak_thetai-rightshift;
	if pref > 180,
		pref = pref -180;
	end
	title(['Pref: ' int2str(pref) '  CV: ' num2str(CV,2) '  bandwidth@1/sqrt(2): ' num2str(bw) ' deg']);
	


	% find preferred orientation using center of mass of the
	% region defined by polar plot (after Furman and Gur, submitted)
	
	theta_rad = (theta)*pi/180;
	
	x_int = 0;
	y_int = 0;
	% calculated the double integrals by sections;
	circ_directions_rad = [theta_rad theta_rad(1)];
	circ_responses = [r r(1)];
			  
	for sec_ind = 1:length(theta_rad)
      
		% find pair of angles, 
		t1 = circ_directions_rad(sec_ind);
		t2 = circ_directions_rad(sec_ind+1);
      
		if t2<t1
			t2 = t2+2*pi;
		end
      
		r1 = circ_responses(sec_ind);
		r2 = circ_responses(sec_ind+1);
      
		x_int = x_int + ...
			-r1*sin(t1)+ r2*sin(t2) + ...
			(r2-r1)*(cos(t2) - cos(t1))/(t2-t1);
          
		y_int = y_int + ...
			r1*cos(t1) - r2*cos(t2) + ...
			(r2-r1)*(sin(t2) - sin(t1))/(t2-t1);
      
	end
   
	pref_dir_1 = atan2(y_int,x_int)- rightshift/180*pi;

	if pref_dir_1 <0 
		pref_dir_1 = pref_dir_1+2*pi;
	end

	subplot(1,2,2)
	polar(circ_directions_rad,circ_responses,'k-');
	
	[theta_pref,r_pref] = cart2pol(x_int,y_int);
	hold on
	polar(theta_pref,r_pref,'r.');
	line([0 x_int],[0 y_int],'Color','r');
	hold on
	
	[x,y] = pol2cart(circ_directions_rad(1:end-1),circ_responses(1:end-1) );
	[x_c,y_c] = comass(x,y);
	plot(x_c,y_c,'b.');
	hold on
	plot(x,y,'k.')
	line([0 x_c],[0 y_c],'Color','b','LineWidth',1.5);

	pref_dir_2 = atan2(y_c,x_c) - rightshift/180*pi;

	if pref_dir_2 <0 
		pref_dir_2 = pref_dir_2+2*pi;
	elseif pref_dir_2 > pi
		pref_dir_2 = pref_dir_2-pi;
	end
	
	title(['pref. orient. as center-of-mass: ' int2str(pref_dir_2/pi*180) ' (' int2str(pref_dir_1/pi*180) ')' ])  

	unitret('display_message',sprintf('%d \t %.2f \t %.1f',pref,CV,bw));
	
	if DefValues(160)
		excelclipcopy([pref CV bw],0);
	end
	
        %-------------------------------------------
elseif(strcmp(action,'tuning_width')),
        
        x = in1(:,1);
        y = in1(:,2);
        
        lengthy=length(y);
        [YMaxVal, MaxIntInd] = max(y);
        XMaxVal = x(MaxIntInd);
        maxind = find((x==XMaxVal) & ((y==YMaxVal))); 
        YBound = YMaxVal/2;
        
        LeftInd = max(find((x < x(maxind)) & ( y <= YBound)));
        if isempty(LeftInd), 
                xx1 = min(x) ;
        else
                CrossPoints = [x(LeftInd), y(LeftInd);x(LeftInd+1), y(LeftInd+1);];                          
                xx1 = process('linear_X',CrossPoints,YBound); 
                %						   hold on; plot([x(LeftInd) x(LeftInd+1)],[y(LeftInd), y(LeftInd+1)],'rx', xx1, YBound);
        end;
        
        RightInd = min(find((x > x(maxind)) & ( y <= YBound)));
        if isempty(RightInd),
                xx2 =max(x); 
        else
                CrossPoints = [x(RightInd), y(RightInd);x(RightInd-1), y(RightInd-1);];
                xx2 = process('linear_X',CrossPoints,YBound); 
                %                            hold on; plot([x(RightInd) x(RightInd-1)],[y(RightInd), y(RightInd-1)],'bo',xx2, YBound);
        end;
        
        out1 = [xx1 xx2 YBound];
        % disp(YMaxVal)
        %--------------------------------------------------------------------------------------------------------------------------------------------------------------
        
elseif(strcmp(action,'linear_X')),
        
        x1 = in1(1,1);
        y1 = in1(1,2);
        x2 = in1(2,1);
        y2 = in1(2,2);
        y  = in2;
        out1 = (y-y1)*(x2-x1)/(y2-y1)+x1;
        %-------------------------------------------
        
        
        % TEMPORARY ADDITION
        %----------------------------------------------------------------
        
elseif strcmp(action,'average_conc'),
        global AV_1 AV_2
        tit = str2mat('Aver. resp','Concatenated'); 
        for i=1:2,
                s(i,:) = ['CurSpk = []; CurSpk =in',num2str(i), ';'];
        end;
        
        
        
        
        if DefValues(35),			%Separate Drawing
                
                for m=1:2,	% MODALITY COUNTER
                        eval(s(m,:));
                        if (sum(size(CurSpk))>2),			% in1 or in2 == 0
                                vect=process('average_spk_calculation',CurSpk);
                                [rate,ifr] = process('mean_rate_calculation',CurSpk);
                                
                                %velocity = process('average_vel_calculation',CurSpk);
                                velocity=69; 
                                
                                xmin = 0;
                                xmax = max(CurSpk(:,4) - CurSpk(:,3));
                                
                                axpos = [.075+0.5*(~(m-1)) .12 .4 .1];
                                
                                set(axes,'Position',axpos,'DrawMode','fast','TickDir','out','XLimMode',...
                                        'Manual','XLim',[xmin xmax]);
                                h=get(gca,'title'); 
                                
                                if length(vect)>0
                                        bins=(xmin+DefValues(20)/2:DefValues(20):xmax-DefValues(20)/2);
                                        [N,SpikeHist]=hist(vect,bins);
                                        [xx,yy] = bar(SpikeHist,N);
                                        com = ['AV_',num2str(m), '= [SpikeHist; N];'];               
                                        eval(com) ;   
                                        
                                        set(line,'Xdata',xx,'YData',yy,'Color','b');
                                        %set(line,'Xdata',xx,'YData',yy/(size(CurSpk,1)*DefValues(20)*0.001),'Color','b');
                                        
                                        ylim=get(gca,'YLim');
                                        xlim=get(gca,'XLim');
                                        
                                        if DefValues(41),          			%if autoscale   
                                                drawYlims = [0,ylim(2)];
                                        else,
                                                drawYlims = [0,DefValues(42)];		%manual scale
                                        end;
                                        
                                        if DefValues(43),          			%if autoscale   
                                                drawXlims = [0,xlim(2)];
                                        else,
                                                drawXlims = [DefValues(44),DefValues(45)];	%manual scale
                                        end;
                                        
                                        set(gca,'XLim',drawXlims,'YLim',drawYlims);
                                        
                                end  
                                
                                
                                if isempty(h)
                                        h = text('horiz','center');
                                        set(gca,'title',h);
                                end;
                                str=sprintf('%s, %5.1f spks, %5.2f spk/s',deblank(tit(m,:)), length(vect)/size(CurSpk,1),rate);
                                
                                set(h,'string',str, 'color',[0.5020 0 0.2510],'FontSize',9,'FontWeight','demi');
                                
                                AccDrawMenu= get(TRET_DAT(7),'UserData');
                                AccAverageDraw = get(AccDrawMenu(2),'Checked');
                                if strcmp(AccAverageDraw,'on'),    % accumulation flag turned 'on'
                                        unitret('redraw_accumulation',sort(vect));
                                end;
                                
                        end;
                end;  
                
                
        else				%draw them together
                %%%%%%%%%%%%%%%%%%%%%%%%%%%
                Spk=[];
                
                for m=1:2,	% MODALITY COUNTER           
                        eval(s(m,:));
                        if  max(size(CurSpk)) ~= 1,		%nonzero CurSpk
                                Spk=[Spk;CurSpk];
                        end;  
                end;
                CurSpk=Spk;
                
                vect=process('average_spk_calculation',CurSpk);
                [rate,ifr] = process('mean_rate_calculation',CurSpk);
                
                %velocity = process('average_vel_calculation',CurSpk); 
                velocity=69;
                
                xmin = 0;
                xmax = max(CurSpk(:,4) - CurSpk(:,3));
                axpos = [.3 .12 .4 .1];
                set(axes,'Position',axpos,'DrawMode','fast','TickDir','out','XLimMode',...
                        'Manual','XLim',[xmin xmax],'UserData',velocity);
                h=get(gca,'title'); 
                
                
                if length(vect)>0
                        bins=(xmin+DefValues(20)/2:DefValues(20):xmax-DefValues(20)/2);
                        [N,SpikeHist]=hist(vect,bins);
                        [xx,yy] = bar(SpikeHist,N);
                        %             com = ['AV_',num2str(0), '= [SpikeHist; N];'];               
                        %             eval(com) ;               
                        %             set(line,'Xdata',xx,'YData',yy/size(CurSpk,1),'Color','b');
                        set(line,'Xdata',xx,'YData',yy/(size(CurSpk,1)*DefValues(20)*0.001),'Color','b');
                        ylim=get(gca,'YLim');
                        xlim=get(gca,'XLim');
                        
                        if DefValues(41),          			%if autoscale   
                                drawYlims = [0,ylim(2)];
                        else,
                                drawYlims = [0,DefValues(42)];		%manual scale
                        end;
                        
                        if DefValues(43),          			%if autoscale   
                                drawXlims = [0,xlim(2)];
                        else,
                                drawXlims = [DefValues(44),DefValues(45)];	%manual scale
                        end;
                        
                        set(gca,'XLim',drawXlims,'YLim',drawYlims);
                        
                        if isempty(h)
                                h = text('horiz','center');
                                set(gca,'title',h);
                        end;
                        str=sprintf('%s, %5.1f spks, %5.2f spk/s','On and Off', length(vect)/size(CurSpk,1),rate);
                        set(h,'string',str, 'color',[0.5020 0 0.2510],'FontSize',9,'FontWeight','demi');
                        
                end  
                
                
                AccDrawMenu= get(TRET_DAT(7),'UserData');
                AccAverageDraw = get(AccDrawMenu(2),'Checked');
                if strcmp(AccAverageDraw,'on'),    % accumulation flag turned 'on'
                        unitret('redraw_accumulation',sort(vect));
                end;
                
                
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
elseif strcmp(action,'harmonic_anal')
        global AV_1 AV_2 AVC SINDATA
        
        bc = get(gcf, 'Color');
        
        if in1==0 % on PSTH
                for i = 2:2,      
                        com = ['data = AV_', num2str(i),';'];
                        eval(com);
                        [m,n] = size(data);
                        if n > 0,
                                
                                SpecLength = fix(n/2);
                                T=(data(1,2) - data(1,1))/1000; % s!
                                F = 1/T;
                                Fdisplay = DefValues(53);
                                FreqAxis = (0:n-1)*F/n;
                                y =  (fft(data(2,:)));
                                
                                len_y=length(y); 
                                
                                FdisplayInd = find(FreqAxis == Fdisplay);
                                
                                % check if Fdisplay does not exceed len_y:
                                if (Fdisplay-1) > len_y,
                                        errordlg('Fdisplay is too large!');
                                        return;              
                                end
                                
                                a0=2*y(1)/len_y;
                                DC=a0/2;
                                
                                
                                if DefValues(54) % BaseLineActivity (BLA)  
                                        
                                        % Calculation by simple subtraction
                                        DC=DC-DefValues(54)*1000; % in spikes/s
                                end
                                
                                for i=1:FdisplayInd
                                        ak=2*real(y(i+1))/len_y;
                                        bk=2*imag(y(i+1))/len_y;
                                        
                                        AC(i)=(ak^2+bk^2)^.5;
                                end
                                
                                TF = DefValues(75); 
                                ph =( angle(y) + pi/2)*360/(2*pi);
                                F1ind = find(abs(FreqAxis-TF) == min(abs(FreqAxis-TF)));
                                F1m = AC(F1ind-1);
                                ratio=F1m/DC;
                                F1p = ph(F1ind);
                                
                                AC = AC;
                                DC = DC;
                                
                                unistem(0,DC,'r'),hold on
                                % stem(-1,DC,'g')
                                unistem(AC,'b')
                                hold off
                                FreqInd=length( str2num(get(gca,'XTickLabels')) );
                                
                                
                                
                                
                        end
                        
                end
        elseif in1 == 1 % on Raw Concatenated
                [Trials,m] = size(in2);
                % N = 2^(nextpow2(m));
                Fsample = 1000;
                Fdisplay = DefValues(53);
                %FreqAxis = (0:512/2-1)*Fsample/512;
                FreqAxis = (0:m-1)*Fsample/m; 
                
                FdisplayInd = find(FreqAxis == Fdisplay);
		if isempty(FdisplayInd)
			FdisplayInd = find(abs(FreqAxis - Fdisplay) == min(abs(FreqAxis - Fdisplay)));
		end
                FdisplayInd = FdisplayInd(1); 
                for i =1:1 %Trials,
                        y =  (fft(in2(i,:)));
                        f=(0:length(y)-1)'*1000/length(y);
                        a0=2*y(1)/length(y);
                        DC=a0/2;
                        
                        if DefValues(54) % BaseLineActivity (BLA)  
                                % Calculation via fft on BLA  
                                %BLA=DefValues(54)*ones(1,length(data(2,:)));
                                %BLAS=abs(fft(BLA));
                                %y=y-BLAS;
                                
                                % Calculation by simple subtraction
                                DC=DC-DefValues(54); 
                        end
                        
                        for i=1:FdisplayInd
                                ak=2*real(y(i+1))/length(y);
                                bk=2*imag(y(i+1))/length(y);
                                
                                AC(i)=(ak^2+bk^2)^.5;
                        end
                        
                        TF = DefValues(75); 
                        ph =( angle(y) + pi/2)*360/(2*pi);
                        F1ind = find(abs(FreqAxis-TF) == min(abs(FreqAxis-TF))); 
                        if (F1ind-1) > FdisplayInd,
                                unitret('errormessage',['Fdisplay is too small!'],'ERROR');
                                return;
                        end
                        
                        AC = 1000*AC;
                        DC = 1000*DC;
                        
                        unistem(0,DC,'r'),hold on
                        plot(AC,'b')
                        
                 
                        hold off
                        
                        
                end;
                
        elseif (in1==2) % Harmonic Analysis on Average
                
                if isempty(AVC),
                        set(gca,'visible','off')
                        text(.1, .4,'NO AVERAGED DATA...','Color','r','FontSize',[18])
                        %uicontrol(gcf,'Style','text','Position',[.1 .4 .8 .5],...
                        %   'Units','normalized','BackgroundColor','w',...
                        %   'ForegroundColor','r','String','NO AVERAGED DATA...','HorizontalAlignment','center','FontSize',[18]);
                        
                        return
                end
                
                data=AVC;
                
                [m,n] = size(data);
                if n > 0,
                        
                        SpecLength = fix(n/2);
                        T=(data(1,2) - data(1,1))/1000; % s!
                        F = 1/T;
                        Fdisplay = DefValues(53);
                        FreqAxis = (0:n-1)*F/n;
                        y = (fft(data(2,:)));
                        FdisplayInd = find(FreqAxis == Fdisplay);
                        
                        if DefValues(54) % BaseLineActivity (BLA) 
                                % Calculation via fft       
                                BLA=DefValues(54)*ones(1,length(data(2,:)));
                                BLAS=(fft(BLA));
                                y=y-BLAS;
                        end
                        
                        len_y=length(y); 
                        a0=2*y(1)/len_y;
                        DC=a0/2;
                        
                        % check if Fdisplay does not exceed len_y:
                        if (Fdisplay-1) > len_y,
                                errordlg('Fdisplay is too large!');
                                return;              
                        end
                        for i=1:FdisplayInd
                                ak=2*real(y(i+1))/len_y;
                                bk=2*imag(y(i+1))/len_y;
                                
                                AC(i)=(ak^2+bk^2)^.5;
                        end
                        
                        
                        TF = DefValues(75); 
                        ph =( angle(y) + pi/2)*360/(2*pi);
                        F1ind = find(abs(FreqAxis-TF) == min(abs(FreqAxis-TF)));
                        F1m = AC(F1ind-1);
                        ratio=F1m/DC;
                        F1p = ph(F1ind);
                        
                        AC =1000*AC;
                        DC =1000*DC;
                        unistem(0,DC,'r'),hold on
                        unistem(AC,'b')
                        hold off
                        FreqInd=length( str2num(get(gca,'XTickLabels')) );
                        
                        
                        
                end
                
        elseif (in1==3) % Harmonic Analysis on Raw (Concatenated): Hochstein
                
                [Trials,m] = size(in2);
                N = 2^(nextpow2(m));
                Fsample = 1000;
                Fdisplay = DefValues(53);
                FreqAxis = (0:512/2-1)*Fsample/512;
                
                r = find(FreqAxis <= Fdisplay);
                
                for i =1:1 %Trials,
                        y = (fft(in2(i,:)));
                        f=(0:length(y)-1)'*1000/length(y);
                        a0=2*y(1)/length(y);
                        DC=a0/2;
                        
                        if DefValues(54) % BaseLineActivity (BLA)  
                                % Calculation via fft   
                                %BLA=DefValues(54)*ones(1,length(data(2,:)));
                                %BLAS=abs(fft(BLA));
                                %y=y-BLAS;
                                
                                % Calculation by simple subtraction
                                DC=DC-DefValues(54); 
                        end
                        
                        for i=1:Fdisplay
                                ak=2*real(y(i+1))/length(y);
                                bk=2*imag(y(i+1))/length(y);
                                
                                AC(i)=(ak^2+bk^2)^.5;
                        end
                        
                        ratio=max(AC)/DC;
                        stem(0,DC,'r'),hold on
                        stem(AC)
                        hold off
                        
                        
                        
                end; 
                
        elseif in1 == 4 % Harmonic Analysis per each segment - average
                [nsegm,m] = size(in2);
                % N = 2^(nextpow2(m));
                Fsample = 1000;
                Fdisplay = DefValues(53);
                %FreqAxis = (0:512/2-1)*Fsample/512;
                FreqAxis = (0:m-1)*Fsample/m; 
                
                
                FdisplayInd = find(FreqAxis == Fdisplay);
                
                DCt = [];
                ACt = [];
                Pht = [];
                for i =1:nsegm
                        y =  (fft(in2(i,:)));
                        f=(0:length(y)-1)'*1000/length(y);
                        a0=2*y(1)/length(y);
                        DC=a0/2;
                        
                        if DefValues(54) % BaseLineActivity (BLA)  
                                % Calculation via fft on BLA  
                                %BLA=DefValues(54)*ones(1,length(data(2,:)));
                                %BLAS=abs(fft(BLA));
                                %y=y-BLAS;
                                
                                % Calculation by simple subtraction
                                DC=DC-DefValues(54); 
                        end
                        
                        for i=1:FdisplayInd
                                ak=2*real(y(i+1))/length(y);
                                bk=2*imag(y(i+1))/length(y);
                                
                                AC(i)=(ak^2+bk^2)^.5;
                        end
                        
                        TF = DefValues(75);
                        F1ind = find(abs(FreqAxis-TF) == min(abs(FreqAxis-TF)));  
                        
                        ph =( angle(y) + pi/2)*360/(2*pi);
                        
                        % unwrap only phase of first harmonic - Matlab unwrap bug!!!
                        
                        ph(F1ind) = ( unwrap(ph(F1ind)*2*pi/360) )*360/(2*pi);
                        
                        
                        if (F1ind-1) > Fdisplay,
                                unitret('errormessage',['Fdisplay is too small!'],'ERROR');
                                return;
                        end
                        DCt = [DCt; DC]; 
                        ACt = [ACt; AC];
                        Pht = [Pht; ph];
                        
                end % of for each segment
                DCt = 1000*DCt;
                ACt = 1000*ACt;
                
                DC = mean(DCt,1);
                AC = mean(ACt,1);
                ph = mean(Pht,1);
                
                DCstd = std(DCt);	
                ACstd = std(ACt);
                % phstd = std(Pht);	
                
                unistem(0,DC,'r'), hold on
                unistem(AC,'b'), hold on
                plot(0,DC+DCstd,'m+',0,DC-DCstd,'m+'), hold on
                plot(AC+ACstd,'m+'), hold on,
                plot(AC-ACstd,'m+'), 
                
                
                
                hold off
                
                
        elseif in1 == 5 % Harmonic Analysis per each segment - no averaging
                
                [nsegm,m] = size(in2);
                Fsample = 1000;
                Fdisplay = DefValues(53);
                
                spikes_inform = unitret('get_markeddata');
                
                
                nsegm2plot = 12;
                ncol	= 2;	
                nn 	= 1;
                
                for k =1:nsegm
                        
                        
                        SpkStarts = spikes_inform(k,3);
                        SpkEnds =  spikes_inform(k,4);
                        
                        y =  (fft(in2(k,1:SpkEnds-SpkStarts)));
                        
                        FreqAxis=(0:length(y)-1)*1000/length(y);
                        FdisplayInd = find(FreqAxis == Fdisplay);
                        
                        a0=2*y(1)/length(y);
                        DC=a0/2;
                        
                        if DefValues(54) % BaseLineActivity (BLA)  
                                % Calculation via fft on BLA  
                                %BLA=DefValues(54)*ones(1,length(data(2,:)));
                                %BLAS=abs(fft(BLA));
                                %y=y-BLAS;
                                
                                % Calculation by simple subtraction
                                DC=DC-DefValues(54); 
                        end
                        
                        for i=1:FdisplayInd
                                ak=2*real(y(i+1))/length(y);
                                bk=2*imag(y(i+1))/length(y);
                                
                                AC(i)=(ak^2+bk^2)^.5;
                        end
                        
                        TF = DefValues(75); 
                        ph =( angle(y) + pi/2)*360/(2*pi);
                        F1ind = find(abs(FreqAxis-TF) == min(abs(FreqAxis-TF))); 
                        if (F1ind-1) > Fdisplay,
                                unitret('errormessage',['Fdisplay is too small!'],'ERROR');
                                return;
                        end
                        
                        if nn <= nsegm2plot/2,
                                kk = 2*nn-1;
                        else
                                kk = 2*(nn - nsegm2plot/2);
                        end
                        subplot(nsegm2plot/2,ncol,kk);
                        
                        DC = 1000*DC;
                        AC = 1000*AC;
                        
                        
                        unistem(0,DC,'r'), hold on
                        unistem(AC,'b'), hold on
                        
                        AC1_2_DC = AC(F1ind-1)/DC;
                        ph_AC1 = ph(F1ind);
                        
                        % AXIS LABELS
                        XTick = 0:F1ind-1:FdisplayInd; 
                        %%% old version: XTickLabels = ([0:TF:round(Fdisplay/(F1ind-1))*TF]);
                        XTickLabels = ([0:TF:Fdisplay]);
                        set(gca, 'XTick', XTick, 'XTickLabels', XTickLabels);
                        text(0.1,5,['Hz.  AC(1)/DC = ',num2str(AC1_2_DC,2),'  AC(1) phase = ',num2str(ph_AC1,2)],'FontSize',8)
                        
                        set(gca,'Xlim',[0 FdisplayInd-1]) %%% limit by Fdisplay
                        
                        set(gca,'FontSize',[8],'XColor','k','YColor','k') 
                        %'Position',[0.1 0.38 0.8 0.57]
                        
                        nn = nn + 1;
                        if nn > nsegm2plot,
                                unitret('figname')
                                figure
                                nn = 1;
                        end
                end % of for each segment
                unitret('figname');        
                return           
                
        end % of if for which harmonic analysis should be made
        
        str1 = ['Magnitude '];
        str2 = ['Phase     '];
        str3 = ['Ratio     '];

	F1ind = F1ind(length(F1ind));
        for k = 1:min(floor(FdisplayInd/(F1ind-1)), 6),
                Fm(k) = AC((F1ind-1)*k);
                ratio(k) = Fm(k)/DC;
                Fp(k) = ph(F1ind*k);
                str1 = [str1,'  ',num2str(k),': ',num2str(Fm(k),2)];
                str2 = [str2,'  ',num2str(k),': ',num2str(Fp(k),3)];
                str3 = [str3,'  ',num2str(k),': ',num2str(ratio(k),2)];
        end
        
        % Maximal harmonic (besides DC)
	maxAC 	= max(AC);
	maxACind = find(AC==maxAC)+1;

        
        % AXIS LABELS
        XTick = 0:F1ind-1:FdisplayInd; 
        %%% old version: XTickLabels = ([0:TF:round(Fdisplay/(F1ind-1))*TF]);
        XTickLabels = ([0:TF:Fdisplay]);
        set(gca, 'XTick', XTick, 'XTickLabels', XTickLabels);
        xlabel('Hz')

        set(gca,'Xlim',[0 FdisplayInd-1]) %%% limit by Fdisplay
        
        set(gca,'Position',[0.1 0.38 0.8 0.57],'FontSize',[8],'XColor','k','YColor','k') 
        unitret('figname');
        
        % PRINTOUTS
        
        mainstr=['DC: ',num2str(DC),'  F1 (',num2str(FreqAxis(F1ind),3),'): ',num2str(Fm(1),2),' ',num2str(Fp(1),3),'  Fmax: ',num2str(FreqAxis(maxACind),3),'Hz [',num2str(maxAC,3),']'];
        
        
        set(text,'Position',[-.45 -.70],'String',str1,'Color','b','Fontsize',10,'FontName','Times','FontWeight','normal');
        set(text,'Position',[-.45 -.75],'String',str2,'Color','b','Fontsize',10,'FontName','Times','FontWeight','normal');
        set(text,'Position',[-.45 -.80],'String',str3,'Color','m','Fontsize',10,'FontName','Times','FontWeight','normal');
        set(text,'Position',[-.05 -.95],'String',mainstr,'Color','r','Fontsize',10,'FontName','Times','FontWeight','normal');
        set(text,'Position',[-.45 -.95],'String',['RM: ',num2str(ratio(1))],'Color','r','Fontsize',10,'FontName','Times','FontWeight','demi');
        
        if DefValues(54)
                set(text,'Position',[-.20 -.95],'String',['-BLA=',num2str(DefValues(54))],'Color',[.3 .4 .1],'Fontsize',10,'FontName','Times','FontWeight','demi');
	end

	Ph1std = 0;
        if in1 == 4,
                set(text,'Position',[-.45 -.85],'String',[num2str(nsegm),' segm., AC(1) magn. std ', num2str(ACstd(F1ind-1),2),' |  ',num2str( [ACt(:,F1ind-1)]',2 )],'Color','k','Fontsize',10,'FontName','Times','FontWeight','normal');       
                
                % Phase analysis
                Ph1 = Pht(:,F1ind);
                
                % [x,y] = pol2cart( Ph1*2*pi/360,ones(size(Ph1)) );
                % Ph1std = std(x) + std(y);
                
                PhSort = sort(Ph1);
                NPh = length(PhSort);
                VectPhStd = zeros(size(PhSort));
                VectPhStd(1) = std(PhSort);
                Ph = PhSort;
                for n=2:NPh,
                        Ph = [Ph(2:NPh); Ph(1)+360];
                        VectPhStd(n) = std(Ph);		
                end
                
                Ph1std = min(VectPhStd);
		% std( unwrap(Ph1*2*pi/360, pi/2) )*360/(2*pi);		
                               
                set(text,'Position',[-.45 -.90],'String',['AC(1) phase std ', num2str(Ph1std,2),' |   ',num2str( Ph1',2 )],'Color','k','Fontsize',10,'FontName','Times','FontWeight','normal');       
                figure
                subplot(2,1,1)
                plot([1:length(Ph1)],Ph1,'.');
                xlabel('#Cycle')
                ylabel('AC(1) phase (deg)')
                grid
                subplot(2,1,2)
                polar(Ph1*2*pi/360,ones(size(Ph1)),'.')
                
                
                %set(text,'Position',[-.45 -.90],'String',['AC(1) phase std ', num2str(phstd(F1ind),2),' |  ',num2str( [Pht(:,F1ind)]',2 )],'Color','k','Fontsize',10,'FontName','Times','FontWeight','normal');       
	end            
	out1.rm = ratio(1);
	out1.phase = Ph1std;
	out1.dc	= DC;
	out1.f = Fm;
	out1.max_harmonic = FreqAxis(maxACind);
	out1.max_harmonic_amp = maxAC;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %---------------------------------------------------------------------------------------
elseif strcmp(action,'SF_tuning')
        Fig= figure('Backingstore','off','Name',['SF tuning curve for ',in1],'Color','w',...
                'Units','Normal','Position',[.22 .3 .3 .3],'Resize','On', 'Menubar','none','Number','off');
        set(gcf,'Pointer','Watch');
        set(gcf,'WindowButtonDownFcn','ud = get(0,''Userdata'');ud(72) = gcf;set(0,''Userdata'',ud);');
        
        
        SFHndl=uicontrol(Fig,'style','edit','String','1,2,3,5','Units','normalized','Position',[.05 .85 .5 .1],'ForegroundColor','b','BackgroundColor',[0.996 0.9255 0.7373],...
                'Callback','process(''update_tuning'',1)');     
        RFHndl=uicontrol(Fig,'style','edit','String',' , , , ','Units','normalized','Position',[.05 .7 .5 .1],'ForegroundColor',[ 0 0.2510 0.5020],'BackgroundColor',[0.996 0.9255 0.7373],...
                'Callback','process(''update_tuning'',2)');  
        
        
        
        OK_button=uicontrol(Fig,'Style','pushbutton','Position',[.75 .8 .22 .1],...
                'Units','Normalized','String','OK','Userdata',SFHndl,'Callback',['process(''update_tuning'',1,get(gco,''Userdata''));delete(',num2str(Fig ), ');  process(''draw_tuning_curve'');']);
        
        cancel=uicontrol(Fig,'Style','pushbutton','Position',[.75 .62 .22 .1],...
                'Units','Normalized','String','Cancel','Callback',['delete(',num2str(Fig ), '), return']);
        
        
        set(Fig,'Visible','on','Pointer','arrow');
        
        %=====================================================================================================================
        
elseif strcmp(action,'draw_tuning_curve')
        
        Fig= figure('Backingstore','off','Name',['SF tuning curve'],...
                'Units','Normal','Position',[.22 .4 .3 .6],'Resize','On', 'Menubar','none','Number','off');
        set(gcf,'WindowButtonDownFcn','ud = get(0,''Userdata'');ud(72) = gcf;set(0,''Userdata'',ud);');
        halfAmp=(max(RF)+min(RF))/2;
        maxSF=find(max(RF)==RF);
        line([ (min(SF))  (max(SF))],[ halfAmp halfAmp],'Color','b'); hold on
        loglog(SF,RF)
        xlabel('SF (cycles/degree)')
        ylabel('Cell Response')
        set(gca,'XTick',SF,'YTick',[min(RF)-1 RF' max(RF)+1])
        
        %=====================================================================================================================
        
elseif strcmp(action,'update_tuning')
        if nargin==2,
                h=get(gcf,'CurrentObject');  
        else 
                h=in2;
        end
        
        newdata = get(h,'String'); 
        comma=find(newdata==',');
        if (~isempty(comma)),
                for i=1:length(comma),
                        newdata(comma(i)) = ' ';
                end;   
        end;
        data = sscanf(newdata,'%f');
        
        if in1==1, %SF
                SF= data;
        else % RF
                RF= data;
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %---------------------------------------------------------------------------------------
elseif strcmp(action,'OI_setup')
        
        ud=get(0,'Userdata');
        ud(69)=1; % Index Mode on
        set(0,'Userdata',ud);
        
        
        
        if ud(68) % INC
                pol='selecting INC';
        else
                pol='selecting DEC';
        end
        
        
        fc=[0.5 0  1];
        bc=[0.7059 0.7059 0.8549]; % Base Color
        mode = 'off';

	h = findobj('Tag','OI setup');
	if ~isempty(h);
		delete(h);
	end
	
        Fig = figure('Backingstore','off','Name',['OI for ',in1],'Color',bc,...
                'Units','Normal','Position',[.4 .4 .3 .4],'Resize','On', 'Menubar','none','Number','off','Tag','OI setup');
        set(gcf,'Pointer','Watch');
        
        PolSign=uicontrol(Fig,'Style','text','Position',[.7 .02 .3 .05],...
                'Units','normal','ForegroundColor',fc,...
                'BackgroundColor',bc,'String',[pol],...
                'HorizontalAlignment','center');
        
        IncHndl=uicontrol(Fig,'Style','pushbutton','Position',[.37 .77 .1 .05],...
                'Units','normal','ForegroundColor','w',...
                'BackgroundColor',bc,'String','INC',...
                'HorizontalAlignment','center','Callback',...
                ['ud=get(0,''Userdata''); ud(68)=1; set(0,''Userdata'',ud); PolSign=get(gco,''Userdata''); set(PolSign,''string'',''selecting INC'');']);
        
        DecHndl=uicontrol(Fig,'Style','pushbutton','Position',[.37 .57 .1 .05],...
                'Units','normal','ForegroundColor','k',...
                'BackgroundColor',bc,'String','DEC',...
                'HorizontalAlignment','center','Callback',...
                ['ud=get(0,''Userdata''); ud(68)=0; set(0,''Userdata'',ud); PolSign=get(gco,''Userdata''); set(PolSign,''string'',''selecting DEC'');']);
        
        set(IncHndl,'Userdata',PolSign);
        set(DecHndl,'Userdata',PolSign);
        
        if sum(DefValues(64:67)), % Borders were calculated already
                
                mode = 'on';
                
                u1=uicontrol(Fig,'style','edit','String',num2str(DefValues(64),3),'Units','normalized',...
                        'Position',[.1 .80 .2 .08],'ForegroundColor','b','BackgroundColor','w',...
                        'Callback',['ud=get(0,''Userdata''); ud(64)=str2num(get(gco,''string'')); set(0,''UserData'',ud);']);
                
                u2=uicontrol(Fig,'style','edit','String',num2str(DefValues(65),3),'Units','normalized',...
                        'Position',[.1 .70 .2 .08],'ForegroundColor','b','BackgroundColor','w',...
                        'Callback',['ud=get(0,''Userdata''); ud(65)=str2num(get(gco,''string'')); set(0,''UserData'',ud);']);
                
                u3=uicontrol(Fig,'style','edit','String',num2str(DefValues(66),3),'Units','normalized',...
                        'Position',[.1 .60 .2 .08],'ForegroundColor','b','BackgroundColor','w',...
                        'Callback',['ud=get(0,''Userdata''); ud(66)=str2num(get(gco,''string'')); set(0,''UserData'',ud);']);
                
                u4=uicontrol(Fig,'style','edit','String',num2str(DefValues(67),3),'Units','normalized',...
                        'Position',[.1 .50 .2 .08],'ForegroundColor','b','BackgroundColor','w',...
                        'Callback',['ud=get(0,''Userdata''); ud(67)=str2num(get(gco,''string'')); set(0,''UserData'',ud);']);

		u5=uicontrol(Fig,'style','edit','String',num2str(DefValues(105),3),'Units','normalized',...
                        'Position',[.1 .30 .1 .08],'ForegroundColor','b','BackgroundColor','w',...
                        'Callback',['ud=get(0,''Userdata''); ud(105)=str2num(get(gco,''string'')); set(0,''UserData'',ud);']);

		u5=uicontrol(Fig,'style','edit','String',num2str(DefValues(173),3),'Units','normalized',...
                        'Position',[.3 .30 .1 .08],'ForegroundColor','b','BackgroundColor','w',...
                        'Callback',['ud=get(0,''Userdata''); ud(173)=str2num(get(gco,''string'')); set(0,''UserData'',ud);']);
				
                
        end
        
        %---
        
        %u5=uicontrol(Fig,'style','edit','String',num2str(DefValues(64)),'Units','normalized',...
        %  'Position',[.1 .80 .2 .08],'ForegroundColor','b','BackgroundColor','w',...
        %  'Callback',['ud=get(0,''Userdata''); ud(64)=str2num(get(gco,''string'')); set(0,''UserData'',ud);']);
        
        %u6=uicontrol(Fig,'style','edit','String',num2str(DefValues(65)),'Units','normalized',...
        %  'Position',[.1 .70 .2 .08],'ForegroundColor','b','BackgroundColor','w',...
        %  'Callback',['ud=get(0,''Userdata''); ud(65)=str2num(get(gco,''string'')); set(0,''UserData'',ud);']);
        
        %u7=uicontrol(Fig,'style','edit','String',num2str(DefValues(66)),'Units','normalized',...
        %  'Position',[.1 .60 .2 .08],'ForegroundColor','b','BackgroundColor','w',...
        %  'Callback',['ud=get(0,''Userdata''); ud(66)=str2num(get(gco,''string'')); set(0,''UserData'',ud);']);
        
        %u8=uicontrol(Fig,'style','edit','String',num2str(DefValues(67)),'Units','normalized',...
        %  'Position',[.1 .50 .2 .08],'ForegroundColor','b','BackgroundColor','w',...
        %  'Callback',['ud=get(0,''Userdata''); ud(67)=str2num(get(gco,''string'')); set(0,''UserData'',ud);']);
        
        %---------------------
        set(gca,'visible','off')
	
        calcstr=['ud=get(0,''Userdata''); x1=ud(64); x2=ud(65); x3=ud(66); x4=ud(67);',...
			'vel = ud(173); bw = ud(105); x1=x1*vel/1000; x2=x2*vel/1000; x3=x3*vel/1000; x4=x4*vel/1000;',...
			'bwd = 2; x1 = x1+0.5*(bw/bwd); x2 = x2-0.5*(bw/bwd); x3 = x3+0.5*(bw/bwd); x4 = x4-0.5*(bw/bwd);',... 
                        'INC=x2-x1; DEC=x4-x3; m1=x1+INC/2; m2=x3+DEC/2; sep=abs(m1-m2);',...
                        'OI=(0.5*(INC+DEC)-sep)/(0.5*(INC+DEC)+sep); CRF=max([0.5*INC + sep + 0.5*DEC INC DEC]); ow = INC+DEC-CRF;',...
                        'cla; text(.1, .10, [''OI = '',num2str(OI),sprintf(''\n''),',...
			'''inc '',num2str(INC),'' dec '',num2str(DEC),sprintf(''\n''),''CRF '',num2str(CRF),'' ow '',num2str(ow)],''FontSize'',12,''Color'',''b'');',...
			'if ud(160), excelclipcopy([INC DEC CRF OI],0); end;'];
        
        OK_button=uicontrol(Fig,'Style','pushbutton','Position',[.7 .8 .25 .1],...
                'Units','Normalized','String','Calculate','Callback',calcstr,'enable',mode);
        
        cancel=uicontrol(Fig,'Style','pushbutton','Position',[.7 .65 .25 .1],...
                'Units','Normalized','String','Close','Callback',['delete(',num2str(Fig), ');',...
                        'ud=get(0,''Userdata''); ud(69)=0; set(0,''Userdata'',ud); return']);
        
        clear_button=uicontrol(Fig,'Style','pushbutton','Position',[.7 .5 .25 .1],...
                'Units','Normalized','String','Clear','enable',mode,'Callback',...
                ['u=get(gco,''Userdata''); ud=get(0,''Userdata''); for i=1:4, set(u(i),''string'',''0''); ud(63+i)=0; ,end; set(0,''UserData'',ud);']);
        
        load_button=uicontrol(Fig,'Style','pushbutton','Position',[.7 .35 .25 .1],...
                'Units','Normalized','String','Load','Callback',['delete(',num2str(Fig), ');',...
                        'process(''OI_setup'',''',in1,''');'],'enable','on');
        
        set(clear_button,'Userdata',[u1 u2 u3 u4 u5]);
        
        
        
        
        set(gcf,'Pointer','Arrow');
        set(TRET_DAT(1),'Pointer','Arrow');
        set(0,'CurrentFigure',TRET_DAT(2));
        
        % Enable Accumulation
        AccDrawMenu= get(TRET_DAT(7),'UserData');
        if strcmp('off',get(AccDrawMenu(2),'Checked')),
                set(AccDrawMenu(2),'Checked','on');
                unitret('redraw_spk',1);
        end
        set(0,'CurrentFigure',Fig);
        %---------------------------------------------------------------------------------------
elseif strcmp(action,'Segments_Statistics')
        
        Spk = unitret('get_markeddata');
        bc = 'w'; %[0.2510    0.3333    0.5020];
        SegNum = size(Spk, 1);
        
        Fig = figure('Backingstore','off','Name',['Segments statistics for ',in1],'Color',bc,...
                'Units','Normal','Position',[.1 .1 .8 .8],'Resize','On', 'Menubar','none','Number','off');
        set(gcf,'Pointer','Watch');
        set(gcf,'WindowButtonDownFcn','ud = get(0,''Userdata'');ud(72) = gcf;set(0,''Userdata'',ud);');
        set(gca,'Visible','off')
        
        OutStat = ['========= Segments Statistics =========', sprintf('\n')];
        OutStat = [OutStat, ['File: ', in1], sprintf('\n')];
        
        ISItotal = [];
        for i=1:SegNum,
                SpikesNum(i) = Spk(i,2); 
                TrialNum(i)  = Spk(i,1);
                To(i)        = Spk(i,3); 
                Tend(i)      = Spk(i,4); 
                SpikesData   = Spk(i,6:SpikesNum(i)+5); 
                ISI          = diff(SpikesData); 
                ISItotal     = [ISItotal ISI];
                meanISIs(i)   = mean(ISI);
        end
        
        OutStat = [OutStat, 'Segments: ', num2str(SegNum), '  events: ', num2str(sum(SpikesNum)), sprintf('\n')];
        
        
        % COUNTS
        mean_count = mean(SpikesNum);
        var_count  = var(SpikesNum);
        
        % ISI total 
        mean_ISI = mean(ISItotal);
        SD_ISI   = std(ISItotal);
        var_ISI  = var(ISItotal);
        CV_ISI   = SD_ISI/mean_ISI;
        
        % ISI means
        % Here we take mean and  var of ISIs in each segment, and then we calculate
        % mean(meanISI) and var(meanISI)
        
        mean_ISIs = mean(meanISIs);
        var_ISIs  = var(meanISIs);
        
        
        %OutStat = [OutStat, 'Counts: mean ', num2str(mean_count),'  variance ', num2str(var_count), sprintf('\n')];
        %OutStat = [OutStat, 'ISI total: mean ', num2str(mean_ISI),'  SD ', num2str(SD_ISI),'  variance ',num2str(var_ISI),'  CV ',num2str(CV_ISI), sprintf('\n')];
        %OutStat = [OutStat, 'mean ISIs: mean ', num2str(mean_ISIs),'  variance ', num2str(var_ISIs), sprintf('\n')];
        
        %uout=uicontrol(Fig,'style','edit','String',OutStat,'Units','normalized','Max', 10, ...
        %	  'Position',[.05 .05 .5 .3],'ForegroundColor','b','BackgroundColor','w');
        
        CountStr    = ['Counts: mean ', num2str(mean_count),'  variance ', num2str(var_count)];
        ISItotalStr = ['ISI total: mean ', num2str(mean_ISI),'  SD ', num2str(SD_ISI),'  variance ',num2str(var_ISI),'  CV ',num2str(CV_ISI)];
        ISImeanStr  = ['mean ISIs: mean ', num2str(mean_ISIs),'  variance ', num2str(var_ISIs)];
        
        %%% TEXT %%%
        set(text,'Position', [-0.08 .3], 'String', '============= Segments Statistics =============', 'Color',[0 0.2510 0.5020], 'FontSize', 10, 'FontName','Arial');
        set(text,'Position', [-0.08 .25], 'String', ['Segments: ', num2str(SegNum), '  events: ', num2str(sum(SpikesNum))], 'Color','b', 'FontSize', 10, 'FontName','Arial');
        set(text,'Position', [-0.08 .2], 'String', CountStr, 'Color','b', 'FontSize', 10, 'FontName','Arial');
        set(text,'Position', [-0.08 .15], 'String',ISItotalStr, 'Color','b', 'FontSize', 10, 'FontName','Arial');
        set(text,'Position', [-0.08 .1], 'String', ISImeanStr, 'Color','b', 'FontSize', 10, 'FontName','Arial');
        
        
        close_bttn=uicontrol(Fig,'Style','pushbutton','Position',[.8 .05 .15 .05],...
                'Units','Normalized','String','Close','Callback',['delete(',num2str(Fig), ');']);
        
        set(gcf,'Pointer','Arrow');
        process('ISI_hist',ISItotal);
        
        %%% TEMP sort(ISItotal)
        
        
        % Raster Plot
        axes('Position',[0.55 0.5 0.4 0.4])
        
        process('raster',NAME,0);
        
        unitret('figname')
        
        %---------------------------------------------------------------------------------------
elseif strcmp(action,'ISI_hist')
        
        ISItotal = in1;
        % plot Frequency Histogram for ISI 
        %ISIplotHndl = figure('Backingstore','off','Name',['ISI Frequency histogram for ',in1],'Color',bc,...
        %		    'Units','Normal','Position',[.1 .5 .5 .6],'Resize','On', 'Menubar','none','Number','off');
        %set(gcf,'WindowButtonDownFcn','ud = get(0,''Userdata'');ud(72) = gcf;set(0,''Userdata'',ud);');
        
        
        ax1 = axes('position',[0.1 .5 0.4 0.4]);
        set(gca,'Userdata',ISItotal);
        
        ISIbin = DefValues(87);
        % NBins = 10;
        
        if length([0+ISIbin/2:ISIbin:max(ISItotal)-ISIbin/2]) > 2,
                [H,bins] = hist(ISItotal,[0+ISIbin/2:ISIbin:max(ISItotal)-ISIbin/2]);
        else
                ISIbin = 1;
                [H,bins] = hist(ISItotal,[0+ISIbin/2:ISIbin:max(ISItotal)-ISIbin/2]);
        end
        
        
        [XX,YY] = bar(bins,H);
        plot(XX,YY,'k');
        
        
        % axis([0 max(bins)+1 0 max(H)+1])
        
        
        if DefValues(88),
                xmin = DefValues(88);
        else
                xmin = 0;
        end
        
        if DefValues(89),
                xmax = DefValues(89);
        else
                xmax = max(ISItotal);
        end
        
        set(gca,'XColor','k','YColor','k','FontSize',8,'Ylim',[0 max(H)+1],'Xlim',[xmin xmax]);
        
        
        
        ISIbin_Hndl = uicontrol(gcf,'Style','edit','Position',[.8 .15 .05 .03],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','m',...
                'Callback',['ISIbin = (get(gco,''string'')); ud = get(0,''Userdata''); ud(87) = str2num(ISIbin); set(0,''Userdata'',ud); ax = get(gcf,''Userdata''); ISItotal = get(ax(1),''Userdata''); delete(ax(1)); process(''ISI_hist'',ISItotal);']);
        
        Xlim_1_Hndl = uicontrol(gcf,'Style','edit','Position',[.8 .25 .05 .03],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','b',...
                'Callback',['Xlim_1 = (get(gco,''string'')); gca = get(gcf,''Userdata''); XlimOld =  get(gca(1),''Xlim''); set(gca(1),''Xlim'',[str2num(Xlim_1)   XlimOld(2)] ); ud=get(0,''UserData''); ud(88) = str2num(Xlim_1); set(0,''Userdata'',ud);']);
        Xlim_2_Hndl = uicontrol(gcf,'Style','edit','Position',[.8 .2 .05 .03],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','b',...
                'Callback',['Xlim_2 = (get(gco,''string'')); gca = get(gcf,''Userdata''); XlimOld =  get(gca(1),''Xlim''); set(gca(1),''Xlim'',[XlimOld(1) str2num(Xlim_2)  ] ); ud=get(0,''UserData''); ud(89) = str2num(Xlim_2); set(0,''Userdata'',ud);']);
        
        
        %close_bttn=uicontrol(ISIplotHndl,'Style','pushbutton','Position',[.8 .02 .2 .1],...
        %	'Units','Normalized','String','Close','Callback',['delete(',num2str(Fig), ');']);
        
        xlabel(['ISI (ms), bin ',num2str(ISIbin),' ms'], 'Color', 'k')
        ylabel('Frequency', 'Color', 'k')
        title(['N = ', num2str(length(ISItotal)),' intervals'], 'Color', 'k')
        
        set(gca, 'Userdata', ISItotal);
        ud = get(gcf,'Userdata');
        ud(1) = gca;
        set(gcf, 'Userdata', ud);
        
        
        
        %---------------------------------------------------------------------------------------
elseif strcmp(action,'raster')
        Spk = unitret('get_markeddata');
        bc ='w';
        SegNum = size(Spk, 1);
        
        if in2 == 1, % new figure
                Fig = figure('Backingstore','off','Name',['Raster plot for ',in1],'Color',bc,...
                        'Units','Normal','Position',[.3 .3 .6 .5],'Resize','On', 'Menubar','none','Number','off','Renderer','painters','RendererMode','manual');
                set(gcf,'WindowButtonDownFcn','ud = get(0,''Userdata'');ud(72) = gcf;set(0,''Userdata'',ud);');
                axes('Position',[0.1 0.15 0.8 0.75]);
        end
        
        
        
        CurPos = 0.90; % Upper segment - first
        PosDec = (CurPos - 0.10)/SegNum;
        for i=1:SegNum,
                SpikesNum(i) = Spk(i,2); 
                TrialNum(i)  = Spk(i,1);
                To(i)        = Spk(i,3);
                Tend(i)      = Spk(i,4); 
                SpikesData   = Spk(i,6:SpikesNum(i)+5);
		make_raster(SpikesData - To(i),CurPos,PosDec-0.1*PosDec,0.1*PosDec,'LineWidth',1,'Color','k');
                % plot(SpikesData - To(i), CurPos*ones(1, SpikesNum(i)),'k.', 'MarkerSize', 10), hold on
		% text(SpikesData - To(i), CurPos*ones(1, SpikesNum(i)),'|','FontSize',5,'FontWeight','light'), hold on
                CurPos = CurPos - PosDec;
        end
        
        if DefValues(83),
                xmin = DefValues(83);
        else
                xmin = 0;
        end
        
        if DefValues(84),
                xmax = DefValues(84);
        else
                xmax = max(Tend-To);
        end
        
        
        set(gca, 'XColor', 'k', 'YColor', 'k','FontSize',8)
        xlabel('Time (ms)')
        set(gca, 'Xlim', [xmin xmax])
        set(gca, 'Ydir', 'normal')
        set(gca, 'YTick', CurPos + PosDec, 'YTickLabels',num2str(SegNum))
        
        
        %ylabel('#segment')
        if in2==0,
                uicontrol(gcf,'Style','text','String','X Scale','Units','normalized','Position',[.8 .3 .15 .03],'ForegroundColor','b');
                Xlim_1_Hndl = uicontrol(gcf,'Style','edit','Position',[.9 .25 .05 .03],...
                        'Units','normalized','BackgroundColor','w',...
                        'ForegroundColor','b',...
                        'Callback',['Xlim_1 = (get(gco,''string'')); gca = get(gcf,''Userdata''); XlimOld =  get(gca(2),''Xlim''); set(gca(2),''Xlim'',[str2num(Xlim_1)   XlimOld(2)] ); ud=get(0,''UserData''); ud(83) = str2num(Xlim_1); set(0,''Userdata'',ud);']);
                Xlim_2_Hndl = uicontrol(gcf,'Style','edit','Position',[.9 .2 .05 .03],...
                        'Units','normalized','BackgroundColor','w',...
                        'ForegroundColor','b',...
                        'Callback',['Xlim_2 = (get(gco,''string'')); gca = get(gcf,''Userdata''); XlimOld =  get(gca(2),''Xlim''); set(gca(2),''Xlim'',[XlimOld(1) str2num(Xlim_2)  ] ); ud=get(0,''UserData''); ud(84) = str2num(Xlim_2); set(0,''Userdata'',ud);']);
                
        end
        
        ud = get(gcf,'Userdata');
        set(gcf,'Userdata',[ud gca]);
        
        unitret('figname')
        
        
        %---------------------------------------------------------------------------------------
elseif strcmp(action,'save_hgl')
        
        hglname = ['.hgl'];
        
        
        [filename, pathname] = uiputfile(hglname, 'Save As ... in HGL format', .4, .4);
        if filename == 0, return; 
        end;
        hglname = [pathname,filename];
        set(0,'CurrentFigure',DefValues(72));
        com = ['print -dhpgl ', hglname];
        eval(com);
        
        %---------------------------------------------------------------------------------------
elseif strcmp(action,'save_m')
        
        mname = ['.m'];
        
        
        [filename, pathname] = uiputfile(mname, 'Save As ... in *.m format', .4, .4);
        if filename == 0, return; 
        end;
        mname = [pathname,filename];
        set(0,'CurrentFigure',DefValues(72));
        com = ['print -dmfile ', mname];
        eval(com);
        
        %---------------------------------------------------------------------------------------
elseif strcmp(action,'save_bmp')
        
        bmpname = ['.bmp'];
        
        
        [filename, pathname] = uiputfile(bmpname, 'Save As ... in BMP format', .4, .4);
        if filename == 0, return; 
        end;
        bmpname = [pathname,filename];
        com = ['print -dbmp16m ', bmpname];
        eval(com);
        %---------------------------------------------------------------------------------------


elseif(strcmp(action,'eye_spectra'))

	eyedata = in1;

	% TEST
	%eyedata.time = 0:1/200:(5-1/200);
	%eyedata.hor  = 0*sin(2*pi*1*eyedata.time)+0.707*sin(2*pi*5*eyedata.time);

        WC = get(gcf, 'Color');

        Fsample  = DefValues(167);
        Fdisplay = DefValues(162);
        nfft     = DefValues(163);
        noverlap = DefValues(164);
        window   = boxcar(DefValues(165));
        p        = DefValues(166);
        
	len	= length(eyedata.time);
        if DefValues(163) == 0,
		nfft = len;
	end
                
	if DefValues(165) == 0,
		window   = boxcar(nfft);
	end
                

        if DefValues(161) % with DC
                dflag='none';
        else
                dflag='mean';
        end
        
	if DefValues(171)==2 % velocity
		eyedata.hor = sqrt((eyedata.hor).^2 + (eyedata.ver).^2);
		eyedata.ver = zeros(size(eyedata.hor));
	end
		
        
        
		
	[Ph,FreqAxis] = spectrum(eyedata.hor, nfft,noverlap,window,Fsample,p,dflag);
	[Pv,FreqAxis] = spectrum(eyedata.ver, nfft,noverlap,window,Fsample,p,dflag);
	r = find(FreqAxis <= Fdisplay);
	switch DefValues(168) 
		case 1 	% Amplitude
			Ph = sqrt(4*Ph*norm(window)^2/sum(window)^2);
			Pv = sqrt(4*Pv*norm(window)^2/sum(window)^2);
			Ylabel = 'Amplitude';
		case 2 	% Power
			Ph = (4*Ph*norm(window)^2/sum(window)^2);
			Pv = (4*Pv*norm(window)^2/sum(window)^2);
			Ylabel = 'Power';
		case 3	% dB
			Ph = 10*log10(4*Ph*norm(window)^2/sum(window)^2);
			Pv = 10*log10(4*Pv*norm(window)^2/sum(window)^2);
			Ylabel = 'dB';
	end

	ch = [Ph(r,1)+Ph(r,2) Ph(r,1)-Ph(r,2)];
	cv = [Pv(r,1)+Pv(r,2) Pv(r,1)-Pv(r,2)];			

	plot(FreqAxis(r),Ph(r,1),'Color',[0 0.5 0]), hold on
	plot(FreqAxis(r),Pv(r,1),'m'), hold on


	% plot(FreqAxis(r),c(:,1),'r:',FreqAxis(r),c(:,2),'r:'), hold on 

	xlabel(['Frequency, Hz    Resolution: ',num2str(FreqAxis(2)-FreqAxis(1)),' Hz'],'Color','k');
	ylabel(Ylabel,'Color','k')
		
	        %---------------------------------------------------------------------------------	
elseif(strcmp(action,'analyze'))
	measureAnalysis = DefValues(175);

	
	Spk = process('make_train',in1);
	ConcSpk=[];
	SpkLen=size(Spk,2);
	for i=1:size(Spk,1)
		% OffSet=(i-1)*SpkLen+1;
		ConcSpk=[ConcSpk Spk(i,1:in1(i,4)-in1(i,3))];
	end

	out2 = [];
		
	out3.xx = [];
	out3.yy = [];

	out4.err = [];

	
	switch measureAnalysis
	case 1				% spike count
		out1 = sum(ConcSpk)/size(Spk,1);
		if DefValues(184)==2
			out4.err = std(sum(Spk,2));
		elseif DefValues(184)==3
			out4.err = std(sum(Spk,2))/sqrt(size(Spk,1));
		end
		
	case 2				% mean firing rate
		out1 = sum(ConcSpk)/length(ConcSpk)*1000;
		if DefValues(184)==2
			out4.err = std(sum(Spk,2)./size(Spk,2)*1000);
		elseif DefValues(184)==3
			out4.err = (std(sum(Spk,2)./size(Spk,2)*1000))/sqrt(size(Spk,1));
		end
		
	case 3				% harmonic content
		% out = process('spectra',7,in1);	% on raw concatenated
		% out1 = [out.dc out.f(1:2)];

		if DefValues(147),	% split to cycles
			out1 = process('spectra',7,in1);
			out2 = process('spectra',12,in1);  
			data = [out1.rm out2.rm out1.dc out2.phase out1.max_harmonic out1.max_harmonic_amp];

			out1 = [out1.dc out1.f(1:2)];
			out2 = data;
		else
			
			out1 = process('spectra',7,in1);
			data = [out1.rm -1 out1.dc -1 out1.max_harmonic out1.max_harmonic_amp];
				
			out1 = [out1.dc out1.f(1:2)];
			out2 = data;
		end
		
		
	case 4				% peak response
		[vect,histog]=process('average_spk_calculation',in1);
                xmin = 0;
               	xmax = max(in1(:,4) - in1(:,3));
   		bins=(xmin+DefValues(20)/2:DefValues(20):xmax-DefValues(20)/2);
                [N,SpikeHist]=hist(vect,bins);
		BinNormVector = process('bin_norm_calculation',in1,bins); 
                N = N./(BinNormVector*DefValues(20)/1000);

		if DefValues(196), % Smooth PSTH
			N = unitret('smooth',N,5,5);
		end
		
		peak = max(N);
		out1 = peak(1);

	case 5				% condition time
		out1 = length(ConcSpk);
	end

	if DefValues(194),		% plot individual histograms
		if measureAnalysis~=4
			[vect,histog]=process('average_spk_calculation',in1);
			xmin = 0;
	               	xmax = max(in1(:,4) - in1(:,3));
	   		bins=(xmin+DefValues(20)/2:DefValues(20):xmax-DefValues(20)/2);
	                [N,SpikeHist]=hist(vect,bins);
			BinNormVector = process('bin_norm_calculation',in1,bins); 
	                N = N./(BinNormVector*DefValues(20)/1000);
		end	

		if DefValues(196),	% Smooth PSTH
			out3.xx = SpikeHist;
			out3.yy = N;
		else
			[xx,yy] = bar(SpikeHist,N,1);
			out3.xx = xx;
			out3.yy = yy;
		end
	end	
	%---------------------------------------------------------------------------------

end;      %strcmp(action,'........')

