function out1 = unisplit(action, in1,in2,in3);
% unisplit	- multiunit isolation (spike shape separation)
%--------------------------------------------------------------------------------
% UNIversal SPLIT (initially based on split97)  			
% Written by A. Beylin and I. Kagan						
% (c) 1996-2002 								
%										
% *******************************						
% * Last modification: 12.12.02 *						
% *******************************
%						
% Short Description:								
% Reads all years shapes-containing files, including new system files		
% MATLAB 5.x compatible version					
%
% This program requires unitret.m and subprograms unifread.m and klt.exe
%--------------------------------------------------------------------------------

if nargin<1,
        clear global;
        h=get(0,'Children');
        close(h);
        fclose('all');
        action='start';
        matlabpath = path;
        act_dir = cd;
        path(matlabpath,act_dir);         
        if exist('tmp.mat')
                delete tmp.mat;
        end;
        warning('off');
end;


global SPLIT_DAT AXES_DAT FULLPAGE INIFILE APP_PATH SAVE_AS_SORTED_HNDL
global emformat emlen spformat shformat
global C				% initial centroids for autoprocessing
global HFIG_SHAPE_PER_TRIAL



DefValues = get(0,'UserData');


if strcmp(action,'start'),
        APP_PATH = which('unisplit');
        APP_PATH = APP_PATH(1:end-10);
        INIFILE = [APP_PATH 'unisplit.ini'];
        
        % graphics initialization
        ini_exist = exist(INIFILE);
        if(ini_exist~=2),
                
                % Default values: - Stored in ROOT object (0) as a UserData
                HPeriod = 0;				% 1
                MinHPeriod = 0;				% 2
                MaxHPeriod = 100;			% 3
                
                Trials = 1;				% 4
                MinTrials = 1;				% 5
                MaxTrials = 3;				% 6
                
                Ymin = -50;				% 7
                MinYmin = -400;				% 8
                MaxYmin = -10;				% 9
                
                Ymax = 50;				% 10
                MinYmax = 10;				% 11
                MaxYmax = 400;				% 12
                
                
                ShowSpikes = 500;			% 13
                MinShowSpikes = 1;			% 14
                MaxShowSpikes = 500;			% 15
                
                DeffuzValue = 0.5;			% 16
                MinDeffuzValue = 0.1;			% 17
                MaxDeffuzValue = 1;			% 18
                
                Autoscale = 0;				% 19
                
                % empty values - not relevant in this program   
                
                HAverPeriod = 10;			% 20
                MinHAverPeriod = 0;			% 21
                MaxHAverPeriod = 100;			% 22
                
                EyeMovement = 0;			% 23   
                Grid         =0;			% 24
                SaccadeTreshold=5;			% 25
                MarkSelect = 0;				% 26
                
                NormalEyeMovement=0;			% 27
                RejectSegmentEdge = 0;			% 28
                SafetyInterval   =1.1;			% 29
                
                ShapeStripPlot  =0;			% 30
                StripPlotWindow = 500;			% 31
                MinStripPlotWindow = 1;			% 32
                MaxStripPlotWindow = 5000;		% 33
                
                ShrinkTime=0;				% 34
                Kohonen = 0;				% 35
                Shift = 1;				% 36
                ShiftPhase = 0;				% 37 
                % 0 shift relatively minimum
                % 1 shift relatively maximum
                %-------------------------------------------------------------------
                
                FULLPAGE = 0; 				% 38 graphic mode
                RejectSpkWin = 1;			% 39 Reject Spikes in After-trigger Shape sampling window
                
                pca_plane_autoscale = 1;		% 40 pca plane autoscale on (1) / off (0)
                pca_plane_scale_xmin = -200;		% 41 pca plane xmin
                pca_plane_scale_xmax = 200;		% 42 pca plane xmax
                pca_plane_scale_ymin = -200;		% 43 pca plane ymin
                pca_plane_scale_ymax = 200;		% 44 pca plane ymax
                
                auto_process_file	= 0;		% 45 autoprocess the whole (rest) of the file based on the current trial (0/1)
                semi_auto_process_file	= 0;		% 46 semiauto - ask if auto separation is ok? 
		
                DefValues = [HPeriod, MinHPeriod, MaxHPeriod,... 
                                Trials,     MinTrials,  MaxTrials,...
                                Ymin,       MinYmin,    MaxYmin,...
                                Ymax ,      MinYmax,    MaxYmax,...
                                ShowSpikes, MinShowSpikes, MaxShowSpikes,...
                                DeffuzValue, MinDeffuzValue, MaxDeffuzValue,...
                                Autoscale, HAverPeriod, MinHAverPeriod, ...
                                MaxHAverPeriod,EyeMovement, Grid,...
                                SaccadeTreshold, MarkSelect,NormalEyeMovement,...
                                RejectSegmentEdge,SafetyInterval,ShapeStripPlot,...
                                StripPlotWindow,MinStripPlotWindow,MaxStripPlotWindow,...
                                ShrinkTime,Kohonen,Shift,ShiftPhase,FULLPAGE,RejectSpkWin,...
                                pca_plane_autoscale,pca_plane_scale_xmin,pca_plane_scale_xmax,pca_plane_scale_ymin,pca_plane_scale_ymax,...
				auto_process_file, semi_auto_process_file];
                
        else
                DefValues = unitret('read_ini',INIFILE);
        end
        
        set(0,'UserData',DefValues,...
                'DefaultTextFontName','Helvetica',...
                'DefaultFigureInvertHardcopy','off',...
                'DefaultTextFontSize',9,...
                'DefaultTextColor','k',...
                'DefaultTextFontWeight','demi',...
                'DefaultFigureNumberTitle','off', ...
                'DefaultFigurePaperType','usletter');
        
        Fig1 = figure('Backingstore','on','Color',[ 0.7098    0.7608    0.6549],...
                'Units','Normal','Position',[0.005 0.04 .27 .91],'Resize','Off', 'Name',...
                'UNISPLIT v.3.1 rev. 12.12.02','MenuBar','none','Number','off');
        
        if (exist('unispliticon.png'))
                I  = imread('unispliticon.png','png');
                axes('Units','Pixels','Position',[1 1 32 32]);
                imshow(I);
                clear I;
        end;
        
        axis('off');
        
        u1 = uimenu(Fig1,'Label','&File');
        u101 = uimenu(u1,'Label','Open &Experimental Record','Callback','unisplit(''read_header'')');
        SAVE_AS_SORTED_HNDL(1) = uimenu(u1,'Label','Save Trial &As Sorted CELL1','Callback','unisplit(''save_as_sorted'',1);','Separator','on');
        SAVE_AS_SORTED_HNDL(2) = uimenu(u1,'Label','Save Trial &As Sorted CELL2','Callback','unisplit(''save_as_sorted'',2);');
        SAVE_AS_SORTED_HNDL(3)  = uimenu(u1,'Label','Save Trial &As Sorted CELL3','Callback','unisplit(''save_as_sorted'',3);');
        SAVE_AS_SORTED_HNDL(4) = uimenu(u1,'Label','Save Trial &As Sorted CELL4','Callback','unisplit(''save_as_sorted'',4);');
        
        set(SAVE_AS_SORTED_HNDL, 'Enable','off');
        
        u102 =  uimenu(u1,'Label','Call &UNITRET','Callback','unisplit(''exit''); unitret','Separator','on');      
        u103 = uimenu(u1,'Label','E&xit','Separator','on','Callback','unisplit(''exit'')');
        
        u2 = uimenu(Fig1,'Label','&Help');
        u201 = uimenu(u2,'Label','About Unisplit...','Callback','unisplit(''about Unisplit'')');
        
        u203 = uimenu(u2,'Label','Online Help','Callback','web(''http://igoresha.virtualave.net/Matlab/Unisplit.html'')');
        % u204 = uimenu(u2,'Label','HTML Help','Callback','global APP_PATH; oripath=pwd;cd(APP_PATH);!unisplit.chm;cd(oripath);)');
	u204 = uimenu(u2,'Label','HTML Help','Callback','web([''file:///'' which(''UnisplitHelp.html'')]);');
        
        u205 = uimenu(u2,'Label','Readme','Callback','global APP_PATH; oripath=pwd;cd(APP_PATH);!readme.txt;cd(oripath);)','Separator','on');
        u207 = uimenu(u2,'Label','License','Callback','global APP_PATH; oripath=pwd;cd(APP_PATH);!license.txt;cd(oripath);)','Separator','off');
        
        u208 = uimenu(u2,'Label','Delete *.INI file (unisplit.ini) and restart','Callback','unisplit(''delete_ini'')','Separator','on');
        
        
        SPLIT_DAT(1) = Fig1;
        set(gcf,'Pointer','Arrow');
        
        %--------------------------------------------------------------------------------------------------------------
elseif(strcmp(action,'about Unisplit'))
        
        Fig = figure('Backingstore','on','Color',[ 0.6137    0.6961    0.6725],...
                'Units','Normalized','Position',[0.1 0.1 .8 .8],'Resize','Off', 'Name',...
                'About Unisplit','MenuBar','none','Number','off'); 
        
        about_unisplit_string = help('unisplit');
        
        u = uicontrol(gcf,'Style','edit','FontName','Fixedsys','Units','Normalized','Position',[0.05 0.2 0.9 0.75],'Max',2,...
                'String',about_unisplit_string,'HorizontalAlignment','left');
        
        hclose=uicontrol(Fig,'Style','pushbutton','Units','Normalized','Position',[.45 .05 .1 .05],...
                'String','Close','Callback',['delete(',num2str(Fig), '), return']);
        
        
        uicontrol('Style','text','Units','Normalized','Position',[0.05 0.13 0.8 0.05],'String',...
                'Scroll down for more info (click in text window and press "PageDown" or "Down Arrow")',...
                'HorizontalAlignment','left','BackgroundColor',[ 0.6137    0.6961    0.6725],'ForegroundColor',[0 0.502 0.502]);
        
        %---------------------------------------------------------------------------------------------------------------------------------------------------------------     
elseif(strcmp(action,'delete_ini')),
        delete(INIFILE);
        fclose('all');
        kids = get(0,'Children');
        delete(kids);
        unisplit;
        
        
        %---------------------------------------------------------------------------------------------------------------------------------------------------------------    
elseif(strcmp(action,'setup_exp'))
        
        
        FrameTrial=get(SPLIT_DAT(3),'userdata'); 
        Fig1 = figure('Backingstore','off','Name','Setup','Color','w',...
                'Units','Normal','Position',[.22 .4 .5 .5],'Resize','Off', 'Menubar','none','Number','off');
        set(gcf,'Pointer','Watch');
        
        uicontrol(Fig1,'Style','text','Position',[.2 .91 .5 .08],...
                'Units','normal','ForegroundColor',[0    0.5020    0.5020],...
                'BackgroundColor','w','String','Options:',...
                'HorizontalAlignment','center');
        
        uicontrol(Fig1,'Style','text','Position',[.02 .85 .5 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','String',['Histogram, (', num2str(DefValues(2)), '...',...
                        num2str(DefValues(3)),') ms'],'HorizontalAlignment','left');
        
        
        
        u1 = uicontrol(Fig1,'Style','edit','Position',[.38 .85 .11 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','String',num2str(DefValues(1)),...
                'HorizontalAlignment','left','Callback','unisplit(''update'',1);');
        %-----------------  
        uicontrol(Fig1,'Style','text','Position',[.02 .77 .5 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','String',['Shapes, (', num2str(DefValues(14)), '...',...
                        num2str(DefValues(15)),')'],'HorizontalAlignment','left');
        
        
        u2 = uicontrol(Fig1,'Style','edit','Position',[.38 .77 .11 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','String',num2str(DefValues(13)),...
                'Callback','unisplit(''update'',13);');
        
        %----------------
        uicontrol(Fig1,'Style','text','Position',[.02 .69 .5 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','String',['Defuzz. threshold, (', num2str(DefValues(17)), '...',...
                        num2str(DefValues(18)),')'],'HorizontalAlignment','left');
        
        
        u21 = uicontrol(Fig1,'Style','edit','Position',[.38 .69 .11 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','String',num2str(DefValues(16)),...
                'Callback','unisplit(''update'',16);');
        
        %----------------
        
        uicontrol(Fig1,'Style','text','Position',[.02 .59 .5 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','b','String','Trial Y Scale:','HorizontalAlignment','left');
        
        %-----------------
        
        uicontrol(Fig1,'Style','text','Position',[.02 .51 .5 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','String',...
                ['From (',num2str(DefValues(8)),'...',...
                        num2str(DefValues(9)),')'], 'HorizontalAlignment','left');
        
        
        u3 = uicontrol(Fig1,'Style','edit','Position',[.18 .51 .11 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','String',num2str(DefValues(7)),...
                'Callback','unisplit(''update'',7);');
        
        %----------------------
        uicontrol(Fig1,'Style','text','Position',[.38 .51 .5 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','String',['To (',num2str(DefValues(11)),'...',...
                        num2str(DefValues(12)),')'], 'HorizontalAlignment','left');
        
        
        u4 = uicontrol(Fig1,'Style','edit','Position',[.54 .51 .11 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','String',num2str(DefValues(10)),...
                'Callback','unisplit(''update'',10);');
        %---------------------
        
        
        uicontrol(Fig1,'Style','text','Position',[.02 .43 .5 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','String','Grid, 100 ms', 'HorizontalAlignment','left');
        
        
        u5 = uicontrol(Fig1,'Style','checkbox','Position',[.38 .43 .11 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','value',DefValues(24),'callback',...
                ['u=get(gcf,''CurrentObject''); val = get(u,''value'');',...
                        'ud=get(0,''UserData''); ud(24) = val; set(0,''userdata'',ud);']);
        
        %----------------------------------
        
        %---------------------
        
        
        uicontrol(Fig1,'Style','text','Position',[.02 .35 .5 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','String','Split shape autoscale', 'HorizontalAlignment','left');
        
        
        u6 = uicontrol(Fig1,'Style','checkbox','Position',[.38 .35 .11 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','value',DefValues(19),'callback',...
                ['u=get(gcf,''CurrentObject''); val = get(u,''value'');',...
                        'ud=get(0,''UserData''); ud(19) = val; set(0,''userdata'',ud);']);
        
        % ---
        uicontrol(Fig1,'Style','text','Position',[.02 .27 .5 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','String','PCA plane autoscale', 'HorizontalAlignment','left');
        
        
        u_pca_autoscale = uicontrol(Fig1,'Style','checkbox','Position',[.38 .27 .11 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','value',DefValues(40),'callback',...
                ['u=get(gcf,''CurrentObject''); val = get(u,''value'');',...
                        'ud=get(0,''UserData''); ud(40) = val; set(0,''userdata'',ud);'...
                        'uu = get(u,''UserData''); if val, set(uu,''Enable'',''off''); else set(uu,''Enable'',''on''); end;']);
        
        uicontrol(Fig1,'Style','text','Position',[.54 .31 .5 .05],...
                'Units','normalized','BackgroundColor','w',... 
                'ForegroundColor','k','String','xmin            xmax            ymin            ymax', 'HorizontalAlignment','left');
        
        uu(1) = uicontrol(Fig1,'Style','edit','Position',[.54 .27 .10 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','String',num2str(DefValues(41)),...
                'Callback','unisplit(''update'',41,0,1);');
        
        
        uu(2) = uicontrol(Fig1,'Style','edit','Position',[.65 .27 .10 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','String',num2str(DefValues(42)),...
                'Callback','unisplit(''update'',42,0,1);');
        
        
        uu(3) = uicontrol(Fig1,'Style','edit','Position',[.77 .27 .10 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','String',num2str(DefValues(43)),...
                'Callback','unisplit(''update'',43,0,1);');
        
        uu(4) = uicontrol(Fig1,'Style','edit','Position',[.88 .27 .10 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','String',num2str(DefValues(44)),...
                'Callback','unisplit(''update'',44,0,1);');
        
        pca_xlim = get(AXES_DAT(3),'Xlim');
        pca_ylim = get(AXES_DAT(3),'Ylim');
        
        set(uu(1),'String',num2str(pca_xlim(1)));
        set(uu(2),'String',num2str(pca_xlim(2)));
        set(uu(3),'String',num2str(pca_ylim(1)));
        set(uu(4),'String',num2str(pca_ylim(2)));
        
        DefValues(41:44)=[pca_xlim pca_ylim];
        set(0,'UserData',DefValues);
        
        if (get(u_pca_autoscale,'value')),
                enable_uu = 'off';
        else
                enable_uu = 'on';
        end
        set(uu,'enable',enable_uu);
        set(u_pca_autoscale,'Userdata',uu);
        
        %---------------------
        
        
        uicontrol(Fig1,'Style','text','Position',[.02 .19 .5 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','String','Kohonen', 'HorizontalAlignment','left');
        
        
        u7 = uicontrol(Fig1,'Style','checkbox','Position',[.38 .19 .11 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','value',DefValues(35),'callback',...
                ['u=get(gcf,''CurrentObject''); val = get(u,''value'');',...
                        'ud=get(0,''UserData''); ud(35) = val; set(0,''userdata'',ud);']);
        
        
        %---------------------
        
        
        uicontrol(Fig1,'Style','text','Position',[.02 .11 .5 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','String','Spikes Alignment', 'HorizontalAlignment','left');
        
        
        u8 = uicontrol(Fig1,'Style','checkbox','Position',[.38 .11 .11 .05],...
                'Units','normalized','BackgroundColor','w',...
                'ForegroundColor','k','value',DefValues(36),'callback',...
                ['u=get(gcf,''CurrentObject''); val = get(u,''value'');',...
                        'ud=get(0,''UserData''); kids=get(u,''userdata''); ud(36) = val; set(0,''userdata'',ud);',...
                        'if ~val, set(kids,''enable'',''off'');else set(kids,''enable'',''on'');end;']);
        
        
        
        
        u9 = uicontrol(Fig1,'Style','checkbox','Position',[.55 .11 .2 .05],...
                'Units','normalized','BackgroundColor','w','String','Positive',...
                'ForegroundColor','k','value',DefValues(37),'enable','off','callback',...
                ['u=get(gcf,''CurrentObject''); val = get(u,''value''); opp=get(u,''userdata'');',...
                        'ud=get(0,''UserData''); ud(37) = val; set(0,''userdata'',ud);',...
                        'if val, set(opp,''value'',0); else set(opp,''value'',1);end;']);
        
        
        u10 = uicontrol(Fig1,'Style','checkbox','Position',[.78 .11 .2 .05],...
                'Units','normalized','BackgroundColor','w','String','Negative',...
                'ForegroundColor','k','value',~DefValues(37),'enable','off','callback',...
                ['u=get(gcf,''CurrentObject''); val = get(u,''value''); opp=get(u,''userdata'');',...
                        'ud=get(0,''UserData''); ud(37) = ~val; set(0,''userdata'',ud);',...
                        'if val, set(opp,''value'',0); else set(opp,''value'',1);end;']);
        
        set(u8,'userdata',[u9 u10]);
        set(u9,'userdata',u10);
        set(u10,'userdata',u9);
        
        u11 = uicontrol(Fig1,'Style','checkbox','Position',[.38 .01 .5 .05],...
                'Units','normalized','BackgroundColor','w','String','Reject Spikes in Sample Win.',...
                'ForegroundColor','b','value',DefValues(39),'enable','on','callback',...
                ['u=get(gcf,''CurrentObject''); val = get(u,''value''); opp=get(u,''userdata'');',...
                        'ud=get(0,''UserData''); ud(39) = val; set(0,''userdata'',ud);']);
        
        if DefValues(36), set([u9 u10],'enable','on'); end;
        
        %----------------------------------------------------------------------------------------
        OK_button=uicontrol(Fig1,'Style','pushbutton','Position',[.75 .83 .22 .12],...
                'Units','Normalized','String','OK','Callback',['delete(',num2str(Fig1), '); ',...
                        'drawnow;unisplit(''update'',0,',num2str(FrameTrial),');watchoff;']);
        
        cancel=uicontrol(Fig1,'Style','pushbutton','Position',[.75 .65 .22 .12],...
                'Units','Normalized','String','Cancel','Callback',['delete(',num2str(Fig1), '), return']);
        
        
        set(gcf,'Pointer','Arrow');
        set(SPLIT_DAT(1),'Pointer','Arrow');
        
        %--------------------------------------------------------------------------------------------------------------------------------------------------------------
        
elseif(strcmp(action,'read_header'))
        
        clear global MERGE CLUSTER_INDEX OLD_CLUSTER_INDEX SPIKES STIMULUS MOTION AX_HAND SPLIT_CLUSTER_INDEX C;
        
        
        if  (size(SPLIT_DAT,2) > 1 ),    % second member exists (pointer on graph figure)
                if strcmp(get(SPLIT_DAT(2),'Type'),'figure'),
                        delete(SPLIT_DAT(2));
                        SPLIT_DAT = SPLIT_DAT(1);
                end;  
        end;      
        
        
        
        old_userdata = get(SPLIT_DAT(1),'UserData');
        if ~isempty(old_userdata)
                oldfid = old_userdata(1);
                fclose(oldfid);
        end;
        
        [filename, pathname] = uigetfile('*.*' , 'Retrival of Single Unit Recording Data', .4, .4);
        if filename == 0, return; end;
        FullName = [pathname,filename];
        
        [fid, message] = fopen(FullName,'rb');
        if(fid<0), 
                error(message);
                return;
        end;
        com = ['cd ' pathname(1:length(pathname)-1)]; 
        eval(com);
        
        % delete previous basis.dat, if exists
        if exist([pathname 'basis.dat'])==2,
                delete([pathname 'basis.dat']);
        end
        
        StartPosition = ftell(fid);
        
        
        % ---------- READING THE FILE HEADER BLOCK -----------------------
        % Common for all actions
        
        Ver                     =    fread(fid, 1,'ushort');
        FileLength              =    fread(fid, 1,'long');
        FileHeaderLength        =    fread(fid, 1,'ushort');
        GlobSpecBlockNum        =    fread(fid, 1,'ushort');
        TrialsNum               =    fread(fid, 1,'ushort');
        CommLength              =    fread(fid, 1,'ushort');
        GlobSpecBlockLength     =    fread(fid,GlobSpecBlockNum, 'ushort');
        TrialBegin              =    fread(fid,TrialsNum,'long');
        
        if unisplit('NoSeparator',fid)
                strerr = ['FORMAT ERROR on THE FILE HEADER BLOCK: file ', filename, ' skipped'];
                errordlg(strerr, 'ERROR');
                fclose(fid);
                return;
        end;
        %disp('FILE HEADER BLOCK .... OK');
        
        % ---------- READING THE FILE SPECIFICATION  BLOCK -----------------------
        
        blockSTART     	        =    ftell(fid);
        
        FileName                        =    (setstr(fread(fid, 14,'char')))';
        FileDate                        =    (setstr(fread(fid, 10,'char')))';
        RunModName              =    (setstr(fread(fid, 10,'char')))';
        VideoFramePeriod                =    fread(fid,1,'float');
        ViewDistance            =    fread(fid,1,'float');
        TimeFirstSample         =    fread(fid,1,'float');
        NumSamp2Frame           =    fread(fid,1,'ushort');
        HorizVisualFieldLoc     =    fread(fid,1,'float');
        VertVisualFieldLoc              =    fread(fid,1,'float');
        HorizPosLED             =    fread(fid,1,'float');
        VertPosLED              =    fread(fid,1,'float');
        HorizEyePosGain         =    fread(fid,1,'float');
        VertEyePosGain          =    fread(fid,1,'float');
        EyetrackArbDef          =    fread(fid,1,'float');
        EyetrackArbV0           =    fread(fid,1,'ushort');
        Empty                   =    fread(fid,1,'ushort');
        LeftEdge                        =    fread(fid,1,'ushort');
        BottomEdge              =    fread(fid,1,'ushort');
        RightEdge               =    fread(fid,1,'ushort');
        TopEdge                 =    fread(fid,1,'ushort');
        StabFlag                =    fread(fid,1,'ushort');
        Empty                   =    fread(fid,1,'ushort');     % temporal type: flag
        Empty                   =    fread(fid,1,'ushort');     % spatial type: flag
        CompFlag                =    fread(fid,1,'ushort');
        DateTime                =    (setstr(fread(fid, 18,'uchar')))';
        EyePer                  =    fread(fid,1,'float');
        SpikePer                =    fread(fid,1,'float');
        ShapePer                =    fread(fid,1,'float');
        FixationType		=    unifread([fid, 1,blockSTART,GlobSpecBlockLength],'ushort');
        
        if ~ShapePer,
                ShapePer =  1000/10000;               %ms
        end;
        
        if unisplit('NoSeparator',fid)
                strerr = ['FORMAT ERROR on THE FILE SPECIFICATION  BLOCK: file ', filename, ' skipped'];
                errordlg(strerr, 'ERROR');
                fclose(fid);
                return;
        end;
        % disp('FILE SPECIFICATION  BLOCK ....OK');
        
        % ---------- READING THE FILE COMMENT  BLOCK -----------------------
        
        Comments = (setstr(fread(fid,CommLength, 'char')))';
        
        if unisplit('NoSeparator',fid)
                strerr = ['FORMAT ERROR on THE FILE COMMENT  BLOCK: file ', filename, ' skipped'];
                errordlg(strerr, 'ERROR');
                fclose(fid);
                return;
        end;
        % disp('FILE COMMENT  BLOCK ...OK');
        
        % textout on the left panel:
        cla;
        axes('Position',[0 0 1 1],'Visible','off');
        
        set(text,'Position',[.02 .97],'Color','r','string','File name:'); 
        set(text,'Position',[.5 .97],'Color','k','string',deblank(FileName)); 
        
        set(text,'Position',[.02 .95],'Color','r','string','File Date:'); 
        set(text,'Position',[.5 .95],'Color','k','string',deblank(FileDate)); 
        
        set(text,'Position',[.02 .93],'Color','r','string','Run mode:'); 
        set(text,'Position',[.5 .93],'Color','k','string',deblank(RunModName)); 
        
        set(text,'Position',[.02 .91],'Color','r','string','Video frame T, ms:'); 
        set(text,'Position',[.63 .91],'Color','k','string',num2str(VideoFramePeriod)); 
        
        set(text,'Position',[.02 .89],'Color','r','string','View distance,cm:'); 
        set(text,'Position',[.63 .89],'Color','k','string',num2str(ViewDistance)); 
        
        set(text,'Position',[.02 .87],'Color','r','string','First sample time:'); 
        set(text,'Position',[.63 .87],'Color','k','string',num2str(TimeFirstSample)); 
        
        set(text,'Position',[.02 .85],'Color','r','string','Samp. per frame:'); 
        set(text,'Position',[.63 .85],'Color','k','string',num2str(NumSamp2Frame)); 
        
        set(text,'Position',[.02 .83],'Color','r','string','Vis field loc:'); 
        set(text,'Position',[.63 .83],'Color','k','string',...
                [num2str(HorizVisualFieldLoc),',',num2str(VertVisualFieldLoc)]); 
        
        set(text,'Position',[.02 .81],'Color','r','string','LED position:'); 
        set(text,'Position',[.63 .81],'Color','k','string',...
                [num2str(HorizPosLED),',',num2str(VertPosLED)]); 
        
        set(text,'Position',[.02 .79],'Color','r','string','Eye position gain:'); 
        set(text,'Position',[.63 .79],'Color','k','string',...
                [num2str(HorizEyePosGain),',',num2str(VertEyePosGain)]); 
        
        set(text,'Position',[.02 .77],'Color','r','string','Eye track arb. def.:'); 
        set(text,'Position',[.63 .77],'Color','k','string',num2str(EyetrackArbDef)); 
        
        set(text,'Position',[.02 .75],'Color','r','string','Eye track arb. V0.:'); 
        set(text,'Position',[.63 .75],'Color','k','string',num2str(EyetrackArbV0)); 
        
        set(text,'Position',[.02 .73],'Color','r','string','Edge, left, right:'); 
        set(text,'Position',[.63 .73],'Color','k','string',...
                [num2str(LeftEdge),',',num2str(RightEdge)]); 
        
        set(text,'Position',[.02 .71],'Color','r','string','Edge, bottom, top:'); 
        set(text,'Position',[.63 .71],'Color','k','string',...
                [num2str(BottomEdge),',',num2str(TopEdge)]); 
        
        set(text,'Position',[.02 .69],'Color','r','string','Stab flag:'); 
        str = unisplit('translate','StabFlag',StabFlag);
        set(text,'Position',[.63 .69],'Color','k','string',str); 
        
        set(text,'Position',[.02 .67],'Color','r','string','Comp flag:'); 
        str = unisplit('translate','CompFlag',CompFlag);
        set(text,'Position',[.63 .67],'Color','k','string',str); 
        
        set(text,'Position',[.02 .65],'Color','r','string','Eye period, ms:'); 
        set(text,'Position',[.63 .65],'Color','k','string',num2str(EyePer)); 
        
        set(text,'Position',[.02 .63],'Color','r','string','Spike period, ms:'); 
        set(text,'Position',[.63 .63],'Color','k','string',num2str(SpikePer)); 
        
        set(text,'Position',[.02 .61],'Color','r','string','Shape Period:'); 
        set(text,'Position',[.63 .61],'Color','k','string',num2str(ShapePer)); 
        
        set(text,'Position',[.02 .59],'Color','r','string','Fixation type:'); 
        str = unisplit('translate','FixationType',FixationType);
        set(text,'Position',[.63 .59],'Color','k','string',str); 
        
        set(text,'Position',[.02 .57],'Color','r','string','Time:'); 
        set(text,'Position',[.45 .57],'Color','k','string',deblank(DateTime)); 
        
        if (CommLength),
                set(text,'Position',[.02 .55],'Color','r','string','Comments:'); 
                set(text,'Position',[.02 .53],'Color','k','string',deblank(Comments)); 
        end;
        
        uicontrol('Style','Pushbutton','String','Trial Info','Units','Normalized',...
                'Position',[.25 .4 .5 .05],'Callback','unisplit(''fulldump'',0);');
        
        set(SPLIT_DAT(1),'Name',deblank(filename));
        
        
        trial_inf  = [fid TrialsNum TrialBegin'];
        set(SPLIT_DAT(1),'UserData',trial_inf);
        
        % show graph figure
        set(gcf,'Pointer','Arrow');
        set(SPLIT_DAT(1),'Pointer','Arrow');
        
        
        drawnow;
        
        DefValues(45)=0;
        set(0,'UserData',DefValues);
        
        unisplit('redraw_exp',0); 
        
        %---------------------------------------------------------------------------------------------------------------------------------------------------------------     
elseif(strcmp(action,'redraw_exp'))
        
        global STIMULUS SPIKES MOTION SHNUM AXES_DAT C;
        clear global MERGE CLUSTER_INDEX OLD_CLUSTER_INDEX AX_HAND SPLIT_CLUSTER_INDEX;
        
        if  (size(SPLIT_DAT,2) > 1 ),    % second member exists (pointer on graph figure)
                if strcmp(get(SPLIT_DAT(2),'Type'),'figure'),
                        Fig2 = SPLIT_DAT(2);
                        set(0,'CurrentFigure',Fig2);
                        kids=get(Fig2,'Children');
                        for i=1:length(kids),
                                tp=get(kids(i),'type');
                                if (strcmp(tp,'axes')) | (strcmp(tp,'uicontrol')) 
                                        delete(kids(i));
                                end;
                        end;
                end;
        else,       
                Fig2 = figure('Units','normal', 'Position',[0.281 0.04 .71 .91],...
                        'Backingstore','on','color','w','Menubar','none',...
                        'DefaultAxesFontSize',7,'DefaultAxesDrawMode','fast',...
                        'DefaultAxesYColor','k','DefaultAxesXColor','k');
                u1 = uimenu(Fig2,'Label','&File');
                
                
                
                u13 = uimenu(u1,'Label','P&rint Setup...','Separator','on','Callback','print -dsetup;');     
                u14 = uimenu(u1,'Label','&Print','Callback','orient landscape; print;');
                u16 = uimenu(u1,'Label','&Export to HGL','Callback','unisplit(''hgl'');');
                
                u15 = uimenu(u1,'Label','E&xit','Separator','on','Callback','unisplit(''exit'');');
                
                u2 = uimenu(Fig2,'Label','&Display');
                u21 = uimenu(u2,'Label','&Setup','Callback','unisplit(''setup_exp'')');
                u22 = uimenu(u2,'Label','&Mean Shape','Callback','unisplit(''meanshape'')');
                u23 = uimenu(u2,'Label','&Copy','Callback','print -dmeta');
                
                u5 = uimenu(Fig2,'Label','&Go to trial...');
                
                
                trials_inf = get(SPLIT_DAT(1),'UserData');
                TrialsNum = trials_inf(2);
                TrialBegin = trials_inf(3:TrialsNum+2);
                
                
                if TrialsNum >0,    
                        for(t=1:TrialsNum),
                                uimenu(u5, 'Label',num2str(t),'Callback',['h=get(gcf,''CurrentMenu''); unisplit( ''redraw_exp'', str2num( get(h,''Label'')));']);	      
                        end;
                end;
                
                SPLIT_DAT(2:9) = [Fig2 u21 0 0 0 0 0 0];
                
        end; 
        set(gcf,'Pointer','Watch');
        
        if (in1==0), %<-------- First entry into the file
                in1 = 1; 
                set(Fig2,'UserData',[]);
        end; 
        
        
        
        trials_inf = get(SPLIT_DAT(1),'UserData');
        filename = get(SPLIT_DAT(1),'Name');
        fid = trials_inf(1);
        TrialsNum = trials_inf(2);
        
        lastfig = min(TrialsNum, in1+DefValues(4)-1);
        nextfig = lastfig+1;
        prevfig = max(1,in1 - DefValues(4));
        
        
        
        
        
        set(SPLIT_DAT(3),'userdata', in1);                 % prepare for setup update
        
        % axis location calculations
        
        Trials2Screen = 1;
        col = 1;
        row = 1;
        
        
        
        % READING OF THE TRIAL DATA: 
        for(numrow=0:row-1),
                for(numcol=0:col-1),
                        gind = (numrow*col + numcol);
                        tnum = in1 + gind;
                        
                        if(tnum > lastfig), 
                                break;
                        else
                                status = fseek(fid,0,-1);
                                
                                
                                % ---------- READING THE FILE HEADER BLOCK -----------------------
                                % Common for all actions
                                
                                Ver                     =    fread(fid, 1,'ushort');
                                FileLength              =    fread(fid, 1,'long');
                                FileHeaderLength        =    fread(fid, 1,'ushort');
                                GlobSpecBlockNum        =    fread(fid, 1,'ushort');
                                TrialsNum               =    fread(fid, 1,'ushort');
                                CommLength              =    fread(fid, 1,'ushort');
                                GlobSpecBlockLength     =    fread(fid,GlobSpecBlockNum, 'ushort');
                                TrialBegin              =    fread(fid,TrialsNum,'long');
                                if unisplit('NoSeparator',fid)
                                        strerr = ['FORMAT ERROR IN THE FILE HEADER BLOCK : ', filename, ' skipped'];
                                        errordlg(strerr, 'ERROR');
                                        fclose(fid);
                                        return;
                                end;
                                %	disp('FILE HEADER BLOCK .... OK');
                                % ---------- READING THE FILE SPECIFICATION  BLOCK -----------------------
                                
                                blockSTART     	   =    ftell(fid);
                                
                                FileName                =    (setstr(fread(fid, 14,'char')))';
                                FileDate                =    (setstr(fread(fid, 10,'char')))';
                                RunModName              =    (setstr(fread(fid, 10,'char')))';
                                VideoFramePeriod        =    fread(fid,1,'float');
                                ViewDistance            =    fread(fid,1,'float');
                                TimeFirstSample         =    fread(fid,1,'float');
                                NumSamp2Frame           =    fread(fid,1,'ushort');
                                HorizVisualFieldLoc     =    fread(fid,1,'float');
                                VertVisualFieldLoc              =    fread(fid,1,'float');
                                HorizPosLED             =    fread(fid,1,'float');
                                VertPosLED              =    fread(fid,1,'float');
                                HorizEyePosGain         =    fread(fid,1,'float');
                                VertEyePosGain          =    fread(fid,1,'float');
                                EyetrackArbDef          =    fread(fid,1,'float');
                                EyetrackArbV0           =    fread(fid,1,'ushort');
                                Empty                   =    fread(fid,1,'ushort');
                                LeftEdge                        =    fread(fid,1,'ushort');
                                BottomEdge              =    fread(fid,1,'ushort');
                                RightEdge               =    fread(fid,1,'ushort');
                                TopEdge                 =    fread(fid,1,'ushort');
                                StabFlag                =    fread(fid,1,'ushort');
                                Empty                   =    fread(fid,1,'ushort');     % temporal type: flag
                                Empty                   =    fread(fid,1,'ushort');     % spatial type: flag
                                CompFlag                =    fread(fid,1,'ushort');
                                DateTime                =    (setstr(fread(fid, 18,'uchar')))';
                                EyePer                  =    fread(fid,1,'float');
                                SpikePer                =    fread(fid,1,'float');
                                ShapePer                =    fread(fid,1,'float');
                                FixationType		=    unifread([fid, 1,blockSTART,GlobSpecBlockLength],'ushort');
                                
                                if ~ShapePer,
                                        ShapePer =  1000/10000;               %ms
                                end;
                                SPLIT_DAT(9) = ShapePer;
                                
                                if unisplit('NoSeparator',fid)
                                        strerr = ['FORMAT ERROR IN THE FILE SPECIFICATION  BLOCK: ', filename, ' skipped'];
                                        errordlg(strerr, 'ERROR');
                                        fclose(fid);
                                        return;
                                end;
                                %disp('FILE SPECIFICATION  BLOCK ....OK');
                                
                                % ---------- READING THE FILE COMMENT  BLOCK -----------------------
                                
                                Comments = (setstr(fread(fid,CommLength, 'char')))';
                                
                                if unisplit('NoSeparator',fid)
                                        strerr = ['FORMAT ERROR IN THE FILE COMMENT  BLOCK: ', filename, ' skipped'];
                                        errordlg(strerr, 'ERROR');
                                        fclose(fid);
                                        return;
                                end;
                                
                                
                                %disp('FILE COMMENT  BLOCK ...OK');
                                % READING THE SPECIFIC TRIAL "tnum"
                                
                                TempPos = ftell(fid);
                                
                                if tnum > TrialsNum,
                                        error('Invalid trial number');
                                end;
                                Status = fseek(fid, TrialBegin(tnum),-1);
                                if Status ~=0,
                                        error('File format error');
                                end;
                                
                                %--------- READING THE TRIAL HEADER ----------------------------------------
                                
                                blockSTART		        =    ftell(fid);
                                
                                SerNum                          =    fread(fid,1,'ushort');
                                TrialHeaderLength               =    fread(fid,1,'ushort');
                                ParamBlockNum                   =    fread(fid,1,'ushort');
                                DataBlockNum                    =    fread(fid,1,'ushort');
                                ParamBlockLength                =    (fread(fid,ParamBlockNum,'ushort'))';
                                DataBlockLength                 =    (fread(fid,DataBlockNum,'ushort'))';
                                
                                if unisplit('NoSeparator',fid)
                                        strerr = ['FORMAT ERROR IN THE TRIAL HEADER: ', filename, ' skipped'];
                                        errordlg(strerr, 'ERROR');
                                        %	    fclose(fid);
                                        return;
                                end;
                                
                                %disp('TRIAL HEADER DATA ...OK');
                                
                                %--------- READING THE TRIAL PARAMETERS ----------------------------------------
                                blockSTART     	   		=    ftell(fid);
                                
                                TrialTime               =    (setstr(fread(fid,10, 'char')))';
                                TrialDuration           =    fread(fid,1,'ushort');
                                OneStimDuration         =    fread(fid,1,'ushort');
                                TimeBetweenStim         =    fread(fid,1,'ushort');
                                Tilt                    =    fread(fid,1,'ushort');
                                RadialBoxSize           =    fread(fid,1,'ushort');
                                PerpenBoxSize           =    fread(fid,1,'ushort');
                                XStartPos               =    fread(fid,1,'ushort');
                                YStartPos               =    fread(fid,1,'ushort');
                                ExtentStimMotion        =    fread(fid,1,'short');
                                StimVelocity            =    fread(fid,1,'short');
                                ColorCode               =    fread(fid,1,'ushort');
                                FgrRedInt               =    fread(fid,1,'float');
                                FgrGreenInt             =    fread(fid,1,'float');
                                FgrBlueInt              =    fread(fid,1,'float');
                                BckgrRedInt             =    fread(fid,1,'float');
                                BckgrGreenInt           =    fread(fid,1,'float');
                                BckgrBlueInt            =    fread(fid,1,'float');
                                ElemRedInt              =    fread(fid,1,'float');
                                ElemGreenInt            =    fread(fid,1,'float');
                                ElemBlueInt             =    fread(fid,1,'float');
                                %               * * Applicable to Sinusoid & Gabor:  * *
                                SpatialFreq             =    fread(fid,1,'float');
                                PhRed                   =    fread(fid,1,'ushort');
                                PhGreen                 =    fread(fid,1,'ushort');
                                PhBlue                  =    fread(fid,1,'ushort');
                                StnDiv                  =    fread(fid,1,'float');
                                Contrast                =    fread(fid,1,'float');
                                TempFreq                =    fread(fid,1,'float');
                                %            * * Applicable to REGULAR: * *
                                ElemLength              =    fread(fid,1,'float');
                                ElemWidth               =    fread(fid,1,'float');
                                SpLengthDirec           =    fread(fid,1,'float');
                                SpWidthDirect           =    fread(fid,1,'float');
                                %            * * Additional timing values: * *
                                StartEyeData            =    fread(fid,1,'float');
                                StartSpikeData          =    fread(fid,1,'float');
                                EndSpikeData            =    fread(fid,1,'float');
                                TimeCode                =    fread(fid,1,'ushort');
                                StimTempType            =    fread(fid,1,'ushort');
                                StimSpatType            =    fread(fid,1,'ushort');
                                EyeChoice               =    fread(fid,1,'ushort');
                                %            * * Additional positioning info: * *
                                SweepFract              =    fread(fid,1,'float');
                                TrigTech                 =  fread(fid,1,'short');
                                SpikeTrig                =  fread(fid,1,'float');
                                ShapeTrig                =  fread(fid,1,'float');
                                ShapeHist                =  fread(fid,1,'float');
                                ShapeLen                 =  fread(fid,1,'short');
                                ShapeTrigValue		 =  fread(fid,1,'short');
                                
                                %new addition 7/97
                                
                                FirstStimTime 	 	=    unifread([fid, 1,blockSTART,ParamBlockLength],'short');
                                AngleFix 		=    unifread([fid, 1,blockSTART,ParamBlockLength],'float');
                                FixChangeTime 	 	=    unifread([fid, 1,blockSTART,ParamBlockLength],'float');
                                SteadyFixTime 	 	=    unifread([fid, 1,blockSTART,ParamBlockLength],'float');
                                FixNoChangeTime 	=    unifread([fid, 1,blockSTART,ParamBlockLength],'float');
                                TotalFixDistance 	=    unifread([fid, 1,blockSTART,ParamBlockLength],'float');
                                FixFraction 		=    unifread([fid, 1,blockSTART,ParamBlockLength],'float');
                                
                                
                                if unisplit('NoSeparator',fid)
                                        strerr = ['FORMAT ERROR IN THE TRIAL PARAMETERS: ', filename, ' skipped'];
                                        errordlg(strerr, 'ERROR');
                                        fclose(fid);
                                        return;
                                end;
                                %disp('TRIAL PARAMETERS  ...OK');
                                
                                if ~(strcmp(RunModName(1:3),'sys') | strcmp(RunModName(1:3),'no ')),	% old system
                                        emformat = 'short';
                                        emlen = 2;
                                        spformat = 'long';
                                        shformat = 'long';
                                else					% new system (TDT-VSG)
                                        emformat = 'float32';
                                        emlen = 4;
                                        spformat = 'float32';
                                        shformat = 'float32';
                                end
                                
                                %--------- MEMORY ALLOCATION -----------------
                                MaxSampNum =max(DataBlockLength(1)/emlen, DataBlockLength(2)/emlen); 
                                HorizEyePos = zeros(MaxSampNum,1);
                                VertEyePos = zeros(MaxSampNum,1);
                                %Stimulus =  zeros(ceil(TrialDuration/EyePer),1);
                                SpikeArrTime = zeros(DataBlockLength(3)/4,1);
                                
                                
                                
                                %--------- READING THE HORIZONTAL EYE POSITION ----------------------------------------
                                
                                RawHorizEyePos             =    (fread(fid,DataBlockLength(1)/emlen,emformat))';
                                
                                if unitret('NoSeparator',fid)
                                        strerr = ['FORMAT ERROR on Hor. Eye Movement: ', filename, ' skipped'];
                                        unitret('errormessage',strerr,'ERROR');
                                        fclose(fid);
                                        return;
                                end;
                                
                                
                                %----------- READING THE VERTICAL EYE POSITION ---------------------------------------
                                
                                RawVertEyePos              =    (fread(fid,DataBlockLength(2)/emlen,emformat))';
                                
                                if unitret('NoSeparator',fid)
                                        strerr = ['FORMAT ERROR on Vert. Eye Movement: ', filename, ' skipped'];
                                        unitret('errormessage',strerr,'ERROR');
                                        fclose(fid);
                                        return;
                                end;
                                
                                HorizEyePos = (RawHorizEyePos - EyetrackArbV0.*ones(size(RawHorizEyePos)))./(HorizEyePosGain*EyetrackArbDef);
                                % disp('HORIZONTAL EYE POSITION  ...OK');
                                
                                VertEyePos = (RawVertEyePos - EyetrackArbV0.*ones(size(RawVertEyePos)))./(VertEyePosGain*EyetrackArbDef);
                                % disp('VERTICAL EYE POSITION ...OK');
                                
                                % ------------ READING THE SPIKES DATA -------------------------------------------
                                
                                RawSpikeArrTime            =    (fread(fid,DataBlockLength(3)/4,spformat))';
                                if unisplit('NoSeparator',fid)
                                        strerr = ['FORMAT ERROR IN THE SPIKES DATA: ', filename, ' skipped'];
                                        errordlg(strerr, 'ERROR');
                                        fclose(fid);
                                        return;
                                end;
                                
                                
                                % disp('SPIKE ARRIVAL TIMES  ...OK');
                                % disp('OK');
                                
                                if DefValues(39) & emlen == 2, % old system, reject spikes in shape window time
                                        
                                        % ============================================
                                        % Shapeless spikes reject algorithm (18.03.99)
                                        % ============================================
                                        SpikeArrTime = SpikePer.*RawSpikeArrTime;
                                        W = ShapeLen*ShapePer;
                                        shape_ind = [1]; % Indexes of accepted spikes (with shapes)
                                        last = 1; % length of accepted indexes vector 
                                        for i=2:length(SpikeArrTime),
                                                if ( SpikeArrTime(i) - SpikeArrTime(shape_ind(last)) ) >= W,
                                                        shape_ind = [shape_ind i];
                                                        last = last + 1;
                                                end
                                        end
                                        
                                        RawSpikeArrTime = RawSpikeArrTime(shape_ind);    
                                        
                                        
                                        
                                        %%% previous version %%%
                                        % not_reject_ind = [1 find(diff(SpikePer.*RawSpikeArrTime) >= ShapeLen*ShapePer)+1];
                                        %% reject_ind = [find(diff(SpikePer.*RawSpikeArrTime) < ShapeLen*ShapePer)+1]
                                        % RawSpikeArrTime = RawSpikeArrTime(not_reject_ind);
                                        %%%
                                end
                                
                                
                                SpikeArrTime = SpikePer.*RawSpikeArrTime;
                                
                                if emlen==2,	% old system
                                        %------Charley's bug!!!
                                        if (TrialDuration < max(SpikeArrTime)) | (TrialDuration > 10000), 
                                                TrialDuration = max(SpikeArrTime); 
                                        end;
                                end
                                
                                if (~isnan(ShapePer) ) %%%!!!
                                        
                                        % ------------ READING THE SPIKE SHAPE TIMING DATA ------------------------------------------------
                                        
                                        RawShapeArrTime            =    (fread(fid,DataBlockLength(4)/4,shformat))';
                                        
                                        if unitret('NoSeparator',fid)
                                                strerr = ['FORMAT ERROR on SHAPE TIMING : file ', filename, ' skipped'];
                                                unitret('errormessage',strerr,'ERROR');
                                                fclose(fid);
                                                return;
                                        end;
                                        
                                        ShapeArrTime = ShapePer.*RawShapeArrTime;
                                        %disp('SHAPE ARRIVAL TIMES  ...OK');
                                        
                                        
                                        % ------------ READING THE SPIKE SHAPE VALUES DATA ------------------------------------------------
                                        
                                        ShapeAmp            =    (fread(fid,DataBlockLength(5)/2,'short'))';
                                        if unitret('NoSeparator',fid)
                                                strerr = ['FORMAT ERROR on SHAPE VALUES: file ', filename, ' skipped'];
                                                unitret('errormessage',strerr,'ERROR');
                                                fclose(fid);
                                                return;
                                        end;
                                        
                                        
                                        %disp('SHAPE AMPLITUDE ...OK');
                                        %----------------- END READING TRIAL ---------------------------------------------
                                        
                                end;		% of Read Shape?
                                
                                
                                
                                if 0	% old code 
                                        %--------- MEMORY ALLOCATION -----------------
                                        MaxSampNum =max(DataBlockLength(1)/2, DataBlockLength(2)/2); 
                                        HorizEyePos = zeros(MaxSampNum,1);
                                        VertEyePos = zeros(MaxSampNum,1);
                                        %Stimulus =  zeros(ceil(TrialDuration/EyePer),1);
                                        SpikeArrTime = zeros(DataBlockLength(3)/4,1);
                                        %--------- READING THE HORIZONTAL EYE POSITION ----------------------------------------
                                        
                                        RawHorizEyePos             =    (fread(fid,DataBlockLength(1)/2,'short'))';
                                        
                                        if unisplit('NoSeparator',fid)
                                                strerr = ['FORMAT ERROR IN THE HORIZONTAL EYE POSITION : ', filename, ' skipped'];
                                                errordlg(strerr, 'ERROR');
                                                fclose(fid);
                                                return;
                                        end;
                                        
                                        HorizEyePos = (RawHorizEyePos - EyetrackArbV0.*ones(size(RawHorizEyePos)))./(HorizEyePosGain*EyetrackArbDef);
                                        %disp('HORIZONTAL EYE POSITION  ...OK');
                                        
                                        %----------- READING THE VERTICAL EYE POSITION ---------------------------------------
                                        
                                        RawVertEyePos              =    (fread(fid,DataBlockLength(2)/2,'short'))';
                                        
                                        if unisplit('NoSeparator',fid)
                                                strerr = ['FORMAT ERROR IN THE VERTICAL EYE POSITION: ', filename, ' skipped'];
                                                errordlg(strerr, 'ERROR');
                                                fclose(fid);
                                                return;
                                        end;
                                        
                                        VertEyePos = (RawVertEyePos - EyetrackArbV0.*ones(size(RawVertEyePos)))./(VertEyePosGain*EyetrackArbDef);
                                        
                                        %disp('VERTICAL EYE POSITION ...OK');
                                        
                                        % ------------ READING THE SPIKES DATA ------------------------------------------------
                                        
                                        RawSpikeArrTime            =    (fread(fid,DataBlockLength(3)/4,'long'))';
                                        if unisplit('NoSeparator',fid)
                                                strerr = ['FORMAT ERROR IN THE SPIKES DATA: ', filename, ' skipped'];
                                                errordlg(strerr, 'ERROR');
                                                fclose(fid);
                                                return;
                                        end;
                                        
                                        %disp('SPIKE ARRIVAL TIMES  ...OK');
                                        % disp('OK');
                                        
                                        
                                        
                                        
                                        
                                        if DefValues(39), % reject spikes in shape window time
                                                
                                                % ============================================
                                                % Shapeless spikes reject algorithm (18.03.99)
                                                % ============================================
                                                SpikeArrTime = SpikePer.*RawSpikeArrTime;
                                                W = ShapeLen*ShapePer;
                                                shape_ind = [1]; % Indexes of accepted spikes (with shapes)
                                                last = 1; % length of accepted indexes vector 
                                                for i=2:length(SpikeArrTime),
                                                        if ( SpikeArrTime(i) - SpikeArrTime(shape_ind(last)) ) >= W,
                                                                shape_ind = [shape_ind i];
                                                                last = last + 1;
                                                        end
                                                end
                                                
                                                RawSpikeArrTime = RawSpikeArrTime(shape_ind);    
                                                
                                                
                                                
                                                
                                                
                                                
                                                %%% previous version %%%
                                                % not_reject_ind = [1 find(diff(SpikePer.*RawSpikeArrTime) >= ShapeLen*ShapePer)+1];
                                                %% reject_ind = [find(diff(SpikePer.*RawSpikeArrTime) < ShapeLen*ShapePer)+1]
                                                % RawSpikeArrTime = RawSpikeArrTime(not_reject_ind);
                                                %%%
                                        end
                                        
                                        SpikeArrTime = SpikePer.*RawSpikeArrTime;
                                        
                                        
                                        
                                        
                                        %------Charley's bug!!!
                                        if (TrialDuration < max(SpikeArrTime)) | (TrialDuration > 10000), 
                                                TrialDuration = max(SpikeArrTime); 
                                        end;
                                        
                                        % ------------ READING THE SPIKE SHAPE TIMING DATA ------------------------------------------------
                                        
                                        RawShapeArrTime            =    (fread(fid,DataBlockLength(4)/4,'long'))';
                                        
                                        if unisplit('NoSeparator',fid)
                                                strerr = ['FORMAT ERROR IN THE SPIKE SHAPE TIMING DATA: file ', filename, ' skipped'];
                                                errordlg(strerr, 'ERROR');
                                                fclose(fid);
                                                return;
                                        end;
                                        
                                        ShapeArrTime = ShapePer.*RawShapeArrTime;
                                        %disp('SHAPE ARRIVAL TIMES  ...OK');
                                        
                                        
                                        % ------------ READING THE SPIKE SHAPE VALUES DATA ------------------------------------------------
                                        
                                        
                                        ShapeAmp            =    (fread(fid,DataBlockLength(5)/2,'short'))';
                                        if unisplit('NoSeparator',fid)
                                                strerr = ['FORMAT ERROR IN THE SPIKE SHAPE VALUES DATA: file ', filename, ' skipped'];
                                                errordlg(strerr, 'ERROR');
                                                fclose(fid);
                                                return;
                                        end;
                                        
                                        %disp('SHAPE AMPLITUDE ...OK');
                                        %----------------- END READING TRIAL ---------------------------------------------
                                        
                                end	% of old code
                                
                                % CONSIDER THE TIME AXIS... ---------------------------------------------------------
                                EyeTimeAxis = StartEyeData + EyePer.*[0:length(HorizEyePos)-1];
                                if DataBlockLength(3),
                                        
                                        % STORING SPIKES DATA
                                        nrow = size(SPIKES,1);                %=TrialsNum of the given file
                                        ncol = size(SPIKES,2);
                                        
                                        %%%  SpikesNum = DataBlockLength(3)/4; 
                                        SpikesNum = length(SpikeArrTime);
                                        nncol = max((SpikesNum +3), ncol);
                                        tmp = zeros(tnum,nncol);
                                        
                                        for i=1:nrow,
                                                tmp(i,1:ncol) = SPIKES(i,:);
                                        end;
                                        SPIKES = [];
                                        SPIKES = tmp;
                                        
                                        %%%  SPIKES(tnum,1) = DataBlockLength(3)/4;
                                        SPIKES(tnum,1) = SpikesNum;
                                        %          SHNUM = SPIKES(tnum,1);
                                        SPIKES(tnum,2) = StartSpikeData;
                                        SPIKES(tnum,3) = EndSpikeData;
                                        
                                        SPIKES(tnum,4:length(SPIKES(tnum,:))) = zeros(1,length(SPIKES(tnum,:))-3);   
                                        SPIKES(tnum,4:SpikesNum + 3) = SpikeArrTime;
                                        
                                        MOTION(tnum,1) = StimVelocity+0.0001; % sometimes, the velocity is zeroed
                                        MOTION(tnum,2) = Tilt;
                                        MOTION(tnum,3) = PerpenBoxSize;
                                        
                                        %---------- HISTOGRAM COMPUTING -------------------------
                                        if DefValues(1)
                                                bins=(StartSpikeData:DefValues(1):EndSpikeData);
                                                [SpikeHist,N]=hist(SpikeArrTime,bins);
                                                [xx,yy] = bar(N,SpikeHist);
                                                
                                        else
                                                xx = zeros(1,3*SpikesNum);
                                                xx(1:3:(3*SpikesNum-2)) = SPIKES(tnum,4:(SpikesNum + 3));
                                                xx(2:3:(3*SpikesNum-1)) = SPIKES(tnum,4:(SpikesNum + 3));
                                                xx(3:3:(3*SpikesNum))   = SPIKES(tnum,4:(SpikesNum + 3));
                                                yy = zeros(1,3*SpikesNum);
                                                yy(2:3:(3*SpikesNum-1)) = ones(1,SpikesNum); 
                                        end;    
                                end;
                                
                                
                                % STORING STIMULUS DATA;
                                
                                % IF TIME BETWEEN STIMULUS = 0....
                                if TimeBetweenStim == 0,
                                        TimeBetweenStim = OneStimDuration;
                                end;
                                % IF FLAG 'NO MOTION' WAS OBSERVED;
                                if StimTempType==0,             % no motion
                                        StimArrTime=0;
                                        StimNum = 1;
                                        OneStimDuration=0;
                                        TimeBetweenStim=TrialDuration;
                                end;
                                % IF STIMULUS DURATION = 0;
                                if (OneStimDuration+TimeBetweenStim==0)
                                        StimArrTime=0;
                                        StimNum = 1;
                                        OneStimDuration=0;
                                        TimeBetweenStim=TrialDuration;
                                end;
                                StimArrTime = [0:OneStimDuration+TimeBetweenStim:TrialDuration-1];
                                StimNum = length(StimArrTime);
                                
                                ncol = size(STIMULUS,2);
                                nrow = size(STIMULUS,1);                %=TrialsNum of the given file
                                
                                % put new stimuli vector to the global STIMULUS array    
                                nncol = max((StimNum +4), ncol);
                                tmp = zeros(tnum,nncol);
                                
                                for i=1:nrow,
                                        tmp(i,1:ncol) = STIMULUS(i,:);
                                end;
                                STIMULUS = [];
                                STIMULUS = tmp;
                                
                                
                                STIMULUS(tnum,1) = StimNum;
                                STIMULUS(tnum,2) = TrialDuration;
                                STIMULUS(tnum,3) = OneStimDuration;
                                STIMULUS(tnum,4) = TimeBetweenStim;
                                STIMULUS(tnum,5:(StimNum + 4)) = StimArrTime;
                                Front = 0;
                                
                                SpStim = round(OneStimDuration/EyePer); 
                                SpPause = round(TimeBetweenStim/EyePer);
                                SpStimAll = ceil(TrialDuration/EyePer);
                                if(SpStimAll <= 1),
                                        axis off;
                                        text(.3, .5, 'File has no spike data');
                                        return;
                                end;
                                %---------------- Shape Buffer:
                                
                                ShapeBufSize = length(ShapeAmp)/ShapeLen;
                                ShapeBuf = zeros(ShapeBufSize,ShapeLen);
                                for sh = 0:ShapeBufSize-1,
                                        ShapeBuf(sh+1,:) = ShapeAmp(sh*ShapeLen+1:(sh+1)*ShapeLen);
                                end;
                                
                                SHNUM=ShapeBufSize;
                                
                                if DefValues(36),					%if shape alingment is ok        
                                        OrigShapeBuf = ShapeBuf;
                                        ShapeBuf = unisplit('shift',ShapeBuf);      
                                        ShapeLen = size(ShapeBuf,2);
                                        ShapeBufSize = size(ShapeBuf,1);
                                        ShapeAmp = reshape(ShapeBuf', 1,ShapeBufSize*ShapeLen);
                                end;
                                
                                
                                % --------------------drawing:
                                ax = axes('Position',[.04 .85 .42 .125], 'XLim', [0 STIMULUS(tnum,2)], 'YLim',[DefValues(7) DefValues(10)]);
                                if DefValues(24), 
                                        set(ax,'XGrid','on','Xtick',[0:100:STIMULUS(tnum,2)],...
                                                'XtickLabelMode','manual','XTickLabels',[]);  
                                end;
                                AXES_DAT(1) = ax;
                                
                                % PLOTTING THE EYE MOVEMENT
                                
                                HorEyeMov=line(EyeTimeAxis,HorizEyePos,'Color',[0 .502 0]);
                                VerEyeMov=line(EyeTimeAxis,VertEyePos,'Color','m','LineWidth',1);
                                
                                
                                % in order to draw eye movement in selected segment, we must store the pointers to the eye 
                                % movements lines as userdata of axes.
                                
                                set(ax,'UserData',[HorEyeMov VerEyeMov]);
                                
                                
                                %---------------- SPIKES HISTOGRAM DRAWING --------------------------------------
                                if SpikesNum,
                                        
                                        SpikeAmp = (DefValues(10) - DefValues(7)) / (6*max(yy));
                                        h = line(xx,SpikeAmp*yy+DefValues(7),'Color','r','LineWidth',.5);
                                        
                                end;
                                
                                %---------------- STIMULI DRAWING ----------------------------------------------
                                if (OneStimDuration > 100)             % avoid multiple drawing while flash
                                        for i=1:StimNum,
                                                Slope = Front+SpStim;
                                                if Slope > SpStimAll;
                                                        Slope = SpStimAll;
                                                end;
                                                
                                                x = [StimArrTime(i) (OneStimDuration+StimArrTime(i)-1)];
                                                y = [ones(size(x))+DefValues(7)];
                                                line(x,y,'Color',[0 .5 0],'LineWidth',1.5);
                                                
                                                x = [(StimArrTime(i)+OneStimDuration) (StimArrTime(i)+OneStimDuration+TimeBetweenStim-1)];
                                                y = [ones(size(x))+DefValues(7)];
                                                line(x,y,'Color','b','LineWidth',3.5);
                                                
                                                Front = Slope+SpPause;
                                        end;
                                end;
                                
                                % ---------- draw +/- 20% grid lines
                                
                                line([0 STIMULUS(tnum,2)],[DefValues(10)/5 DefValues(10)/5],...
                                        'Color','k','LineStyle',':'); 
                                line([0 STIMULUS(tnum,2)],[DefValues(7)/5 DefValues(7)/5],...
                                        'Color','k','LineStyle',':'); 
                                
                                set(text,'String',['Trial #',num2str(tnum),', ',num2str(SpikesNum), ' spk' ],...
                                        'units','normal','Position',[.2 .9],'FontAngle','oblique');
                                
                                %----------------- draw Shape Strips   
                                
                                
                                %        arg2 = [length(SpikeArrTime), SpikeArrTime, length(ShapeAmp)/ShapeLen, ShapeAmp];
                                AXES_DAT(2) = axes('Position',[.04 .58 .42 .225]);
                                
                                
                                drawshapenum = min(DefValues(13),size(ShapeBuf,1));
                                if (drawshapenum)
                                        plot(ShapeBuf(1:drawshapenum,:)','k');
                                        set(gca,'XLim',[1 size(ShapeBuf,2)]);
                                        
                                        %----------------------------------------
                                        
                                        if ~DefValues(35),
                                                
                                                
                                                %------------------Writing for klt in C ---------------
                                                
                                                
                                                [fidtmp, message] = fopen('shape.tmp','wb');
                                                if(fidtmp<0), 
                                                        error(message);
                                                        return;
                                                end;
                                                fwrite(fidtmp,[ShapeBufSize, ShapeLen, ShapeAmp],'short');
                                                fclose(fidtmp);
                                                
                                                %----------------- KLT:
                                                % MATLAB KLT SUBROUTINE: takes extremely long time > 5 minutes
                                                %    [template_spk]     = spike3('norm_template_spk',ShapeBuf);
                                                %    [C]                = spike3('cross_corr',template_spk);
                                                %    [s1, s2, s3]       = spike3('KL_decomposition',C);   
                                                %    K1 = s1'*ShapeBuf';
                                                %    K2 = s2'*ShapeBuf';
                                                
                                                % klt.exe: program for KLT sorting. Accepts template matrix in format as
                                                % follows: [ShapeBufSize, ShapeLen, ShapeAmp], named as shape.tmp binary file
                                                % Produces the ascii file basis.dat. This file has 2 columns, corresponded to 
                                                % to 2 eigenvectors.
                                                
                                                cur_path = pwd;
                                                if (length(ShapeAmp)) 
                                                        eval(['!' APP_PATH 'klt'])
                                                        hw = 0;
                                                        while(exist([cur_path '\basis.dat'])~=2)
                                                                if hw == 0
                                                                        hw = uniwaitbar(0,'Waiting for klt to finish...');
                                                                else
                                                                        uniwaitbar(1,hw);
                                                                end
                                                        end
                                                        if hw~=0
                                                                close(hw)
                                                        end
                                                        load([cur_path '\basis.dat'],'-ascii');
                                                        
                                                        %        load template.dat;
                                                        size(basis);
                                                        size(ShapeBuf);
                                                        K = basis'*ShapeBuf';
                                                        %         K = basis'*template';
                                                        delete shape.tmp;
                                                        delete([cur_path '\basis.dat']);
                                                else
                                                        K=[0; 0];
                                                end;        
                                                
                                                
                                                %-------------9999
                                                
                                                unisplit('cluster',K');
                                        else,
                                                % %Kohonen method
                                                
                                                disp('Kohonen')
                                                
                                                unisplit('kohonen',ShapeBuf);
                                                
                                                
                                        end;  
                                end;      
                                
                        end;
                end;     
        end;
        
        
        
        if TrialsNum > DefValues(4)
                if nextfig <= TrialsNum,
                        uicontrol(Fig2, 'Units','Normalized','Position',[.9 .025, .08 .05],'String','>>',...
                                'Callback',['unisplit(''redraw_exp'',', num2str(nextfig), ');']);
                end;
                
                if in1 > DefValues(4)
                        uicontrol(Fig2, 'Units','Normalized','Position',[.8 .025, .08 .05],'String','<<',...
                                'Callback',['unisplit(''redraw_exp'',', num2str(prevfig), ');']);
                        
                end;
        end;
        
        
        
        set(SPLIT_DAT(1),'Pointer','Arrow');
        set(SPLIT_DAT(2),'Pointer','Arrow','Name',filename);
        
        if DefValues(36),					%restore orig values        
                ShapeBuf = OrigShapeBuf;
                ShapeLen = size(ShapeBuf,2);
                ShapeBufSize = size(ShapeBuf,1);
                ShapeAmp = reshape(ShapeBuf', 1,ShapeBufSize*ShapeLen);
        end;
        
        save tmp
        
        %     unisplit('figname');
        set(SPLIT_DAT(2),'KeyPressFcn','');
        set(SAVE_AS_SORTED_HNDL,'Enable','on');
        
        
        
        
        uicontrol(Fig2,'Style','checkbox','Units','normalized','Position',[.9 .075 .1 .020],...
                'BackgroundColor','w','String','auto >>',...
                'ForegroundColor','b','value',DefValues(45),'enable','on','callback',...
                ['val=get(gco,''Value'');',...
                        'ud=get(0,''UserData''); ud(45) = val; set(0,''userdata'',ud);']);
	
	uicontrol(Fig2,'Style','checkbox','Units','normalized','Position',[.8 .075 .1 .020],...
                'BackgroundColor','w','String','semi-auto',...
                'ForegroundColor','b','value',DefValues(46),'enable','on','callback',...
                ['val=get(gco,''Value'');',...
                        'ud=get(0,''UserData''); ud(46) = val; set(0,''userdata'',ud);']);
		
        drawnow;
        if DefValues(45),
                for i = 1:size(C,1),
                        unisplit('assign_cntr',C(i,:));
                end
                unisplit('fcm_sort');
                drawnow;
		if DefValues(46),	% semi-auto
			ButtonName=questdlg('Proceed with this ..?', ...
                        'UNISPLIT (semi-auto mode)', ...
                        'Yes','No','Yes');
  
			switch ButtonName,
				case 'Yes', 
					for i = 1:size(C,1),
						unisplit('merge_select',i);
						drawnow;
						unisplit('save',i);
					end
					if nextfig <= TrialsNum,
						unisplit('redraw_exp',nextfig);
					end
			
				case 'No',
					DefValues(45) = 0;
					set(0,'UserData',DefValues);
					C = [];
					unisplit('redraw_exp',tnum); % Re-sort
					
			end
		else
			for i = 1:size(C,1),
				unisplit('merge_select',i);
				drawnow;
				unisplit('save',i);
			end
			if nextfig <= TrialsNum,
				unisplit('redraw_exp',nextfig);
			end

		end
		
        end
        
        if nextfig > TrialsNum,
                DefValues(45) = 0;
                set(0,'UserData',DefValues);
        end
        
        %-------------------------------------------------------------------------------------------------------------------------------------------------------------     
elseif(strcmp(action,'update'))

		set(gcf,'Pointer','Watch');
        
        if (in1),
                if nargin < 3,
                        in3 = 0;
                end
                
                h=get(gcf,'CurrentObject');
                
                
                newdata = str2num(deblank(get(h,'String')));
                if in3,
                        DefValues(in1) = newdata;
                        set(0,'UserData',DefValues);
                else
                        a = newdata >= DefValues(in1+1);
                        b = newdata <= DefValues(in1+2);
                        if (a&b),
                                DefValues(in1) = newdata;
                                set(0,'UserData',DefValues);       
                        else,
                                set(h,'String',num2str(DefValues(in1)));
                        end;
                end;
        else,          %redraw                
                unisplit('redraw_exp',in2); 
        end;
        set(gcf,'Pointer','Arrow');
        set(SPLIT_DAT(1),'Pointer','Arrow');
        
        
        
        %---------------------------------------------------------------------------------------------------------------------------------------------------------------     
        
elseif(strcmp(action,'fulldump'))
        
        
        trials_inf = get(SPLIT_DAT(1),'UserData');
        fid = trials_inf(1);
        TrialsNum = trials_inf(2);
        TrialBegin = trials_inf(3:TrialsNum+2);
        
        if in1==0,
                str = ['1'];
                if TrialsNum >1,    
                        for(i=2:TrialsNum),
                                str = [str, '|', num2str(i)];      
                        end;
                end;
                Fig1 = figure('Backingstore','on','Color','c',...
                        'Units','Normal','Position',[0.01 0.04 .4 .9],'Resize','Off', 'Name',...
                        'Trial Dump','MenuBar','none','Number','off',...
                        'DefaultAxesPosition',[0 0 1 1]);
                set(gcf,'Pointer','Watch');
                
                
                uicontrol(Fig1,'Style','text','Position',[.2 .96 .3 .03],'Backgroundcolor','c','Foregroundcolor','b',...
                        'string','TRIAL #:','units','normal');                  
                
                uicontrol(Fig1,'Style','Popupmenu','Position',[.5 .95 .3 .5],'Units','Normalized','String',str,'Callback',...
                        ['l = get(gcf, ''CurrentObject''); s = get(l,''value''); unisplit(''fulldump'',s);']);
                uicontrol(Fig1,'Style','Pushbutton','Units','Normalized','String','OK','Position',[.3 .02 .4 .03],...
                        'Callback','delete(gcf);');
                
                unisplit('fulldump',1);
        else,
                oldpos = ftell(fid);
                tnum = in1;
                
                if tnum > TrialsNum,
                        error('Invelid trial number');
                end;
                Status = fseek(fid, TrialBegin(tnum),-1);
                if Status ~=0,
                        error('File format error');
                end;
                
                %--------- READING THE TRIAL HEADER ----------------------------------------
                
                blockSTART		        =    ftell(fid);
                
                SerNum                          =    fread(fid,1,'ushort');
                TrialHeaderLength               =    fread(fid,1,'ushort');
                ParamBlockNum                   =    fread(fid,1,'ushort');
                DataBlockNum                    =    fread(fid,1,'ushort');
                ParamBlockLength                =    (fread(fid,ParamBlockNum,'ushort'))';
                DataBlockLength                 =    (fread(fid,DataBlockNum,'ushort'))';
                
                if unisplit('NoSeparator',fid)
                        strerr = ['FORMAT ERROR : ', filename, ' skipped'];
                        errordlg(strerr, 'ERROR');
                        fclose(fid);
                        return;
                end;
                
                % disp('COMMON TRIAL DATA ...OK');
                %--------- READING THE TRIAL PARAMETERS ----------------------------------------
                blockSTART     	   		=    ftell(fid);
                
                TrialTime               =    (setstr(fread(fid,10, 'char')))';
                TrialDuration           =    fread(fid,1,'ushort');
                OneStimDuration         =    fread(fid,1,'ushort');
                TimeBetweenStim         =    fread(fid,1,'ushort');
                Tilt                    =    fread(fid,1,'ushort');
                RadialBoxSize           =    fread(fid,1,'ushort');
                PerpenBoxSize           =    fread(fid,1,'ushort');
                XStartPos               =    fread(fid,1,'ushort');
                YStartPos               =    fread(fid,1,'ushort');
                ExtentStimMotion        =    fread(fid,1,'ushort');
                StimVelocity            =    fread(fid,1,'short');
                ColorCode               =    fread(fid,1,'short');
                FgrRedInt               =    fread(fid,1,'float');
                FgrGreenInt             =    fread(fid,1,'float');
                FgrBlueInt              =    fread(fid,1,'float');
                BckgrRedInt             =    fread(fid,1,'float');
                BckgrGreenInt           =    fread(fid,1,'float');
                BckgrBlueInt            =    fread(fid,1,'float');
                ElemRedInt              =    fread(fid,1,'float');
                ElemGreenInt            =    fread(fid,1,'float');
                ElemBlueInt             =    fread(fid,1,'float');
                %               * * Applicable to Sinusoid & Gabor:  * *
                SpatialFreq             =    fread(fid,1,'float');
                PhRed                   =    fread(fid,1,'ushort');
                PhGreen                 =    fread(fid,1,'ushort');
                PhBlue                  =    fread(fid,1,'ushort');
                StnDiv                  =    fread(fid,1,'float');
                Contrast                =    fread(fid,1,'float');
                TempFreq                =    fread(fid,1,'float');
                %            * * Applicable to REGULAR: * *
                ElemLength              =    fread(fid,1,'float');
                ElemWidth               =    fread(fid,1,'float');
                SpLengthDirec           =    fread(fid,1,'float');
                SpWidthDirect           =    fread(fid,1,'float');
                %            * * Additional timing values: * *
                StartEyeData            =    fread(fid,1,'float');
                StartSpikeData          =    fread(fid,1,'float');
                EndSpikeData            =    fread(fid,1,'float');
                TimeCode                =    fread(fid,1,'ushort');
                StimTempType            =    fread(fid,1,'ushort');
                StimSpatType            =    fread(fid,1,'ushort');
                EyeChoice               =    fread(fid,1,'ushort');
                %            * * Additional positioning info: * *
                SweepFract               =    fread(fid,1,'float');
                TrigTech                 =  fread(fid,1,'short');
                SpikeTrig                =  fread(fid,1,'float');
                ShapeTrig                =  fread(fid,1,'float');
                ShapeHist                =  fread(fid,1,'float');
                ShapeLen                 =  fread(fid,1,'short');
                ShapeTrigValue		 =  fread(fid,1,'short');
                
                %new addition 7/97
                
                FirstStimTime 	 	=    unifread([fid, 1,blockSTART,ParamBlockLength],'short');
                AngleFix 		=    unifread([fid, 1,blockSTART,ParamBlockLength],'float');
                FixChangeTime 	 	=    unifread([fid, 1,blockSTART,ParamBlockLength],'float');
                SteadyFixTime 	 	=    unifread([fid, 1,blockSTART,ParamBlockLength],'float');
                FixNoChangeTime 	=    unifread([fid, 1,blockSTART,ParamBlockLength],'float');
                TotalFixDistance 	=    unifread([fid, 1,blockSTART,ParamBlockLength],'float');
                FixFraction 		=    unifread([fid, 1,blockSTART,ParamBlockLength],'float');
                
                
                
                
                if unisplit('NoSeparator',fid)
                        strerr = ['FORMAT ERROR : ', filename, ' skipped'];
                        errordlg(strerr, 'ERROR');
                        fclose(fid);
                        return;
                end;
                
                % disp('TRIAL PARAMETERS  ...OK');
                
                % textout on the left panel:
                cla; 
                set(text,'Position',[.02 .93],'Color','r','string','Trial Time:'); 
                set(text,'Position',[.52 .93],'Color','k','string',deblank(TrialTime)); 
                
                set(text,'Position',[.02 .91],'Color','r','string','Trial Duration:'); 
                set(text,'Position',[.52 .91],'Color','k','string',num2str(TrialDuration)); 
                
                set(text,'Position',[.02 .89],'Color','r','string','One Stimulus Duration:'); 
                set(text,'Position',[.52 .89],'Color','k','string',num2str(OneStimDuration)); 
                
                set(text,'Position',[.02 .87],'Color','r','string','Time between stimuli:'); 
                set(text,'Position',[.52 .87],'Color','k','string',num2str(TimeBetweenStim)); 
                
                set(text,'Position',[.02 .85],'Color','r','string','Tilt:'); 
                set(text,'Position',[.52 .85],'Color','k','string',num2str(Tilt)); 
                
                set(text,'Position',[.02 .83],'Color','r','string','Radial box size:'); 
                set(text,'Position',[.52 .83],'Color','k','string',num2str(RadialBoxSize)); 
                
                set(text,'Position',[.02 .81],'Color','r','string','Perp. box size'); 
                set(text,'Position',[.52 .81],'Color','k','string',num2str(PerpenBoxSize)); 
                
                set(text,'Position',[.02 .79],'Color','r','string','Start pos:'); 
                set(text,'Position',[.52 .79],'Color','k','string',...
                        [num2str(XStartPos),',',num2str(YStartPos)]); 
                
                set(text,'Position',[.02 .77],'Color','r','string','Extent stim motion:'); 
                set(text,'Position',[.52 .77],'Color','k','string', num2str(ExtentStimMotion)); 
                
                set(text,'Position',[.02 .75],'Color','r','string','Stim. velocity :'); 
                set(text,'Position',[.52 .75],'Color','k','string',num2str(StimVelocity )); 
                
                set(text,'Position',[.02 .73],'Color','r','string','ColorCode:'); 
                set(text,'Position',[.52 .73],'Color','k','string',num2str(ColorCode)); 
                
                set(text,'Position',[.02 .71],'Color','r','string','Foreground intensity:'); 
                set(text,'Position',[.52 .71],'Color','k','string',...
                        [num2str(FgrRedInt),',',num2str(FgrGreenInt), ',', num2str(FgrBlueInt)]); 
                
                set(text,'Position',[.02 .69],'Color','r','string','Background  intensity:'); 
                set(text,'Position',[.52 .69],'Color','k','string',...
                        [num2str(BckgrRedInt),',',num2str(BckgrGreenInt), ',', num2str(BckgrBlueInt)]); 
                
                set(text,'Position',[.02 .67],'Color','r','string','Element intensity:'); 
                set(text,'Position',[.52 .67],'Color','k','string',...
                        [num2str(ElemRedInt),',',num2str(ElemGreenInt), ',', num2str(ElemBlueInt)]); 
                
                set(text,'Position',[.02 .65],'Color','r','string','Spatial frequency :'); 
                set(text,'Position',[.52 .65],'Color','k','string',num2str(SpatialFreq)); 
                
                set(text,'Position',[.02 .63],'Color','r','string','Ph:'); 
                set(text,'Position',[.52 .63],'Color','k','string',...
                        [num2str(PhRed),',',num2str(PhGreen), ',', num2str(PhBlue)]); 
                
                set(text,'Position',[.02 .61],'Color','r','string','Standard diviation:'); 
                set(text,'Position',[.52 .61],'Color','k','string',num2str(StnDiv)); 
                
                set(text,'Position',[.02 .59],'Color','r','string','Contrast:'); 
                set(text,'Position',[.52 .59],'Color','k','string',num2str(Contrast)); 
                
                set(text,'Position',[.02 .57],'Color','r','string','Temporal frequency:'); 
                set(text,'Position',[.52 .57],'Color','k','string',num2str(TempFreq)); 
                
                set(text,'Position',[.02 .55],'Color','r','string','Elem length:'); 
                set(text,'Position',[.52 .55],'Color','k','string',num2str(ElemLength)); 
                
                set(text,'Position',[.02 .53],'Color','r','string','ElemWidth:'); 
                set(text,'Position',[.52 .53],'Color','k','string',num2str(ElemWidth)); 
                
                set(text,'Position',[.02 .51],'Color','r','string','Space length direc:'); 
                set(text,'Position',[.52 .51],'Color','k','string',num2str(SpLengthDirec)); 
                
                set(text,'Position',[.02 .49],'Color','r','string','Space width direct :'); 
                set(text,'Position',[.52 .49],'Color','k','string',num2str(SpWidthDirect )); 
                
                set(text,'Position',[.02 .47],'Color','r','string','Start eye data:'); 
                set(text,'Position',[.52 .47],'Color','k','string',num2str(StartEyeData)); 
                
                set(text,'Position',[.02 .45],'Color','r','string','Start spike data:'); 
                set(text,'Position',[.52 .45],'Color','k','string',num2str(StartSpikeData)); 
                
                set(text,'Position',[.02 .43],'Color','r','string','End spike data:'); 
                set(text,'Position',[.52 .43],'Color','k','string',num2str(EndSpikeData)); 
                
                set(text,'Position',[.02 .41],'Color','r','string','Time code:'); 
                set(text,'Position',[.52 .41],'Color','k','string',num2str(TimeCode)); 
                
                set(text,'Position',[.02 .39],'Color','r','string','Stim temporal type:');
                str = unisplit('translate','StimTempType',StimTempType);
                set(text,'Position',[.52 .39],'Color','k','string',str); 
                
                set(text,'Position',[.02 .37],'Color','r','string','Stim spatial type:'); 
                str = unisplit('translate','StimSpatType',StimSpatType);
                set(text,'Position',[.52 .37],'Color','k','string',str);
                
                set(text,'Position',[.02 .35],'Color','r','string','Eye choice:'); 
                str = unisplit('translate','EyeChoice',EyeChoice);
                set(text,'Position',[.52 .35],'Color','k','string',str); 
                
                set(text,'Position',[.02 .33],'Color','r','string','Sweep fraction:'); 
                set(text,'Position',[.52 .33],'Color','k','string',num2str(SweepFract)); 
                
                set(text,'Position',[.02 .31],'Color','r','string','Spike trigger:'); 
                set(text,'Position',[.52 .31],'Color','k','string',num2str(SpikeTrig)); 
                
                set(text,'Position',[.02 .29],'Color','r','string','Shape trigger:'); 
                set(text,'Position',[.52 .29],'Color','k','string',num2str(ShapeTrig)); 
                
                set(text,'Position',[.02 .27],'Color','r','string','Shape length:'); 
                set(text,'Position',[.52 .27],'Color','k','string',num2str(ShapeLen)); 
                
                set(text,'Position',[.02 .25],'Color','r','string','Shape Offset:'); 
                set(text,'Position',[.52 .25],'Color','k','string',num2str(ShapeTrigValue)); 
                
                set(text,'Position',[.02 .23],'Color','r','string','First Stim. Time, ms:'); 
                set(text,'Position',[.52 .23],'Color','k','string',num2str(FirstStimTime)); 
                
                set(text,'Position',[.02 .21],'Color','r','string','Targets angle, deg:'); 
                set(text,'Position',[.52 .21],'Color','k','string',num2str(AngleFix)); 
                
                set(text,'Position',[.02 .19],'Color','r','string','Change fix. time, sec:'); 
                set(text,'Position',[.52 .19],'Color','k','string',num2str(FixChangeTime)); 
                
                set(text,'Position',[.02 .17],'Color','r','string','Fix. time motion, sec:'); 
                set(text,'Position',[.52 .17],'Color','k','string',num2str(SteadyFixTime)); 
                
                set(text,'Position',[.02 .15],'Color','r','string','No change fix time, sec:'); 
                set(text,'Position',[.52 .15],'Color','k','string',num2str(FixNoChangeTime)); 
                
                set(text,'Position',[.02 .13],'Color','r','string','Fix. motion dist, minarc:'); 
                set(text,'Position',[.52 .13],'Color','k','string',num2str(TotalFixDistance)); 
                
                set(text,'Position',[.02 .11],'Color','r','string','Fix. fraction:'); 
                set(text,'Position',[.52 .11],'Color','k','string',num2str(FixFraction)); 
                
                
                %    drawnow;
                fseek(fid,oldpos,-1); 
        end;
        set(gca,'Visible','Off');
        set(gcf,'Pointer','Arrow');
        set(SPLIT_DAT(1),'Pointer','Arrow');
        
        
        %---------------------------------------------------------------------------------------------------------------------------------------------------------------     
        
        %---------------------------------------------------------------------------------------------------------------------------------------------------------------     
        
elseif(strcmp(action,'save'))
        
        global MERGE
        if length(MERGE) == 1,	      
                set(gcf,'Pointer','Watch');		              
                name=get(gcf,'Name');
                pnt = find(name=='.');
                save_name = [name(1:pnt),'cl',num2str(in1)];
                unisplit('write_sorted_trial',save_name, in1);
                
        end;    
        AllAxes = unisplit('get_obj',SPLIT_DAT(2),'axes');
        set(AllAxes,'color','w');
        MERGE = [];
        
        
        %---------------------------------------------------------------------------------------------------------------------------------------------------------------     
elseif(strcmp(action,'exit')),
        unitret('write_ini',INIFILE);				 
        fclose('all');
        kids = get(0,'Children');
        delete(kids);
        if exist('tmp.mat')==2
                delete tmp.mat;
        end;  
        
        
        
        
        %---------------------------------------------------------------------------------------------------------------------------------------------------------------     
elseif(strcmp(action,'translate'))
        
        in2 = in2+1;
        
        if strcmp(in1,'StimTempType'),
                strings = str2mat('No motion','Alternating','Flashing','Repeating');
                
        elseif strcmp(in1,'StimSpatType'), 
                strings  = str2mat('Solid tilted rectangular','Sinus','Gabor','6-th deriv of Gaussian',...
                        'Regular texture','Random','WIN_SINU','CNTR_SIN','MAX_PAT_TYPE');   
                
        elseif strcmp(in1,'EyeChoice'), 
                strings  = str2mat('No eyes','Left','Right','Both');
                
        elseif strcmp(in1,'StabFlag'),  
                strings  = str2mat('No stab.','Every frame','Even frames');
                
        elseif strcmp(in1,'CompFlag'),  
                strings  = str2mat('Control PC','Anal PC');
                
                
        elseif strcmp(in1,'FixationType'),
                strings  = str2mat('1 static LED','Multiple LEDs','Oscilloscope',...
                        'Stimulus target');
                
        end;
        if in2 > size(strings,1) | isnan(in2)
                out1 = 'unknown';
        else   
                out1 = deblank(strings(in2,:)); 
        end;
        %-----------------------------------------------------------------------  
elseif(strcmp(action,'NoSeparator')),
        
        Separator = (setstr(fread(in1, 4,'uchar')))';
        if Separator  == 'wwww',
                out1 = 0;
        else
                out1 = 1;
        end;
        
        %-----------------------------------------------------------------------  
elseif(strcmp(action,'get_obj'))
        parent_handler = in1;
        wanted_type = in2;
        
        kids = get(parent_handler,'Children');
        for i=1:length(kids),
                tp=get(kids(i),'type');
                if (strcmp(tp,wanted_type))
                        out1 = [out1, kids(i)];
                end;
        end;
        
        %------------------------------------------------------------------------
elseif strcmp(action,'cluster');
        global AX_HAND AXES_DAT;
        set(0,'CurrentFigure',SPLIT_DAT(2)) %%% Matlab 5 - required addition???
        
        % AX_HAND(1) = axes('Position',[.04 .025 .42 .5],'YTickLabel',[],'XTickLabels',[],...
        %        'ButtonDownFcn','unisplit(''assign_cntr'');');
        
        AX_HAND(1) = axes('Position',[.04 .025 .42 .5],...
                'ButtonDownFcn','unisplit(''assign_cntr'');');
        
        line('XData',in1(:,1),'YData',in1(:,2),'LineStyle','none','Marker','*','MarkerSize',1.5,...
                'Color','k','ButtonDownFcn','unisplit(''assign_cntr'');');
        
        if ~DefValues(40),		% not autoscale
                set(AX_HAND(1),'Xlim',[DefValues(41)DefValues(42)],'Ylim',[DefValues(43) DefValues(44)]);
        end
        
        AX_HAND(2)=uicontrol('string','Sort','callback','unisplit(''fcm_sort'');','Style','Pushbutton',...
                'Position',[.5 .025 .08 .05], 'units','norm','Userdata',in1,'visible','on');
        
        AX_HAND(3)=uicontrol('string','Isolate','callback','unisplit(''split'');','Style','Pushbutton',...
                'Position',[.5 .025 .08 .05], 'units','norm','visible','off');
        
        AXES_DAT(3) = AX_HAND(1);   
        
        %------------------------------------------------------------------------
elseif strcmp(action,'assign_cntr');
        global AX_HAND AXES_DAT;   
        set(gcf,'CurrentAxes',AX_HAND(1));
        if nargin > 1,
                PresPnt = in1;
        else
                PresPnt=get(AX_HAND(1),'CurrentPoint');
                C = [C; [PresPnt(1,1) PresPnt(1,2)]]; % CurrentPoint returns 3x2 matrix!
        end
        limx = get(AX_HAND(1),'XLim');
        limy = get(AX_HAND(1),'YLim');
        x=PresPnt(1,1);
        y=PresPnt(1,2);
        if ((x >= limx(1)) & (y >= limy(1)) & (x <= limx(2)) &(y <= limy(2)))
                line('Xdata',x,'YData',y,'Linestyle','o','Color','g','markersize',7,'erasemode','background');
                ud = get(AX_HAND(1),'UserData');        
                set(AX_HAND(1),'UserData',[ud;x y]);
        end;
        %-----------------------------------------------------------------------  
elseif strcmp(action,'fcm_sort');
        global AX_HAND AXES_DAT; 
        q=1.7; e=0.01;
        set(gcf,'CurrentAxes',AX_HAND(1));
        
        K=get(AX_HAND(2),'UserData');
        
        v=get(AX_HAND(1),'UserData');
        c=size(v,1);
        SpkNum = size(K,1);
        if (c>1),                           % more then 1 centroid      
                if (c>SpkNum), 
                        c=SpkNum; 
                end;                             % maximal number of centroids =size of data
                
                
                %  creating u(i,j) - membership degrees.
                %  d(i,j)= Euclidic distance between v(i) and K(j)  ) ^2.
                for i=1:c
                        for j=1:SpkNum
                                di (j,:)=K(j,:)- v(i,:);
                                d(i,j)= di(j,:)*di(j,:)' ;
                                up2(i,j)=d(i,j).^(-1/(q-1));
                        end
                end;
                down2=sum(up2);
                for i=1:c
                        u1(i,:)=up2(i,:)./ down2;
                end
                
                u0 = ones(size(u1));    
                
                % Iterations over centroid matrix and membership matrix.
                
                iter=0;
                while(max(max(abs(u1-u0))))>e
                        iter=iter+1;
                        u0=u1;
                        temp=u0.^q ;
                        up= temp*K; 
                        down=sum(temp'); 
                        for i=1:c
                                v(i,:)=up(i,:)/down(i);
                        end
                        for i=1:c
                                for j=1:SpkNum
                                        di (j,:)=K(j,:)- v (i,:);
                                        d(i,j)= di(j,:)*di(j,:)' ;
                                        up2(i,j)=d(i,j).^(-1/(q-1));
                                end  
                        end
                        down2=sum(up2);
                        for i=1:c
                                u1(i,:)=up2(i,:)./ down2;
                        end
                end
                
                %plotting:
                line('XData',v(:,1),'YData',v(:,2),'Linestyle','o',...
                        'color','r','erasemode','norm','markersize',7);
                title(' Clustering  -  Fuzzy   K - means ');
                
                %contours:
                %  Elliptic  contors.      
                
                sum_n=sum(u1');
                sum_q_n=sum(u1.^q');
                for i=1:c
                        sfi=0;
                        ufi=0;
                        for j=1:SpkNum
                                up3=u1(i,j).^q*(K(j,:)-v(i,:))'*(K(j,:)-v(i,:));
                                up4=u1(i,j)*(K(j,:)-v(i,:))'*(K(j,:)-v(i,:));
                                sfi=sfi+up3;
                                ufi=ufi+up4;
                        end
                        cfi=sfi/sum_q_n(i);
                        fi=ufi/sum_n(i);
                        alfa=-pi:0.05:pi;
                        base=[cos(alfa);sin(alfa)];
                        r=atan(cfi(1,2)/(sqrt(cfi(1,1)*cfi(2,2))));
                        rot=[cos(r),-sin(r);sin(r),cos(r)];
                        p=sqrt(eig(cfi));
                        if  p(1)<p(2)
                                rot=[cos(r),sin(r);-sin(r),cos(r)];
                        end
                        elip=2*rot*diag(sqrt(eig(cfi)))*base;
                        line('XData',elip(1,:)+v(i,1),'YData',elip(2,:)+v(i,2),...
                                'color','r','erasemode','norm');           
                        
                end
                unisplit('defuzzification',u1);
        end;
        %---------------------------------------------------------------------------------
        
elseif(strcmp(action,'defuzzification')),
        global CLUSTER_INDEX SPLIT_CLUSTER_INDEX AXES_DAT;
        
        % soft defuzzification
        u = in1;
        P=DefValues(16);		%defuzzification threshold
        
        [c,SpkNum] = size(u);                   % number of spikes and clusters:
        
        %        CLUSTER_INDEX = zeros(SpkNum + c + 2,1);
        
        CLUSTER_INDEX =[];
        
        % format: [
        %        N= # of clusters, 
        %        SpkNum(1)=# of spikes in 1-st cluster, [indexes of spikes belong to the 1 cluster],...
        %        SpkNum(2)=# of spikes in 2-nd cluster, [indexes of spikes belong to the 2 cluster],...
        %        .....
        %        SpkNum(N)=# of spikes in N-th cluster, [indexes of spikes belong to the N cluster],...
        %        SpkNum(rejected)=# of spikes in rejected cluster, [indexes of spikes which were rejected (P<0.95))]
        %        ];
        
        [max_membership, clust] = max(u);
        rejected_ind = find(max_membership < P);
        if (rejected_ind)
                clust(rejected_ind)=(c+1)*ones(size(rejected_ind)); %last cluster (c+1) includes rejected spikes;
                CLUSTER_INDEX(1)=c+1;
        else
                CLUSTER_INDEX(1)=c;
        end;  
        
        
        for i =1:CLUSTER_INDEX(1),
                
                belong2clust = find(clust==i);
                
                if ~isempty(SPLIT_CLUSTER_INDEX)
                        belong2clust = SPLIT_CLUSTER_INDEX(belong2clust);
                end;   
                CLUSTER_INDEX = [CLUSTER_INDEX, length(belong2clust),belong2clust];         
        end;
        
        
        SPLIT_CLUSTER_INDEX = [];
        axs = unisplit('draw_sort');
        unisplit('draw_button', axs);
        set(gcf,'userdata',axs);
        %---------------------------------------------------------------------------------
elseif(strcmp(action,'draw_sort')),
        
        
        global MERGE CLUSTER_INDEX OLD_CLUSTER_INDEX SHNUM AXES_DAT 
        
        ud = get(0,'Userdata');
        
        FULLPAGE = ud(38);   
        
        
        if FULLPAGE,
                
                
                % Delete "not shapes-related" pictures
                if find(AXES_DAT(1) == get(gcf,'Children')),
                        delete(AXES_DAT(1:2));     
                end
                
                if find(AXES_DAT(3) == get(gcf,'Children')),
                        delete(AXES_DAT(3));    
                end
                % ------------------------------------------
                
                MERGE = [];
                
                % avoid overwrite of SPLIT_DAT, AXES_DAT and C from tmp
                T = SPLIT_DAT;
                TA = AXES_DAT;
                CC = C;
                load tmp;
                SPLIT_DAT = T; 
                AXES_DAT = TA;
                C = CC;
                
                c=CLUSTER_INDEX(1);
                
                if (~isempty(OLD_CLUSTER_INDEX))
                        CLUSTER_INDEX = [c+OLD_CLUSTER_INDEX(1),...
                                        OLD_CLUSTER_INDEX(2:length(OLD_CLUSTER_INDEX)),...
                                        CLUSTER_INDEX(2:length(CLUSTER_INDEX))];
                        OLD_CLUSTER_INDEX = [];
                        axs = get(gcf,'userdata');
                        delete(axs);
                        c=CLUSTER_INDEX(1);
                        
                        
                end;
                
                
                
                offset=1;
                c2 = ceil(c/2);
                ax_sz = min(0.775/c2, 0.3); 
                col_offset = 0;
                n = 1;
                for cl=1:c,
                        if cl>c2,
                                col_offset = 0.505;
                                n = 1; 
                                c2 = 100;
                        end
                        
                        SpikesNum = CLUSTER_INDEX(offset+1);
                        if SpikesNum, 
                                SpkInd = CLUSTER_INDEX(offset+2:offset+1+SpikesNum);
                                offset=offset+1+SpikesNum;
                                SpkArrTime = SPIKES(tnum,SpkInd+3);
                                
                                xx = zeros(1,3*SpikesNum);
                                xx(1:3:(3*SpikesNum-2)) = SpkArrTime;
                                xx(2:3:(3*SpikesNum-1)) = SpkArrTime;
                                xx(3:3:(3*SpikesNum))   = SpkArrTime;
                                yy = zeros(1,3*SpikesNum);
                                yy(2:3:(3*SpikesNum-1)) = ones(1,SpikesNum); 
                                
                                
                                %====
                                % --------------------drawing:
                                % AX_HAND(1) - scatter handler  
                                % rect calculation:
                                
                                ax_sp(cl) = axes('XLim', [0 STIMULUS(tnum,2)], 'YLim',[DefValues(7) DefValues(10)],...
                                        'YTickLabel',[],'userdata',tnum,'Position',[col_offset+.01 .98-n*ax_sz .22 ax_sz-0.005]);
                                
                                if(cl==c+1),
                                        title(['rejected spikes ', num2str(100*SpikesNum/SPIKES(tnum,1)), ' %']);
                                end;
                                
                                
                                
                                % spikes drawing:
                                if SpikesNum,
                                        
                                        SpikeAmp = (DefValues(10) - DefValues(7)) / (3*max(yy));
                                        h = line(xx,SpikeAmp*yy+DefValues(7),'Color','r','LineWidth',.5);
                                end;
                                
                                
                                % PLOTTING THE EYE MOVEMENT
                                
                                HorEyeMov=line(EyeTimeAxis,HorizEyePos,'Color',[0 .502 0]);
                                VerEyeMov=line(EyeTimeAxis,VertEyePos,'Color','m','LineWidth',1);
                                
                                
                                %---------------- STIMULI DRAWING ----------------------------------------------
                                if (OneStimDuration > 100)             % avoid multiple drawing while flash
                                        for i=1:StimNum,
                                                Slope = Front+SpStim;
                                                if Slope > SpStimAll;
                                                        Slope = SpStimAll;
                                                end;
                                                
                                                x = [StimArrTime(i) (OneStimDuration+StimArrTime(i)-1)];
                                                y = [ones(size(x))+DefValues(7)];
                                                line(x,y,'Color',[0 .5 0],'LineWidth',1.5);
                                                
                                                x = [(StimArrTime(i)+OneStimDuration) (StimArrTime(i)+OneStimDuration+TimeBetweenStim-1)];
                                                y = [ones(size(x))+DefValues(7)];
                                                line(x,y,'Color','b','LineWidth',3.5);
                                                
                                                Front = Slope+SpPause;
                                        end;
                                        
                                end;
                                %shape drawing
                                ax_sh(cl) = axes('Position',[col_offset+ .27 .98-n*ax_sz .22 ax_sz-0.005]);
                                n = n +1; 
                                
                                zz=plot(ShapeBuf(SpkInd,:)','k'); hold on
                                zzm = plot(mean(ShapeBuf(SpkInd,:))','Color',[ 0.5020    0.5020    0.7529],'LineWidth',3); hold off
                                if ~DefValues(19),
                                        set(ax_sh(cl),'YLim',[min(min(ShapeBuf')) max(max(ShapeBuf'))],'units','norm');
                                end;
                                set(ax_sh(cl),'UserData',zz);
                                
                                set([zz; zzm],'ButtonDownFcn',['unisplit(''merge_select'',',num2str(cl),');']);
                                set(ax_sh(cl),'XLim',[1 size(ShapeBuf,2)]);
                                str = sprintf('%4d spk, (%4.1f %%)',SpikesNum, 100*SpikesNum/SHNUM);
                                set(text,'units','norm','Position',[.98 .1], 'HorizontalAlignment','right',...
                                        'string',str,'Fontsize',8, 'FontWeight','norm','color','b');
                                
                        end;
                end;
                
                set(SPLIT_DAT(2),'KeyPressFcn','unisplit(''trash'');');
                
        else % Normal graphic mode
                
                MERGE = [];
                % avoid overwrite of SPLIT_DAT and AXES_DAT and C from tmp
                T = SPLIT_DAT;
                TA = AXES_DAT;
                CC = C;
                load tmp;
                SPLIT_DAT = T; 
                AXES_DAT = TA;
                C = CC;
                
                
                c=CLUSTER_INDEX(1);
                
                if (~isempty(OLD_CLUSTER_INDEX))
                        CLUSTER_INDEX = [c+OLD_CLUSTER_INDEX(1),...
                                        OLD_CLUSTER_INDEX(2:length(OLD_CLUSTER_INDEX)),...
                                        CLUSTER_INDEX(2:length(CLUSTER_INDEX))];
                        OLD_CLUSTER_INDEX = [];
                        axs = get(gcf,'userdata');
                        delete(axs);
                        c=CLUSTER_INDEX(1);
                        
                        
                end;
                
               
                offset=1;
                
                ax_sz = min(0.775/c, 0.3); 
                
                for cl=1:c,
                        SpikesNum = CLUSTER_INDEX(offset+1);
                        if SpikesNum, 
                                SpkInd = CLUSTER_INDEX(offset+2:offset+1+SpikesNum);
                                offset=offset+1+SpikesNum;
                                SpkArrTime = SPIKES(tnum,SpkInd+3);
                                
                                xx = zeros(1,3*SpikesNum);
                                xx(1:3:(3*SpikesNum-2)) = SpkArrTime;
                                xx(2:3:(3*SpikesNum-1)) = SpkArrTime;
                                xx(3:3:(3*SpikesNum))   = SpkArrTime;
                                yy = zeros(1,3*SpikesNum);
                                yy(2:3:(3*SpikesNum-1)) = ones(1,SpikesNum); 
                                
                                
                                %====
                                % --------------------drawing:
                                % AX_HAND(1) - scatter handle 
                                % rect calculation:
                                
                                ax_sp(cl) = axes('XLim', [0 STIMULUS(tnum,2)], 'YLim',[DefValues(7) DefValues(10)],...
                                        'YTickLabel',[],'userdata',tnum,'Position',[.5 .98-cl*ax_sz .22 ax_sz-0.005]);
                                
                                if(cl==c+1),
                                        title(['rejected spikes ', num2str(100*SpikesNum/SPIKES(tnum,1)), ' %']);
                                end;
                                
                                
                                
                                % spikes drawing:
                                if SpikesNum,
                                        
                                        SpikeAmp = (DefValues(10) - DefValues(7)) / (3*max(yy));
                                        h = line(xx,SpikeAmp*yy+DefValues(7),'Color','r','LineWidth',.5);
                                end;
                                
                                
                                % PLOTTING THE EYE MOVEMENT
                                
                                HorEyeMov=line(EyeTimeAxis,HorizEyePos,'Color',[0 .502 0]);
                                VerEyeMov=line(EyeTimeAxis,VertEyePos,'Color','m','LineWidth',1);
                                
                                
                                %---------------- STIMULI DRAWING ----------------------------------------------
                                if (OneStimDuration > 100)             % avoid multiple drawing while flash
                                        for i=1:StimNum,
                                                Slope = Front+SpStim;
                                                if Slope > SpStimAll;
                                                        Slope = SpStimAll;
                                                end;
                                                
                                                x = [StimArrTime(i) (OneStimDuration+StimArrTime(i)-1)];
                                                y = [ones(size(x))+DefValues(7)];
                                                line(x,y,'Color',[0 .5 0],'LineWidth',1.5);
                                                
                                                x = [(StimArrTime(i)+OneStimDuration) (StimArrTime(i)+OneStimDuration+TimeBetweenStim-1)];
                                                y = [ones(size(x))+DefValues(7)];
                                                line(x,y,'Color','b','LineWidth',3.5);
                                                
                                                Front = Slope+SpPause;
                                        end;
                                        
                                end;
                                %shape drawing
                                ax_sh(cl) = axes('Position',[.76 .98-cl*ax_sz .22 ax_sz-0.005]);
                                
                                zz=plot(ShapeBuf(SpkInd,:)','k'); hold on;
                                zzm = plot(mean(ShapeBuf(SpkInd,:))','Color',[ 0.5020    0.5020    0.7529],'LineWidth',3); hold off
                                if ~DefValues(19),
                                        set(ax_sh(cl),'YLim',[min(min(ShapeBuf')) max(max(ShapeBuf'))],'units','norm');
                                end;
                                set(ax_sh(cl),'UserData',zz);
                                
                                set([zz; zzm],'ButtonDownFcn',['unisplit(''merge_select'',',num2str(cl),');']);
                                set(ax_sh(cl),'XLim',[1 size(ShapeBuf,2)]);
                                str = sprintf('%4d spk, (%4.1f %%)',SpikesNum, 100*SpikesNum/SHNUM);
                                set(text,'units','norm','Position',[.98 .05], 'HorizontalAlignment','right',...
                                        'string',str,'Fontsize',8, 'FontWeight','norm','color','b');
                                
                                N = size(C,1);
                                if (DefValues(45) & cl <= N),
                                        if 0
                                                if ishandle(HFIG_SHAPE_PER_TRIAL)
                                                        figure(HFIG_SHAPE_PER_TRIAL)
                                                else
                                                        HFIG_SHAPE_PER_TRIAL = figure;
                                                end
                                        end
                                        
                                        hcur = gcf;
                                        figure(3)
                                        
                                        subplot(N,1,cl); hold on;
                                        plot(mean(ShapeBuf(SpkInd,:))','Color',[ 0.5020    0.5020    0.7529],'LineWidth',1);
                                        ylabel(['shape #' num2str(cl)]);
                                        figure(hcur);
                                end
                                
                                
                        end;
                end;
                
                
                
                set(SPLIT_DAT(2),'KeyPressFcn','unisplit(''trash'');');
                
                
        end % of if FullPage
        
        out1 = [ax_sp; ax_sh];
        
        %---------------------------------------------------------------------------------
elseif(strcmp(action,'draw_button')),
        
        global AX_HAND AXES_DAT;
        
        tnum=get(in1(1,1), 'userdata');
        
        uicontrol('string','Re-sort','callback', ['global C; C = []; unisplit(''redraw_exp'',',num2str(tnum), ');'],'Style','Pushbutton',...
                'Position',[.6 .025 .08 .05], 'units','norm');
        
        uicontrol('string','Merge','callback','unisplit(''merge'');','Style','Pushbutton',...
                'Position',[.7 .025 .08 .05], 'units','norm');
        
        uicontrol('string','--','callback','unisplit(''remove'');','Style','Pushbutton',...
                'Position',[.95 .1 .04 .04], 'units','norm');
        
        if DefValues(38),
                
                uicontrol('string','Back','callback','unisplit(''back_page'');','Style','Pushbutton',...
                        'Position',[.82 .1 .06 .04], 'units','norm');
                
        else
                uicontrol('string','Full','callback','unisplit(''full_page'');','Style','Pushbutton',...
                        'Position',[.82 .1 .06 .04], 'units','norm');
        end
        
        SPLIT_DAT(4) = uicontrol('string','Cell 1','callback',['unisplit(''save'',1);'],'Style','Pushbutton',...
                'Position',[.5 .1 .07  .04], 'units','norm');
        SPLIT_DAT(5)=uicontrol('string','Cell 2','callback',['unisplit(''save'',2);'],'Style','Pushbutton',...
                'Position',[.58 .1 .07 .04], 'units','norm');
        SPLIT_DAT(6)=uicontrol('string','Cell 3','callback',['unisplit(''save'',3);'],'Style','Pushbutton',...
                'Position',[.66 .1 .07 .04], 'units','norm');
        SPLIT_DAT(7)=uicontrol('string','Cell 4','callback',['unisplit(''save'',4);'],'Style','Pushbutton',...
                'Position',[.74 .1 .07 .04], 'units','norm');
        SPLIT_DAT(8)=uicontrol('string','undo','Style','Pushbutton',...
                'Position',[.89 .1 .055 .04], 'units','norm','enable','off');
        
        
        set(AX_HAND(2),'visible','off');
        set(AX_HAND(3), 'visible','on')
        
        %------------------------------------------------------------------------
elseif strcmp(action,'full_page');
        % Full graphic mode
        global AXES_DAT
        
        ud = get(0,'Userdata');
        ud(38) = 1;
        set(0,'Userdata',ud);
        
        set(gco, 'String', 'Back', 'Callback', 'unisplit(''back_page'',1);');
        
        axs = get(gcf,'userdata');
        delete(axs);
        
        axs = unisplit('draw_sort'); 
        % unisplit('draw_button', axs);
        set(gcf,'userdata',axs);
        
        
        %------------------------------------------------------------------------
elseif strcmp(action,'back_page');
        % Normal graphic mode
        global AXES_DAT
        
        if in1 == 1, 
                set(gco, 'String', 'Full', 'Callback', 'unisplit(''full_page'');');
        end
        
        ud = get(0,'Userdata');
        ud(38) = 0;
        set(0,'Userdata',ud);
        
        axs = get(gcf,'userdata');
        delete(axs);
        axs = unisplit('draw_sort'); 
        % unisplit('draw_button', axs);
        set(gcf,'userdata',axs);
        
        
        %------------------------------------------------------------------------
elseif strcmp(action,'split');
        
        global MERGE CLUSTER_INDEX AX_HAND OLD_CLUSTER_INDEX SPLIT_CLUSTER_INDEX FP_HNDL AXES_DAT;
        ud = get(0, 'Userdata');
        Merge = MERGE; % Save MERGE DATA, it could be overwritten during "back_page"
        
        if ud(38), % Full Window graphic mode
                % Redraw temporarily in normal mode:
                unisplit('back_page',0);
                ud(38) = 1;
                set(0,'Userdata',ud);
                
        end
        
        % avoid overwrite of SPLIT_DAT and C from tmp
        T = SPLIT_DAT;
        CC = C;
        load tmp;
        SPLIT_DAT = T;
        C = CC;
        
        
        if length(Merge)==1,
                
                ClNum = CLUSTER_INDEX(1);         
                offset=1;
                RestInd = [];
                
                for cl=1:ClNum,
                        SpikesNum = CLUSTER_INDEX(offset+1);
                        SpkInd = CLUSTER_INDEX(offset+2:offset+1+SpikesNum);
                        if ~isempty(find(Merge==cl)),
                                SplitInd=SpkInd;
                        else,
                                RestInd = [RestInd,SpikesNum,SpkInd];
                        end;
                        offset=offset+1+SpikesNum;
                end;
                OLD_CLUSTER_INDEX = [cl-1, RestInd];
                SPLIT_CLUSTER_INDEX = SplitInd;
                
                ShapeData = ShapeBuf(SplitInd,:);
                
                [ShapeBufSize, ShapeLen] = size(ShapeData);
                
                %------------------Writing for klt in C ---------------
                
                
                [fidtmp, message] = fopen('shape.tmp','wb');
                if(fidtmp<0), 
                        error(message);
                        return;
                end;
                fwrite(fidtmp,[ShapeBufSize, ShapeLen],'short');
                fwrite(fidtmp,ShapeData','short');                      % C writes matrix columnwise
                
                fclose(fidtmp);
                eval(['!' APP_PATH 'klt'])
                load basis.dat;
                
                delete shape.tmp;
                K = basis'*ShapeData';
                
                
                
                
                if ( ud(38) | isempty(find(AX_HAND(1) == get(gcf,'Children'))) ),
                        
                        delete(AX_HAND(2:length(AX_HAND)));
                else
                        delete(AX_HAND);
                end
                
                unisplit('cluster',K');
                
        end;
        
        MERGE = [];
        
        
        
        
        
        %---------------------------------------------------------------------------------
elseif(strcmp(action,'merge')),
        global MERGE CLUSTER_INDEX
        
        
        if length(MERGE)>1               % update cluster index
                axs = get(gcf,'userdata');
                delete(axs);
                ClNum = CLUSTER_INDEX(1);         
                offset=1;
                MergeInd = [];
                RestInd = [];
                
                for cl=1:ClNum,
                        SpikesNum = CLUSTER_INDEX(offset+1);
                        SpkInd = CLUSTER_INDEX(offset+2:offset+1+SpikesNum);
                        if ~isempty(find(MERGE==cl)),
                                MergeInd=[MergeInd,SpkInd];
                        else,
                                RestInd = [RestInd,SpikesNum,SpkInd];
                        end;
                        offset=offset+1+SpikesNum;
                end;
                CLUSTER_INDEX=[ClNum-length(MERGE)+1,length(MergeInd), sort(MergeInd), RestInd];                   
                MERGE = [];  
                
                new_axes = unisplit('draw_sort');
                
                set(gcf,'userdata',new_axes);
        end;
        
        
        
        %---------------------------------------------------------------------------------
        %---------------------------------------------------------------------------------
elseif(strcmp(action,'remove')),
        global MERGE CLUSTER_INDEX
        
        
        if length(MERGE)>0               % update cluster index
                axs = get(gcf,'userdata');
                delete(axs);
                ClNum = CLUSTER_INDEX(1);         
                offset=1;
                RemoveInd = [];
                RestInd = [];
                
                for cl=1:ClNum,
                        SpikesNum = CLUSTER_INDEX(offset+1);
                        SpkInd = CLUSTER_INDEX(offset+2:offset+1+SpikesNum);
                        if ~isempty(find(MERGE==cl)),
                                RemoveInd=[RemoveInd,SpkInd];
                        else,
                                RestInd = [RestInd,SpikesNum,SpkInd];
                        end;
                        offset=offset+1+SpikesNum;
                end;
                % CLUSTER_INDEX=[ClNum-length(MERGE)+1,length(RemoveInd), sort(RemoveInd), RestInd];                   
                CLUSTER_INDEX=[ClNum-length(MERGE), RestInd]; 
                MERGE = [];  
                
                new_axes = unisplit('draw_sort');
                
                set(gcf,'userdata',new_axes);
        end;
        
        
        
        %---------------------------------------------------------------------------------
elseif(strcmp(action,'merge_select')),		 
        global MERGE        
        cl_ind = find(MERGE==in1);
        curax = get(gco,'parent');
        
        if ~isempty(cl_ind)
                set(curax,'Color','w');
                MERGE(cl_ind) =[];
        else
                set(curax,'Color',[.9 .9 .9]);     
                MERGE=[MERGE, in1];
        end;
        
        
        %---------------------------------------------------------------------------------
elseif(strcmp(action,'oops')),
        
        fid = fopen(in2,'r+');
        if fid ~= (-1)
                fseek(fid,0,-1);
                if strcmp(in1,'copy')
                        F = fread(fid);
                        set(SPLIT_DAT(8),'UserData',F);
                elseif strcmp(in1,'paste')
                        F = get(SPLIT_DAT(8),'UserData');
                        fwrite(fid,F);
                        set(SPLIT_DAT(8),'Enable','off');   
                end;
                fclose(fid);
                if ~length(F)
                        com = ['delete ',in2];
                        eval(com);
                end;
        end;
        
        %---------------------------------------------------------------------------------
elseif(strcmp(action,'write_sorted_trial')),
        
        global CLUSTER_INDEX MERGE
        
        trial_fname = in1;
        % INIT oops button'
        unisplit('oops','copy',trial_fname);
        set(SPLIT_DAT(8),'callback', ...
                ['global SPLIT_DAT, set(SPLIT_DAT(3+', num2str(in2), '),''enable'',''on'');',...
                        'unisplit(''oops'',''paste'', ''',trial_fname,''');'],'enable','on');
        
        % avoid overwrite of SPLIT_DAT and C from tmp
        T = SPLIT_DAT;
        CC = C;
        
        load tmp;
        SPLIT_DAT = T;
        C = CC;
        
        Separator = 'wwww';
        AppendOK = exist(trial_fname);		%return 2 if file exists
        
        Comments = ['Response of ',num2str(in2), ' cell'];       
        CommLength = length(Comments);
        
        %
        
        if AppendOK~=2,			% file not exists and should be created
                
                [fidtmp, message] = fopen(trial_fname,'w');
                if(fidtmp<0), 
                        error(message);
                        return;
                end;
                
                % ---------- WRITING THE FILE HEADER BLOCK -----------------------
                fwrite(fidtmp,Ver,'ushort');
                FileLengthPos = ftell(fidtmp);
                fwrite(fidtmp,999,'long');
                FileHeaderLengthPos = 20;
                TrialsNum = 1;				% for one trial
                fwrite(fidtmp,[FileHeaderLengthPos,GlobSpecBlockNum,TrialsNum,...
                                CommLength,GlobSpecBlockLength],'ushort');
                
                TrialBeginPos = ftell(fidtmp);
                fwrite(fidtmp,998,'long');
                fwrite(fidtmp, Separator, 'char');
                
                %disp('FILE HEADER BLOCK WRITTEN.... OK');
                
                % ---------- WRITING THE FILE SPECIFICATION  BLOCK -----------------------
                
                fwrite(fidtmp,[FileName,FileDate,RunModName],'char');
                fwrite(fidtmp,[VideoFramePeriod,ViewDistance,TimeFirstSample],'float');
                fwrite(fidtmp,NumSamp2Frame,'ushort');
                fwrite(fidtmp,...
                        [HorizVisualFieldLoc,VertVisualFieldLoc,HorizPosLED,VertPosLED,...
                                HorizEyePosGain,VertEyePosGain,EyetrackArbDef],'float');
                fwrite(fidtmp,...
                        [EyetrackArbV0,Empty,LeftEdge,BottomEdge,RightEdge,TopEdge,StabFlag,...
                                Empty,Empty,CompFlag],'ushort');
                fwrite(fidtmp,DateTime,'char');
                fwrite(fidtmp,[EyePer,SpikePer,ShapePer],'float');
                
                if ~isnan(FixationType),
                        fwrite(fidtmp,FixationType,'ushort');
                end        
                
                fwrite(fidtmp, Separator, 'char');
                
                % ---------- WRITING THE FILE COMMENT  BLOCK -----------------------
                fwrite(fidtmp, Comments, 'char');
                fwrite(fidtmp, Separator, 'char');
                
        else		% file exists and should be read and write
                [fidtmp, message] = fopen(trial_fname,'r+');
                if(fidtmp<0), 
                        error(message);
                        return;
                end;
                
                Ver                     =    fread(fidtmp, 1,'ushort');			%2
                FileLengthPos		=    ftell(fidtmp);
                FileLength              =    fread(fidtmp, 1,'long');			%+4
                FileHeaderLength        =    fread(fidtmp, 1,'ushort');       	 	%+2   
                GlobSpecBlockNum        =    fread(fidtmp, 1,'ushort');			%+2
                
                TrialsNum               =    fread(fidtmp, 1,'ushort');			%+2
                CommLength              =    fread(fidtmp, 1,'ushort');			%+2
                GlobSpecBlockLength     =    fread(fidtmp,GlobSpecBlockNum, 'ushort');	%+2
                
                TrialBegin              =    fread(fidtmp,TrialsNum,'long');		%+4*TrialsNum
                fseek(fidtmp, -(TrialsNum*4),0);			% all the beginnings of the file will be shifted
                fwrite(fidtmp, TrialBegin+4,'long'); 		% right shift for 4 bytes 
                
                TrialBeginPos 		=    ftell(fidtmp);
                fseek(fidtmp,TrialBeginPos,-1); %%% MATLAB BUGSSSSSSSS!!!
                
                RestData		=    fread(fidtmp);    
                
                fseek(fidtmp,10, -1); 
                fwrite(fidtmp,TrialsNum+1,'ushort'); 		% increment of TriaslsNum
                fseek(fidtmp,6, -1); 
                fwrite(fidtmp,FileHeaderLength+4,'ushort'); 	% increment of FileHeaderLength
                
                fseek(fidtmp,TrialBeginPos, -1); 
                fwrite(fidtmp,999,'long');  			% reserve the position for new TrialBegin;
                
                fwrite(fidtmp,RestData); 
                
        end;
        
        % ---------- WRITING THE TRIAL HEADER BLOCK -----------------------
        
        % ----------------- UPDATE TRIAL BEGIN POSITION -----------------------------------   
        CurrentPos = ftell(fidtmp);
        fseek(fidtmp,TrialBeginPos,-1);
        fwrite(fidtmp,CurrentPos,'long');
        fseek(fidtmp,CurrentPos,-1);
        
        %--------- preparation of cluser indexes ----------------------------------------
        ClNum = CLUSTER_INDEX(1);         
        offset=1;
        
        for cl=1:ClNum,
                SpikesNum = CLUSTER_INDEX(offset+1);
                SpkInd = CLUSTER_INDEX(offset+2:offset+1+SpikesNum);
                if MERGE==cl,
                        break;
                end;
                offset=offset+1+SpikesNum;
        end;
        
        %--------------- writing-----------------------------------------------------
        
        fwrite(fidtmp,[SerNum,TrialHeaderLength,ParamBlockNum,DataBlockNum,...
                        ParamBlockLength],'ushort');
        
        DataBlockLengthPos = ftell(fidtmp);
        for j=1:DataBlockNum,
                fwrite(fidtmp,999,'ushort');
        end;  
        fwrite(fidtmp, Separator, 'char');
        
        %--------- WRITING THE TRIAL PARAMETERS ----------------------------------------
        fwrite(fidtmp, TrialTime, 'char');
        fwrite(fidtmp,...
                [TrialDuration,OneStimDuration,TimeBetweenStim,Tilt,RadialBoxSize,...
                        PerpenBoxSize,XStartPos,YStartPos,ExtentStimMotion],'ushort');
        fwrite(fidtmp,[StimVelocity,ColorCode],'short');
        fwrite(fidtmp,[FgrRedInt,FgrGreenInt,FgrBlueInt,BckgrRedInt,...
                        BckgrGreenInt,BckgrBlueInt,ElemRedInt,ElemGreenInt,ElemBlueInt,SpatialFreq],...
                'float');
        fwrite(fidtmp,[PhRed,PhGreen,PhBlue],'ushort');
        fwrite(fidtmp,[StnDiv,Contrast,TempFreq,ElemLength,ElemWidth,...
                        SpLengthDirec,SpWidthDirect,StartEyeData,StartSpikeData,EndSpikeData],...
                'float');
        
        fwrite(fidtmp,[TimeCode,StimTempType,StimSpatType,EyeChoice],'ushort');
        fwrite(fidtmp,SweepFract,'float');
        fwrite(fidtmp,TrigTech,'short');
        fwrite(fidtmp,[SpikeTrig,ShapeTrig,ShapeHist],'float');
        fwrite(fidtmp,[ShapeLen,ShapeTrigValue],'ushort');
        
        if ~isnan(FirstStimTime)
                fwrite(fidtmp,FirstStimTime,'short');
                fwrite(fidtmp,[AngleFix,FixChangeTime,SteadyFixTime,FixNoChangeTime,...
                                TotalFixDistance,FixFraction],'float');
        end
        
        fwrite(fidtmp, Separator, 'char');
        
        
        
        %--------- WRITING THE HORIZONTAL EYE POSITION ----------------------------------------
        
        fwrite(fidtmp,RawHorizEyePos,emformat);
        fwrite(fidtmp, Separator, 'char');
        %----------- WRITING THE VERTICAL EYE POSITION ---------------------------------------
        fwrite(fidtmp,RawVertEyePos,emformat);
        fwrite(fidtmp, Separator, 'char');
        
        % ------------ WRITING THE SPIKES DATA ------------------------------------------------
        fwrite(fidtmp,RawSpikeArrTime(SpkInd),spformat);
        fwrite(fidtmp, Separator, 'char');
        
        % ------------ WRITING THE SPIKE SHAPE TIMING DATA ------------------------------------------------
        fwrite(fidtmp,RawShapeArrTime(SpkInd),shformat);
        fwrite(fidtmp, Separator, 'char');
        
        % ------------ WRITING THE SPIKE SHAPE VALUES DATA ------------------------------------------------
        
        ShapeData = zeros(length(SpkInd)*ShapeLen,1);
        for j=0:length(SpkInd)-1,
                ShapeData(j*ShapeLen+1:(j+1)*ShapeLen) = ShapeAmp((SpkInd(j+1)-1)*ShapeLen+1:SpkInd(j+1)*ShapeLen);
        end;
        fwrite(fidtmp,ShapeData,'short');
        fwrite(fidtmp, Separator, 'char');
        
        
        % ------------ UPDATE DATA BLOCK LENGTH --------------------------------------------------
        CurrentPos = ftell(fidtmp);
        fseek(fidtmp,DataBlockLengthPos,-1);
        fwrite(fidtmp,...
                [emlen*length(RawHorizEyePos), emlen*length(RawVertEyePos),...
                        4*length(SpkInd), 4*length(SpkInd), 2*length(SpkInd)*ShapeLen], 'ushort');
        fseek(fidtmp,CurrentPos,-1);
        %----------------- END WRITING OF TRIAL ---------------------------------------------
        
        
        %%%%%   
        
        % ------------ UPDATE FILE LENGTH POSITION--------------------------------------------------
        
        CurrentPos = ftell(fidtmp);
        fseek(fidtmp,FileLengthPos,-1);
        fwrite(fidtmp,CurrentPos,'long');
        fseek(fidtmp,CurrentPos,-1);
        
        % ------------ CLOSE TRIAL FILE --------------------------------------------------
        fclose(fidtmp);
        
        set(SPLIT_DAT(3+in2),'enable','off');
        
        watchoff;
        
        %---------------------------------
elseif(strcmp(action,'meanshape')),
        ax = get(SPLIT_DAT(2),'userdata');
        if size(ax,1)<2
                warndlg('No cluster selected!','WARNING');
                return;
        end
        ax = ax(2,:);
        
        % tmp = [];
        % for i=1:length(ax),
        %   if (sum(get(ax(i),'color') - [1 1 1]) ~= 0),
        %     tmp = [tmp, ax(i)];
        %   end;
        % end;
        % ax = tmp;
        CurvNum = length(ax);
        
        
        figure; set(gcf,'inverthardcopy','on','pointer','crosshair');
        axes('Position',[.4 0.98 1 1],'Units','normal','visible','off');
        fname = get(SPLIT_DAT(2),'Name');
        shapename = [fname(1:find(fname=='.')),'shp'];
        text(0,0,shapename,'Units','norm','HorizontalAlignment','Center',...
                'FontSize',10,'Color',[0 0 0],'FontName','Times');
        
        orient tall;
        for i = 1:CurvNum,
                ShapeLineHandler = get(ax(i),'userdata');
                ShapeNum = length(ShapeLineHandler);
                Pnt = length(get(ShapeLineHandler(1),'Xdata'));
                YData = zeros(ShapeNum,Pnt); 
                for j = 1:length(ShapeLineHandler)
                        YData(j,:) = get(ShapeLineHandler(j),'Ydata');
                end;
                if size(YData,1)>1,
                        MeanShape = mean(YData);
                else
                        MeanShape = YData;
                end;
                subplot(max(CurvNum,2)*100 + 10 + i);		%doesn't work with subplot(111)
                
                c = line('XData',SPLIT_DAT(9)*[1:Pnt]*1e3,'YData',MeanShape,'Color','b');
                xlabel('mksec','color','k');
                grid;
                set(c,'Buttondownfcn',['unisplit(''shape_timing'',''', shapename, ''');']);
        end;
        
        %-----------------------------------------------------------------------  
        
elseif(strcmp(action,'shape_timing')),
        LineHand = gco;
        LineData = get(LineHand,'YData');
        Time     = get(LineHand,'XData');
        AxHand = get(LineHand,'parent');
        set(gcf,'CurrentAxes',AxHand);
        PresPnt=get(AxHand,'CurrentPoint');
        x=PresPnt(1,1);
        y=PresPnt(1,2);
        ud = get(AxHand,'UserData');
        PntNum = length(ud);
        
        if PntNum <= 3,
                line('XData',x,'YData',y,'color','g','linestyle','o','markersize',6);
                pnts = sort([ud x]);
                set(AxHand,'UserData',pnts); 
                
                if PntNum == 2,
                        t0=pnts(1);
                        t1 = pnts(2)-pnts(1);
                        t2 = pnts(3)-pnts(1);
                        tdif =pnts(3)-pnts(2);
                        
                        [val, indt]=min(abs(Time-pnts(2)));
                        AmpT1 = LineData(indt);
                        
                        [val, indt]=min(abs(Time-pnts(3)));
                        AmpT2 = LineData(indt);
                        
                        
                        Amp = max(LineData)-min(LineData); 
                        AmpS = AmpT2-AmpT1; 
                        
                        tit=sprintf('t1= %4.1f mksec, t2= %4.1f mksec, t2-t1= %4.1f, PeakAmp= %4.1f SlopeAmp= %4.1f',...
                                t1,t2,tdif,Amp, AmpS);
                        line('XData',[pnts(2) pnts(3)],'YData',[AmpT1 AmpT2],'color','r','linestyle','*','markersize',6);
                        title(tit,'color','w','fontsize',8);
                        
                        fid = fopen(in1,'wt');
                        fprintf(fid,'File: 		%s\n',in1);
                        fprintf(fid,'t0 		= %8.1f\n',t0);
                        fprintf(fid,'t1 		= %8.1f\n',t1);
                        fprintf(fid,'t2 		= %8.1f\n',t2);
                        fprintf(fid,'t2-t1 		= %8.1f\n',tdif);
                        fprintf(fid,'Peak Amp.		= %8.1f\n',Amp);
                        fprintf(fid,'Slope Amp.	= %8.1f\n\n',AmpS);
                        
                        
                        
                        fprintf(fid,'%8.1f  %8.1f\n',[Time; LineData]);
                        
                        fclose(fid);
                        
                end;
        end;
        
        %-----------------------------------------------------------------------  
        
elseif(strcmp(action,'kohonen')),
        
        ShapeBuf = in1;
        
        %ShapeBuffer = zeros(ShapeBufSize,ShapeLen);
        % P = reshape(ShapeAmp, ShapeLen, ShapeBufSize);
        ShapeLen = size(ShapeBuf,2);
        ShapeBufSize = size(ShapeBuf,1);
        P = ShapeBuf;
        
        minp = min(min(P));
        maxp = max(max(P));
        PN =   (P - minp)/(maxp-minp);
        
        
        
        figure
        setfsize(300,300);
        
        % ----------------------------------------------- NET PARAMETERS -------------------------------------------------------------
        
        co = 1; % 20; % (1 for not-fuzzy)                         % number of clusters per neuron in first Kohonen.
        S = input('vous is the clusters number ..?');
        S=S*co
        W = initsm(PN',S);
        M = nbman(S,1);
        
        % ----------------------------------------------------------------------------------------------------------------------------
        
        plotsm(W,M);
        df = 50;         	     	 % Frequency of progress displays (in epochs).
        me = 3000;    			 % Maximum number of epochs to train.
        lr = 1;			     	 % Learning rate.
        
        tp = [df me lr];
        
        W = trainsm(W,M,PN',tp);  % training net.
        
        % ----------------------------------------------------------------------------------------------------------------------------
        
        pause
        
        a = simusm(PN',W,M);
        %a = simuc(PN',W);
        b = full(a);              % correcting sparse matrix.
        
        result = zeros(1,ShapeBufSize);
        
        
        for ix=1: ShapeBufSize
                for iy=1: S
                        if b(iy,ix)==1 
                                result(ix)=iy;
                        end
                end
        end
        
        % ----------------------------------------------------------------------------------------------------------------------------
        
        
        qst=input('draw S*co graphs ? ','s');
        if strcmp(qst,'y')
                for i=1 : S
                        class1 = find(result==i);
                        Shape1 = P(class1,:);
                        figure;
                        plot(Shape1');
                end
        end
        
        % ***********************************************************************************************************************
        
        return
        % ----------------------------------------------- NET PARAMETERS -------------------------------------------------------------
        
        PNN = W;    % input for second kohonen.
        
        S = S/co;               	 % number of output neurons.
        W1 = initsm(PNN',S);
        M1 = nbman(S,1);
        
        % ----------------------------------------------------------------------------------------------------------------------------
        
        plotsm(W1,M1);
        df = 50;         	     % Frequency of progress displays (in epochs).
        me = 3000;    			 % Maximum number of epochs to train.
        lr = 1;			     	 % Learning rate.
        
        tp = [df me lr];
        
        
        W1 = trainsm(W1,M1,PNN',tp);  % training net.
        
        % ----------------------------------------------------------------------------------------------------------------------------
        
        pause
        
        aa = simusm(PNN',W1,M1);
        bbb = full(aa);              % correcting sparse matrix.
        
        % result2 = zeros(1,ShapeBufSize);
        
        % combine clusters:
        
        for i=1:S % remove this hara before compiling and stick it in alex's... 
                Shape1 = [];
                tmp = find(bbb(i,:)==1);
                for k=1:length(tmp)
                        %     for l=1:length(b)
                        %       if b(tmp(k),l)==1 
                        %         Shape1=[Shape1 ;P(l,:)];
                        %       end;
                        %     end;
                        l = find(b(tmp(k),:) ==1); 
                        Shape1=[Shape1 ;P(l,:)];
                end;
                
                figure;
                plot(Shape1');     
        end
        
        
        
        
        % ----------------------------------------------------------------------------------------------------------------------------
        
        % ************************************************************************************************************************
        
        %for i=1 : S
        %  class1 = find(result==i);
        %  Shape1 = P(class1,:);
        %  figure;
        %  plot(Shape1');
        %end
        
        % ----------------------------------------------------------------------------------------------------------------------------
        
elseif(strcmp(action,'shift_xcor')),
        template_spk 			= in1;
        [template_spk_num, match_len]	= size(template_spk);
        MaxIter = 100;
        
        shift = 100;
        while(max(abs(shift))>1 & MaxIter),
                % do until no shift is necessary or shift < 1
                MaxIter = MaxIter-1;
                mean_template_spk = mean(template_spk);				% mean template_spk calculation
                for k=1:template_spk_num,
                        u = xcorr(template_spk(k,:), mean_template_spk, 'coeff');
                        [val, pos] = max(u);
                        shift(k) = (match_len-pos);			
                        if shift(k) >0
                                for i=1:shift(k),
                                        for j=2:match_len,		
                                                template_spk(k,j-1) = template_spk(k,j);	% left shift
                                        end;						% end for
                                end;							% end for
                        elseif shift(k) < 0	  
                                for i=1:abs(shift(k)),
                                        for j=match_len:-1:2,
                                                template_spk(k,j) = template_spk(k,j-1);		% right shift
                                        end;						% end for
                                end;							% end for
                        end;							% end if
                end;							% end for
        end;							% end while   
        out1 = template_spk;
        %-----------------------------------------------------------------------  
        
elseif(strcmp(action,'shift')),
        
        template_spk 			= in1;
        out1 = zeros(size(template_spk));
        % save kkk
        [template_spk_num, match_len]	= size(template_spk);
        
        % [tmp, max_pos] = max(abs(template_spk)');
        if DefValues(37)
                [tmp, max_pos] = max(template_spk'); 		%alingment vs positive phase
        else
                [tmp, max_pos] = min(template_spk'); 		%alingment vs negative phase
        end   
        
        MeanMaxPos = round(mean(max_pos));
        shift = MeanMaxPos - max_pos;  
        abs_shift = abs(shift);
        max_allow_shift = round(match_len/4);
        %disp(max(abs_shift)) 
        
        reasonable_shift = [];
        for i = 1 : template_spk_num,
                if (shift(i)>0) & (abs_shift(i) < max_allow_shift )
                        addition = template_spk(i,1) * ones(1,abs_shift(i));
                        out1(i,:) = [addition, template_spk(i,1:match_len-abs_shift(i))];
                        reasonable_shift = [reasonable_shift, shift(i)];
                        
                elseif (shift(i)<0) & (abs_shift(i) < max_allow_shift )
                        addition = template_spk(i,match_len-abs_shift(i)) * ones(1,abs_shift(i));
                        out1(i,:) = [template_spk(i,1+abs_shift(i):match_len) addition];
                        reasonable_shift = [reasonable_shift, shift(i)];
                        
                else
                        out1(i,:) = template_spk(i,:);
                end;
                
        end;
        
        
        if max(reasonable_shift) > 0, 
                left_cutoff = max(reasonable_shift)+1;
        else
                left_cutoff = 1;
        end;
        
        if min(reasonable_shift) < 0, 
                right_cutoff = match_len + min(reasonable_shift);
        else
                right_cutoff = match_len;
        end;
        out1 = out1(:,left_cutoff:right_cutoff);
        %save bbb
        
        %-----------------------------------------------------------------------  
        
elseif(strcmp(action,'hgl')),
        
        
        name=get(SPLIT_DAT(2),'Name');
        lg = max(find(name == '.'));
        sctname = [name(1:lg),'hgl'];
        [filename, pathname] = uiputfile(sctname, 'Save Screen in HGL format As..', .4, .4);
        if filename == 0, return; end;
        sctname = [pathname,filename];
        com = ['print -dhpgl ', sctname];
        eval(com);
        set(gcf,'Pointer','arrow');
        
        
        %-----------------------------------------------------------------------  
        
elseif(strcmp(action,'trash')),
        
        a=get(gcf,'CurrentCharacter');
        if (strcmp (a, 'd') | strcmp (a,'D'))
                
                fprintf('\a')
                
        end;
        %-----------------------------------------------------------------------  
        
elseif(strcmp(action,'save_as_sorted')),
        
        set(SAVE_AS_SORTED_HNDL(in1),'Enable','off');
        name=get(gcf,'Name');
        pnt = find(name=='.');
        save_name = [name(1:pnt),'cl',num2str(in1)];
        
        load tmp;
        
        if 1
                Separator = 'wwww';
                AppendOK = exist(save_name);		%return 2 if file exists
                
                
                Comments = ['Response of ',num2str(in1), ' cell'];       
                CommLength = length(Comments);
                
                %
                
                if AppendOK~=2,		% file not exists and should be created
                        [fidtmp, message] = fopen(save_name,'w');
                        if(fidtmp<0), 
                                error(message);
                                return;
                        end;
                        
                        % ---------- WRITING THE FILE HEADER BLOCK -----------------------
                        fwrite(fidtmp,Ver,'ushort');
                        FileLengthPos = ftell(fidtmp);
                        fwrite(fidtmp,999,'long');
                        FileHeaderLengthPos = 20;
                        TrialsNum = 1;				% for one trial
                        fwrite(fidtmp,[FileHeaderLengthPos,GlobSpecBlockNum,TrialsNum,...
                                        CommLength,GlobSpecBlockLength],'ushort');
                        
                        TrialBeginPos = ftell(fidtmp);
                        fwrite(fidtmp,998,'long');
                        fwrite(fidtmp, Separator, 'char');
                        
                        %disp('FILE HEADER BLOCK WRITTEN.... OK');
                        
                        % ---------- WRITING THE FILE SPECIFICATION  BLOCK -----------------------
                        
                        fwrite(fidtmp,[FileName,FileDate,RunModName],'char');
                        fwrite(fidtmp,[VideoFramePeriod,ViewDistance,TimeFirstSample],'float');
                        fwrite(fidtmp,NumSamp2Frame,'ushort');
                        fwrite(fidtmp,...
                                [HorizVisualFieldLoc,VertVisualFieldLoc,HorizPosLED,VertPosLED,...
                                        HorizEyePosGain,VertEyePosGain,EyetrackArbDef],'float');
                        fwrite(fidtmp,...
                                [EyetrackArbV0,Empty,LeftEdge,BottomEdge,RightEdge,TopEdge,StabFlag,...
                                        Empty,Empty,CompFlag],'ushort');
                        fwrite(fidtmp,DateTime,'char');
                        fwrite(fidtmp,[EyePer,SpikePer,ShapePer],'float');
                        
                        if ~isnan(FixationType),
                                fwrite(fidtmp,FixationType,'ushort');
                        end        
                        
                        fwrite(fidtmp, Separator, 'char');
                        
                        % ---------- WRITING THE FILE COMMENT  BLOCK -----------------------
                        fwrite(fidtmp, Comments, 'char');
                        fwrite(fidtmp, Separator, 'char');
                        
                else		% file exists and should be read and write
                        [fidtmp, message] = fopen(save_name,'r+');
                        if(fidtmp<0), 
                                error(message);
                                return;
                        end;
                        
                        Ver                     =    fread(fidtmp, 1,'ushort');			%2
                        FileLengthPos		=    ftell(fidtmp);
                        FileLength              =    fread(fidtmp, 1,'long');			%+4
                        FileHeaderLength        =    fread(fidtmp, 1,'ushort');       	 	%+2   
                        GlobSpecBlockNum        =    fread(fidtmp, 1,'ushort');			%+2
                        
                        TrialsNum               =    fread(fidtmp, 1,'ushort');			%+2
                        CommLength              =    fread(fidtmp, 1,'ushort');			%+2
                        GlobSpecBlockLength     =    fread(fidtmp,GlobSpecBlockNum, 'ushort');	%+2
                        
                        TrialBegin              =    fread(fidtmp,TrialsNum,'long');		%+4*TrialsNum
                        fseek(fidtmp, -(TrialsNum*4),0);			% all the beginnings of the file will be shifted
                        fwrite(fidtmp, TrialBegin+4,'long'); 		% right shift for 4 bytes 
                        
                        TrialBeginPos 		=    ftell(fidtmp);
                        fseek(fidtmp,TrialBeginPos,-1); %%% MATLAB BUGSSSSSSSS!!!
                        
                        RestData		=    fread(fidtmp);    
                        
                        fseek(fidtmp,10, -1); 
                        fwrite(fidtmp,TrialsNum+1,'ushort'); 		% increment of TriaslsNum
                        fseek(fidtmp,6, -1); 
                        fwrite(fidtmp,FileHeaderLength+4,'ushort'); 	% increment of FileHeaderLength
                        
                        fseek(fidtmp,TrialBeginPos, -1); 
                        fwrite(fidtmp,999,'long');  			% reserve the position for new TrialBegin;
                        
                        fwrite(fidtmp,RestData); 
                        
                end;
                
                % ---------- WRITING THE TRIAL HEADER BLOCK -----------------------
                
                % ----------------- UPDATE TRIAL BEGIN POSITION -----------------------------------   
                CurrentPos = ftell(fidtmp);
                fseek(fidtmp,TrialBeginPos,-1);
                fwrite(fidtmp,CurrentPos,'long');
                fseek(fidtmp,CurrentPos,-1);
                
                
                %--------------- writing-----------------------------------------------------
                
                fwrite(fidtmp,[SerNum,TrialHeaderLength,ParamBlockNum,DataBlockNum,...
                                ParamBlockLength],'ushort');
                
                DataBlockLengthPos = ftell(fidtmp);
                for j=1:DataBlockNum,
                        fwrite(fidtmp,999,'ushort');
                end;  
                fwrite(fidtmp, Separator, 'char');
                
                %--------- WRITING THE TRIAL PARAMETERS ----------------------------------------
                fwrite(fidtmp, TrialTime, 'char');
                fwrite(fidtmp,...
                        [TrialDuration,OneStimDuration,TimeBetweenStim,Tilt,RadialBoxSize,...
                                PerpenBoxSize,XStartPos,YStartPos,ExtentStimMotion],'ushort');
                fwrite(fidtmp,[StimVelocity,ColorCode],'short');
                fwrite(fidtmp,[FgrRedInt,FgrGreenInt,FgrBlueInt,BckgrRedInt,...
                                BckgrGreenInt,BckgrBlueInt,ElemRedInt,ElemGreenInt,ElemBlueInt,SpatialFreq],...
                        'float');
                fwrite(fidtmp,[PhRed,PhGreen,PhBlue],'ushort');
                fwrite(fidtmp,[StnDiv,Contrast,TempFreq,ElemLength,ElemWidth,...
                                SpLengthDirec,SpWidthDirect,StartEyeData,StartSpikeData,EndSpikeData],...
                        'float');
                
                fwrite(fidtmp,[TimeCode,StimTempType,StimSpatType,EyeChoice],'ushort');
                fwrite(fidtmp,SweepFract,'float');
                fwrite(fidtmp,TrigTech,'short');
                fwrite(fidtmp,[SpikeTrig,ShapeTrig,ShapeHist],'float');
                fwrite(fidtmp,[ShapeLen,ShapeTrigValue],'ushort');
                
                if ~isnan(FirstStimTime)
                        fwrite(fidtmp,FirstStimTime,'short');
                        fwrite(fidtmp,[AngleFix,FixChangeTime,SteadyFixTime,FixNoChangeTime,...
                                        TotalFixDistance,FixFraction],'float');
                end
                
                fwrite(fidtmp, Separator, 'char');
                
                
                
                %--------- WRITING THE HORIZONTAL EYE POSITION ----------------------------------------
                
                fwrite(fidtmp,RawHorizEyePos,emformat);
                fwrite(fidtmp, Separator, 'char');
                %----------- WRITING THE VERTICAL EYE POSITION ---------------------------------------
                fwrite(fidtmp,RawVertEyePos,emformat);
                fwrite(fidtmp, Separator, 'char');
                
                % ------------ WRITING THE SPIKES DATA ------------------------------------------------
                fwrite(fidtmp,RawSpikeArrTime,spformat);
                fwrite(fidtmp, Separator, 'char');
                
                % ------------ WRITING THE SPIKE SHAPE TIMING DATA ------------------------------------------------
                fwrite(fidtmp,RawShapeArrTime,shformat);
                fwrite(fidtmp, Separator, 'char');
                
                % ------------ WRITING THE SPIKE SHAPE VALUES DATA ------------------------------------------------
                fwrite(fidtmp,ShapeAmp,'short');
                fwrite(fidtmp, Separator, 'char');
                
                
                % ------------ UPDATE DATA BLOCK LENGTH --------------------------------------------------
                CurrentPos = ftell(fidtmp);
                fseek(fidtmp,DataBlockLengthPos,-1);
                fwrite(fidtmp,...
                        [emlen*length(RawHorizEyePos), emlen*length(RawVertEyePos),...
                                4*length(RawSpikeArrTime), 4*length(RawShapeArrTime), 2*length(ShapeAmp)], 'ushort');
                fseek(fidtmp,CurrentPos,-1);
                %----------------- END WRITING OF TRIAL ---------------------------------------------
                
                
                %%%%%   
                
                % ------------ UPDATE FILE LENGTH POSITION--------------------------------------------------
                
                CurrentPos = ftell(fidtmp);
                fseek(fidtmp,FileLengthPos,-1);
                fwrite(fidtmp,CurrentPos,'long');
                fseek(fidtmp,CurrentPos,-1);
                
                % ------------ CLOSE TRIAL FILE --------------------------------------------------
                fclose(fidtmp);
                
        end
        
        %-----------------------------------------------------------------------  
end;           %elseif(strcmp(action,'.....')),
